
# ****************************************************************************
#               INITIALISATION du CHEMIN d'INSTALLATION de SMURFER
# ****************************************************************************

# Sous Unix/Linux
#.PATH_SMURFER <- "/home/iooss/src"
#.PATH_SMURFER <- "/usbkey"

# Sous Windows
#.PATH_SMURFER <- "~/Mes Documents/Programmes"
#.PATH_SMURFER <- "//Lurs/_Comptes/BI165542/personnel/documents/Mes documents Windows/Programmes"
.PATH_SMURFER <- "K:/SMURFER"

# ******************************************************************
#                           NETTOYAGE
# ******************************************************************

# Si on veut on effacer tous les modeles et les objets en memoire
# (sauf ceux commencant par un ".")
rm(list=ls(all.names=FALSE)) 

# ****************************************************************************
#                               LIBRAIRIES
# ****************************************************************************

library(boot)
library(e1071)
library(evd)
library(gbm)
library(pls)
library(randomForest)
library(nnet)
library(mgcv)
library(triangle)
library(sensitivity)
library(lhs)
library(mlegp)
library(car)
library(JointModeling)
library(DiceKriging)

# ******************************************************************
#                               SOURCES SMURFER
# ******************************************************************

# On charge toutes les fonctions R. 
# Cela remplace pour toutes les fonctions: 
#	source(paste(.PATH_SMURFER,"/SMURFER/current/MSE.R",sep=""))

pathSMURFER <- paste(.PATH_SMURFER,"/SMURFER/current",sep="")
for (nm in list.files(path=pathSMURFER, pattern = "\\.[RrSsQq]$")) {
    cat(nm,":")           
    source(file.path(pathSMURFER, nm))
    cat("\n")}

# exemple pour charger un tableau
#source(paste(.PATH_SMURFER,"/SMURFER/current/ex-tableaux/tableau53.R",sep=""))
# exemple pour charger une fonction
#source(paste(.PATH_SMURFER,"/SMURFER/current/ex-fonctions/fonctions.R",sep=""))

# ****************************************************************************
#                     FICHIERS de DONNEES de VALIDATION v0.5
# ****************************************************************************

# CHAMP GAUSSIEN
champ<-read.table(paste(.PATH_SMURFER,"/SMURFER/VALIDATION/VALIDATION_v0.5/champ.dat",sep=""))
champ<-as.matrix(champ)
xc<-champ[,1:2]
yc<-champ[,3]
xcm1<-c(xc[,1]-mean(xc[,1]))
xcm2<-c(xc[,2]-mean(xc[,2]))
xcm<-matrix(c(xcm1,xcm2),nr=1600,nc=2,dimnames=list(1:1600,c("V1","V2")))
ycm<-c(yc-mean(yc))

# GASCON
gascon<-read.table(paste(.PATH_SMURFER,"/SMURFER/VALIDATION/VALIDATION_v0.5/gascon.dat",sep=""))
gascon<-as.matrix(gascon)
xg<-gascon[,1:48]
yg<-gascon[,49]
xgm<-xg
for(i in 1:48){
xgm[,i]<-xg[,i]-mean(xg[,i])
}
ygm<-c(yg-mean(yg))

# DEPOT
donnees<-read.table(paste(.PATH_SMURFER,"/SMURFER/VALIDATION/VALIDATION_v0.5/depot",sep=""))
donnees<-as.matrix(donnees)
xd<-donnees[,1:3]
yd<-donnees[,4]
xdm1<-c(xd[,1]-mean(xd[,1]))
xdm2<-c(xd[,2]-mean(xd[,2]))
xdm3<-c(xd[,3]-mean(xd[,3]))
xdm<-matrix(c(xdm1,xdm2,xdm3),nr=10,nc=3,dimnames=list(1:10,c("V1","V2","V3")))
ydm<-c(yd-mean(yd))

# Stabi
donnees3<-read.table(paste(.PATH_SMURFER,"/SMURFER/VALIDATION/VALIDATION_v0.5/stabi_module_10",sep=""),dec=",")
donnees3<-as.matrix(donnees3)
xd3<-donnees3[,1:3]
yd3<-donnees3[,4]
xd3m1<-c(xd3[,1]-mean(xd3[,1]))
xd3m2<-c(xd3[,2]-mean(xd3[,2]))
xd3m3<-c(xd3[,3]-mean(xd3[,3]))
xd3m<-matrix(c(xd3m1,xd3m2,xd3m3),nr=17,nc=3,dimnames=list(1:17,c("V1","V2","V3")))
yd3m<-c(yd3-mean(yd3))

# EDE
donnees4<-read.table(paste(.PATH_SMURFER,"/SMURFER/VALIDATION/VALIDATION_v0.5/exemple_EDE_total",sep=""),dec=",")
donnees4<-as.matrix(donnees4)
x1<-donnees4[,1:8]
x2<-factor(donnees4[,9],levels=c(0,3))
xd4<-matrix(c(x1,x2),nr=193,nc=9,dimnames=list(1:193,c("V1","V2","V3","V4","V5","V6","V7","V8","V9")))
yd4<-donnees4[,10]
xd4m1<-c(xd4[,1]-mean(xd4[,1]))
xd4m2<-c(xd4[,2]-mean(xd4[,2]))
xd4m3<-c(xd4[,3]-mean(xd4[,3]))
xd4m4<-c(xd4[,4]-mean(xd4[,4]))
xd4m5<-c(xd4[,5]-mean(xd4[,5]))
xd4m6<-c(xd4[,6]-mean(xd4[,6]))
xd4m7<-c(xd4[,7]-mean(xd4[,7]))
xd4m8<-c(xd4[,8]-mean(xd4[,8]))
xd4m9<-c(xd4[,9]-mean(xd4[,9]))
xd4m<-matrix(c(xd4m1,xd4m2,xd4m3,xd4m4,xd4m5,xd4m6,xd4m7,xd4m8,xd4m9),nr=193,nc=9,
dimnames=list(1:193,c("V1","V2","V3","V4","V5","V6","V7","V8","V9")))
yd4m<-c(yd4-mean(yd4))

donnees4b<-read.table(paste(.PATH_SMURFER,"/SMURFER/VALIDATION/VALIDATION_v0.5/exemple_EDE_partiel",sep=""),dec=",")
donnees4b<-as.matrix(donnees4b)
x1b<-donnees4b[,1:8]
x2b<-factor(donnees4b[,9],levels=c(0,3))
xd4b<-matrix(c(x1b,x2b),nr=194,nc=9,dimnames=list(1:194,c("V1","V2","V3","V4","V5","V6","V7","V8","V9")))
yd4b<-donnees4b[,10]
xd4bm1<-c(xd4b[,1]-mean(xd4b[,1]))
xd4bm2<-c(xd4b[,2]-mean(xd4b[,2]))
xd4bm3<-c(xd4b[,3]-mean(xd4b[,3]))
xd4bm4<-c(xd4b[,4]-mean(xd4b[,4]))
xd4bm5<-c(xd4b[,5]-mean(xd4b[,5]))
xd4bm6<-c(xd4b[,6]-mean(xd4b[,6]))
xd4bm7<-c(xd4b[,7]-mean(xd4b[,7]))
xd4bm8<-c(xd4b[,8]-mean(xd4b[,8]))
xd4bm9<-c(xd4b[,9]-mean(xd4b[,9]))
xd4bm<-matrix(c(xd4bm1,xd4bm2,xd4bm3,xd4bm4,xd4bm5,xd4bm6,xd4bm7,xd4bm8,xd4bm9),nr=194,nc=9,
dimnames=list(1:194,c("V1","V2","V3","V4","V5","V6","V7","V8","V9")))
yd4bm<-c(yd4b-mean(yd4b))

donnees4c<-read.table(paste(.PATH_SMURFER,"/SMURFER/VALIDATION/VALIDATION_v0.5/exemple_EDE_sans",sep=""),dec=",")
donnees4c<-as.matrix(donnees4c)
x1c<-donnees4c[,1:8]
x2c<-factor(donnees4c[,9],levels=c(0,3))
xd4c<-matrix(c(x1c,x2c),nr=192,nc=9,dimnames=list(1:192,c("V1","V2","V3","V4","V5","V6","V7","V8","V9")))
yd4c<-donnees4c[,10]
xd4cm1<-c(xd4c[,1]-mean(xd4c[,1]))
xd4cm2<-c(xd4c[,2]-mean(xd4c[,2]))
xd4cm3<-c(xd4c[,3]-mean(xd4c[,3]))
xd4cm4<-c(xd4c[,4]-mean(xd4c[,4]))
xd4cm5<-c(xd4c[,5]-mean(xd4c[,5]))
xd4cm6<-c(xd4c[,6]-mean(xd4c[,6]))
xd4cm7<-c(xd4c[,7]-mean(xd4c[,7]))
xd4cm8<-c(xd4c[,8]-mean(xd4c[,8]))
xd4cm9<-c(xd4c[,9]-mean(xd4c[,9]))
xd4cm<-matrix(c(xd4cm1,xd4cm2,xd4cm3,xd4cm4,xd4cm5,xd4cm6,xd4cm7,xd4cm8,xd4cm9),nr=192,nc=9,
dimnames=list(1:192,c("V1","V2","V3","V4","V5","V6","V7","V8","V9")))
yd4cm<-c(yd4c-mean(yd4c))

# Dosimetres
donnees5<-read.table(paste(.PATH_SMURFER,"/SMURFER/VALIDATION/VALIDATION_v0.5/donnees_cuivre_calcul3",sep=""),dec=",")
donnees5<-as.matrix(donnees5)
xd5<-donnees5[,1:10]
yd5<-donnees5[,14]
xd5m1<-c(xd5[,1]-mean(xd5[,1]))
xd5m2<-c(xd5[,2]-mean(xd5[,2]))
xd5m3<-c(xd5[,3]-mean(xd5[,3]))
xd5m4<-c(xd5[,4]-mean(xd5[,4]))
xd5m5<-c(xd5[,5]-mean(xd5[,5]))
xd5m6<-c(xd5[,6]-mean(xd5[,6]))
xd5m7<-c(xd5[,7]-mean(xd5[,7]))
xd5m8<-c(xd5[,8]-mean(xd5[,8]))
xd5m9<-c(xd5[,9]-mean(xd5[,9]))
xd5m10<-c(xd5[,10]-mean(xd5[,10]))
xd5m<-matrix(c(xd5m1,xd5m2,xd5m3,xd5m4,xd5m5,xd5m6,xd5m7,xd5m8,xd5m9,xd5m10),nr=1000,nc=10,
dimnames=list(1:1000,c("V1","V2","V3","V4","V5","V6","V7","V8","V9","V10")))
yd5m<-c(yd5-mean(yd5))

donnees6<-read.table(paste(.PATH_SMURFER,"/SMURFER/VALIDATION/VALIDATION_v0.5/donnees_Np7",sep=""),dec=",")
donnees6<-as.matrix(donnees6)
xd6<-donnees6[,1:52]
yd6<-donnees6[,56]
xd6<-matrix(c(xd6),nr=1000,nc=52,dimnames=list(1:1000,c("V1","V2","V3","V4",
"V5","V6","V7","V8","V9","V10","V11","V12","V13","V14","V15","V16","V17","V18",
"V19","V20","V21","V22","V23","V24","V25","V26","V27","V28","V29","V30","V31",
"V32","V33","V34","V35","V36","V37","V38","V39","V40","V41","V42","V43","V44",
"V45","V46","V47","V48","V49","V50","V51","V52")))
xd6m<-xd6
for(i in 1:52){
xd6m[,i]<-xd6[,i]-mean(xd6[,i])
}
xd6m<-matrix(c(xd6m),nr=1000,nc=52,dimnames=list(1:1000,c("V1","V2","V3","V4",
"V5","V6","V7","V8","V9","V10","V11","V12","V13","V14","V15","V16","V17","V18",
"V19","V20","V21","V22","V23","V24","V25","V26","V27","V28","V29","V30","V31",
"V32","V33","V34","V35","V36","V37","V38","V39","V40","V41","V42","V43","V44",
"V45","V46","V47","V48","V49","V50","V51","V52")))
yd6m<-c(yd6-mean(yd6))

donnees7<-read.table(paste(.PATH_SMURFER,"/SMURFER/VALIDATION/VALIDATION_v0.5/donnees_U238",sep=""),dec=",")
donnees7<-as.matrix(donnees7)
xd7<-donnees7[,1:49]
yd7<-donnees7[,53]
xd7<-matrix(c(xd7),nr=1000,nc=49,dimnames=list(1:1000,c("V1","V2","V3","V4",
"V5","V6","V7","V8","V9","V10","V11","V12","V13","V14","V15","V16","V17","V18",
"V19","V20","V21","V22","V23","V24","V25","V26","V27","V28","V29","V30","V31",
"V32","V33","V34","V35","V36","V37","V38","V39","V40","V41","V42","V43","V44",
"V45","V46","V47","V48","V49")))
xd7m<-xd7
for(i in 1:49){
xd7m[,i]<-xd7[,i]-mean(xd7[,i])
}
xd7m<-matrix(c(xd7m),nr=1000,nc=49,dimnames=list(1:1000,c("V1","V2","V3","V4",
"V5","V6","V7","V8","V9","V10","V11","V12","V13","V14","V15","V16","V17","V18",
"V19","V20","V21","V22","V23","V24","V25","V26","V27","V28","V29","V30","V31",
"V32","V33","V34","V35","V36","V37","V38","V39","V40","V41","V42","V43","V44",
"V45","V46","V47","V48","V49")))
yd7m<-c(yd7-mean(yd7))

# ****************************************************************************
#                       EXECUTIONS PROGRAMME VALIDATION v0.5
# ****************************************************************************

# on efface les modeles
#rm(modeleGLM,modelePLS,modeleSVM,modeleBOOSTING,modeleRF,modeleRN,modeleGAM)

smurfer(xc,yc,methodes=c(FALSE,FALSE,TRUE,FALSE,FALSE,FALSE,FALSE,F,F),sobol=TRUE,R=10,S=1000,Sij=TRUE,lois=c(0,2),paramlois=array(c(0,1,0,0),dim=c(4,2)),vc=FALSE,N=3,bootstrap=FALSE,B=5,affiche=FALSE,forme="creationformule",tabA1=tabA1,tabD1=tabD1,degreformule=4,stepwise=TRUE,SVMexplore=FALSE,cmin=51,epsmin=0.01,gmin=5.5,ntree=2000)

# ****************************************************************************
#                     FICHIERS de DONNEES de VALIDATION v1.1
# ****************************************************************************

# BCI
matBCI<-read.table(paste(.PATH_SMURFER,"/SMURFER/VALIDATION/VALIDATION_v1.1/BCI.dat",sep=""),header=TRUE)
xBCI<-matBCI[,2:length(matBCI)]

# MARTHE
param<-read.table(paste(.PATH_SMURFER,"/SMURFER/VALIDATION/VALIDATION_v1.1/elena_marthe.inout",sep=""),header=TRUE)
param<-as.matrix(param)
x<-param[,1:20]
y1<-param[,21]

source(paste(.PATH_SMURFER,"/SMURFER/VALIDATION/VALIDATION_v1.1/sobol_elena.R",sep=""))

# ****************************************************************************
#                       EXECUTIONS PROGRAMME VALIDATION v1.1
# ****************************************************************************

statS(xBCI)

smurfer(x,y1,methodes=c(F,F,F,T,F,F,F,F,F),vc=TRUE,interaction.depth=3,n.trees=10000,affiche=FALSE,sobol=T,Sij=FALSE,S=1000,R=10,lois=c(0,0,0,0,0,0,0,0,0,0,0,0,0,3,3,3,0,0,0,0),paramlois=array(c(1,15,0,0,5,20,0,0,1,15,0,0,1,15,0,0,1,15,0,0,1,15,0,0,1,15,0,0,0.05,2,0,0,0.05,2,0,0,0.05,2,0,0,0.01,0.1,0,0,0.01,0.1,0,0,0.01,0.1,0,0,1.1597,19.9875,0,0,0.891597,24.4455,0,0,1.27363,22.4986,0,0,0.3,0.37,0,0,0,0.0001,0,0,0,1,0,0,0,1,0,0),dim=c(4,20)))

mat<-read.table(paste(.PATH_SMURFER,"/SMURFER/VALIDATION/VALIDATION_v1.1/MOX_DEI.txt",sep=""),header=TRUE,dec=",")
x<-mat[,2:length(mat)]
x<-x+runif(dim(x)[[1]]*dim(x)[[2]],min=-0.0005,max=0.0005)
#statS(x)

mat<-read.table(paste(.PATH_SMURFER,"/SMURFER/VALIDATION/VALIDATION_v1.1/MOX_DEI1.txt",sep=""),header=TRUE,dec=",") # sans les valeurs aberrantes
x<-mat[,2:length(mat)]
x<-x+runif(dim(x)[[1]]*dim(x)[[2]],min=-0.0005,max=0.0005)
statS(x)

mat<-read.table(paste(.PATH_SMURFER,"/SMURFER/VALIDATION/VALIDATION_v1.1/Pentix.txt",sep=""),header=TRUE)
x<-mat[,c(4,8,12,19,23,27,34,38,42,49,53,57,64,68,72,79,83,87,94,98,102,109,113,117,124,128,132)]
x<-x+runif(dim(x)[[1]]*dim(x)[[2]],min=-0.0005,max=0.0005)
statS(x)

mat<-read.table(paste(.PATH_SMURFER,"/SMURFER/VALIDATION/VALIDATION_v1.1/Pentix_global.txt",sep=""),header=TRUE)
x<-mat[,c(4,8,12)]
x<-x+runif(dim(x)[[1]]*dim(x)[[2]],min=-0.0005,max=0.0005)
statS(x)


# ****************************************************************************
#                     FICHIERS de DONNEES de VALIDATION v1.3
# ****************************************************************************

# METEOR
meteor<-read.table(paste(.PATH_SMURFER,"/SMURFER/VALIDATION/VALIDATION_v1.3/meteor_lhs.inout",sep=""),header=TRUE)
meteor<-as.matrix(meteor)
xmet<-meteor[,1:5]
ymet<-meteor[,6:9]

# Boulons Phenix
boul23 <- read.table(paste(.PATH_SMURFER,"/SMURFER/VALIDATION/VALIDATION_v1.3/base1_boulons.dat",sep=""),header=TRUE,dec=",")
boul23<-as.matrix(boul23[,2:5])
boul74 <- read.table(paste(.PATH_SMURFER,"/SMURFER/VALIDATION/VALIDATION_v1.3/basetot_boulons.dat",sep=""),header=TRUE,dec=",")
boul74<-as.matrix(boul74[,2:5])

# ****************************************************************************
#                       EXECUTIONS PROGRAMME VALIDATION v1.3
# ****************************************************************************

moyenne(xmet)
correlationS(ymet)
statS(ymet)
sensibilite(xmet,ymet[,1],nseg=5)
smurfer(xmet,ymet[,1],methodes=c(T,F,F,F,F,F,F,F,F),interaction.depth=2,affiche=T,vc=T,bootstrap=T,stepwise=F)
sobolS(modele=modeleGLM,dim_x=5,methode=1,R=10,S=10000,Sij=T,lois=rep(1,5),paramlois=array(c(2.6,0.05,0,0,4.0965,0.005,0,0,4.745,0.005,0,0,11.3,0.5,0,0,0.005,0.00125,0,0),dim=c(4,5)),LHS=T)

source(paste(.PATH_SMURFER,"/SMURFER/VALIDATION/VALIDATION_v1.3/tableau5.R",sep=""))
smurfer(xmet,ymet[,1],methodes=c(T,F,F,F,F,F,T,F,F),forme="maformule",tabA1=tabA1,tabA2=tabA2,tabA3=tabA3,tabB1=tabB1,tabAgam=tabAgam,affiche=T,vc=T,bootstrap=T,stepwise=F)

statS(boul23[,3])
statS(boul74[,2])

statsconverg(boul23,nboot=100,fboot=5)
statsconverg(boul74,nboot=100,fboot=5)

x<-samplingLHS(dim_x=10,N=1000,lois=c(0,1,2,3,4,5,6,7,8,9),paramlois=array(c(0,1,0,0,0,1,0,0,0,1,0,0,1,1,0,0,1,0,0,0,0.1,0.2,0,0,0,3,4,0,1,2,5,7,0,1,-2,2,0,1,1,6),dim=c(4,10)))
statS(x)

source(paste(.PATH_SMURFER,"/SMURFER/VALIDATION/VALIDATION_v1.3/fonctions.R",sep=""))
sobolS(f=hs,dim_x=3,S=10000,R=10,LHS=TRUE,paramlois=array(c(-pi,pi,0,0,-pi,pi,0,0,-pi,pi,0,0),dim=c(4,3)))

# ******************************************************************
# EXEMPLES d'UTILISATION du PACKAGE SENSITIVITY
# ******************************************************************

smurfer(xmet,ymet[,1],methodes=c(T,F,F,F,F,F,F,F,F),affiche=F,vc=F,bootstrap=F,forme="creationformule",stepwise=F,degreformule=1)
#sa <- srcpcc(model=modeleGLM,x=as.data.frame(xmet),rank=T,nboot=10)
#print(sa)
#par(mfrow = c(2,1))
#plot(sa, ask = FALSE)

#sa <- morris(model=morris.fun,factors=20,levels=10,R=10)
#print(sa)
#par(mfrow = c(1,1))
#plot(sa)

#sa <- fast(model=NULL, factors = 3, n = 1000,q = "qunif", q.arg = list(min = -pi, max = pi))
#y <- ishigami.fun(sa$x)
#tell(sa,y)
#print(sa)
#plot(sa)

#sa <- fast(model = ishigami.fun, factors = 3, n = 1000, q=c("qunif","qunif","qunif"), q.arg = list(list(min = -pi, max = pi), list(min=0,max=1),list(min=-pi,max=pi)))
