affichemodeleGLM<-function(x,y,FOUT,modele,resultatcriteres,forme,degreformule,
               intercept=TRUE,
	       tabA1=rep("no",dim(x)[2]),
               tabA2=rep("no",dim(x)[2]),
               tabA3=rep("no",dim(x)[2]),
	       tabA4=rep("no",dim(x)[2]),
               tabB1=array("no",dim=c(dim(x)[2],dim(x)[2])),
	       tabC1=array("no",dim=c(dim(x)[2],dim(x)[2],dim(x)[2])),
	       tabC2=array("no",dim=c(dim(x)[2],dim(x)[2])),
	       tabD1=array("no",dim=c(dim(x)[2],dim,dim(x)[2],dim(x)[2])),
	       tabD2=array("no",dim=c(dim(x)[2],dim(x)[2])),
	       tabD3=array("no",dim=c(dim(x)[2],dim(x)[2],dim(x)[2])),
	       tabD4=array("no",dim=c(dim(x)[2],dim(x)[2]))
	       ){

#-----------------------------------------------------------------------
#Fonction permettant d'afficher le modele GLM calcule                  |
#----------------------------------------------------------------------|
#Arguments                                                             |
#obligatoires : x : matrice des variables d'entree                     |
#               y : variable de sortie                                 |
#               FOUT : nom du fichier ou inscrire les resultats        |
#               modele : variable contenant le modele GLM calcule      |
#               resultatcriteres : variable contenant les criteres de  |
#               qualite du modele                                      |
#               forme : pour indiquer la facon dont on a cree la       |
#               formule de regression                                  |
#               degreformule : le degre de la formule de regression    |
#               si forme="creationformule"                             |
#               intercept : indique s'il y a une constante dans la     |
#               formule de regression si forme="maformule"             |
#               tabA1 : pour introduire les Vi dans "maformule"        |
#               tabA2 : pour introduire les Vi^2 dans "maformule"      |
#               tabA3 : pour introduire les Vi^3 dans "maformule"      |
#               tabA4 : pour introduire les Vi^4 dans "maformule"      |
#               tabB1 : pour introduire les ViVj dans "maformule"      |
#               tabC1 : pour introduire les ViVjVk dans "maformule"    |
#               tabC2 : pour introduire les Vi^2Vj dans "maformule"    |
#               tabD1 : pour introduire les ViVjVkVl dans "maformule"  |
#               tabD2 : pour introduire les Vi^3 dans "maformule"      |
#               tabD3 : pour introduire les Vi^2VjVk dans "maformule"  |
#               tabD4 : pour introduire les Vi^2Vj^2 dans "maformule"  |
#----------------------------------------------------------------------|
#Sortie : la fonction affiche le modele dans le fichier                |
#res-SURFER.out qui se trouve dans le repertoire de travail            |
#----------------------------------------------------------------------|
#      Auteur : A. Ducellier et B. Iooss                               |
#----------------------------------------------------------------------|

d<-dim(x)[2]

write("-------------------------------------",file=FOUT)
write("| Les coefficients de la regression |",file=FOUT)
write("-------------------------------------",file=FOUT)
write(" ",file=FOUT)

#--------------------------------------
#Affichage si forme="creationformule" |
#--------------------------------------

if(forme=="creationformule"){

coeff<-summary(modele)$coefficients
coeff<-format(coeff,digits=5)
l<-0.25*length(coeff)

is.aliased<-summary(modele)$aliased
l2<-length(is.aliased)

#-----------------------------
#Affichage si degreformule=1 |
#-----------------------------

if(degreformule==1){

write("            Estimation   Ecart-type   t-value   Pr(>|t|)",file=FOUT)
write(" ",file=FOUT)

#Affichage de la constante
if(is.aliased[1]==FALSE){
   write(c("Intercept"," ",coeff[c(1,l+1,2*l+1,3*l+1)]),ncolumns=6,file=FOUT)
   write(" ",file=FOUT)
   coeff<-coeff[c(2:l,(l+2):(2*l),(2*l+2):(3*l),(3*l+2):(4*l))]
   l<-0.25*length(coeff)
   is.aliased<-is.aliased[2:l2]
   l2<-length(is.aliased)
   }
else{
   write(c("Intercept"," ","NA","NA","NA","NA"),ncolumns=6,file=FOUT)
   write(" ",file=FOUT)
   is.aliased<-is.aliased[2:l2]
   l2<-length(is.aliased)
   }

#Affichage des Vi
for(i in 1:d){
   if(is.aliased[1]==FALSE){
      write(c("V",i,"       ",coeff[c(1,l+1,2*l+1,3*l+1)]),ncolumns=7,file=FOUT)
      write(" ",file=FOUT)
      coeff<-coeff[c(2:l,(l+2):(2*l),(2*l+2):(3*l),(3*l+2):(4*l))]
      l<-0.25*length(coeff)
      is.aliased<-is.aliased[2:l2]
      l2<-length(is.aliased)
      }
   else{
      write(c("V",i,"       ","NA","NA","NA","NA"),ncolumns=7,file=FOUT)
      write(" ",file=FOUT)
      is.aliased<-is.aliased[2:l2]
      l2<-length(is.aliased)
      }
   }

}

#-----------------------------
#Affichage si degreformule=2 |
#-----------------------------

if(degreformule==2){

write("            Estimation   Ecart-type   t-value   Pr(>|t|)",file=FOUT)
write(" ",file=FOUT)

#Affichage de la constante
if(is.aliased[1]==FALSE){
   write(c("Intercept"," ",coeff[c(1,l+1,2*l+1,3*l+1)]),ncolumns=6,file=FOUT)
   write(" ",file=FOUT)
   coeff<-coeff[c(2:l,(l+2):(2*l),(2*l+2):(3*l),(3*l+2):(4*l))]
   l<-0.25*length(coeff)
   is.aliased<-is.aliased[2:l2]
   l2<-length(is.aliased)
   }
else{
   write(c("Intercept"," ","NA","NA","NA","NA"),ncolumns=6,file=FOUT)
   write(" ",file=FOUT)
   is.aliased<-is.aliased[2:l2]
   l2<-length(is.aliased)
   }

#Affichage des Vi
for(i in 1:d){
   if(is.aliased[1]==FALSE){
      write(c("V",i,"       ",coeff[c(1,l+1,2*l+1,3*l+1)]),ncolumns=7,file=FOUT)
      write(" ",file=FOUT)
      coeff<-coeff[c(2:l,(l+2):(2*l),(2*l+2):(3*l),(3*l+2):(4*l))]
      l<-0.25*length(coeff)
      is.aliased<-is.aliased[2:l2]
      l2<-length(is.aliased)
      }
   else{
      write(c("V",i,"       ","NA","NA","NA","NA"),ncolumns=7,file=FOUT)
      write(" ",file=FOUT)
      is.aliased<-is.aliased[2:l2]
      l2<-length(is.aliased)
      }
   }

#Affichage des Vi^2
for(i in 1:d){
   if(is.aliased[1]==FALSE){
      write(c("V",i,"^2","    ",coeff[c(1,l+1,2*l+1,3*l+1)]),ncolumns=8,file=FOUT)
      write(" ",file=FOUT)
      coeff<-coeff[c(2:l,(l+2):(2*l),(2*l+2):(3*l),(3*l+2):(4*l))]
      l<-0.25*length(coeff)
      is.aliased<-is.aliased[2:l2]
      l2<-length(is.aliased)
      }
   else{
      write(c("V",i,"^2","    ","NA","NA","NA","NA"),ncolumns=8,file=FOUT)
      write(" ",file=FOUT)
      is.aliased<-is.aliased[2:l2]
      l2<-length(is.aliased)
      }
   }

#Affichage des ViVj
if (d>1){
  for(i in 1:(d-1)){
    for(j in (i+1):d){
      if(is.aliased[1]==FALSE){
         write(c("V",i,"V",j,"   ",coeff[c(1,l+1,2*l+1,3*l+1)]),ncolumns=9,file=FOUT)
         write(" ",file=FOUT)
         coeff<-coeff[c(2:l,(l+2):(2*l),(2*l+2):(3*l),(3*l+2):(4*l))]
         l<-0.25*length(coeff)
	 is.aliased<-is.aliased[2:l2]
	 l2<-length(is.aliased)
	 }
      else{
         write(c("V",i,"V",j,"   ","NA","NA","NA","NA"),ncolumns=9,file=FOUT)
         write(" ",file=FOUT)
	 is.aliased<-is.aliased[2:l2]
	 l2<-length(is.aliased)
	 }
      }
    }
  }
}

#-----------------------------
#Affichage si degreformule=3 |
#-----------------------------

if(degreformule==3){

write("              Estimation   Ecart-type   t-value   Pr(>|t|)",file=FOUT)
write(" ",file=FOUT)

#Affichage de la constante
if(is.aliased[1]==FALSE){
   write(c("Intercept    ",coeff[c(1,l+1,2*l+1,3*l+1)]),ncolumns=5,file=FOUT)
   write(" ",file=FOUT)
   coeff<-coeff[c(2:l,(l+2):(2*l),(2*l+2):(3*l),(3*l+2):(4*l))]
   l<-0.25*length(coeff)
   is.aliased<-is.aliased[2:l2]
   l2<-length(is.aliased)
   }
else{
   write(c("Intercept    ","NA","NA","NA","NA"),ncolumns=5,file=FOUT)
   write(" ",file=FOUT)
   is.aliased<-is.aliased[2:l2]
   l2<-length(is.aliased)
   }

#Affichage des Vi
for(i in 1:d){
   if(is.aliased[1]==FALSE){
      write(c("V",i,"         ",coeff[c(1,l+1,2*l+1,3*l+1)]),ncolumns=7,file=FOUT)
      write(" ",file=FOUT)
      coeff<-coeff[c(2:l,(l+2):(2*l),(2*l+2):(3*l),(3*l+2):(4*l))]
      l<-0.25*length(coeff)
      is.aliased<-is.aliased[2:l2]
      l2<-length(is.aliased)
      }
   else{
      write(c("V",i,"         ","NA","NA","NA","NA"),ncolumns=7,file=FOUT)
      write(" ",file=FOUT)
      is.aliased<-is.aliased[2:l2]
      l2<-length(is.aliased)
      }
   }

#Affichage des Vi^2
for(i in 1:d){
   if(is.aliased[1]==FALSE){
      write(c("V",i,"^2","      ",coeff[c(1,l+1,2*l+1,3*l+1)]),ncolumns=8,file=FOUT)
      write(" ",file=FOUT)
      coeff<-coeff[c(2:l,(l+2):(2*l),(2*l+2):(3*l),(3*l+2):(4*l))]
      l<-0.25*length(coeff)
      is.aliased<-is.aliased[2:l2]
      l2<-length(is.aliased)
      }
   else{
      write(c("V",i,"^2","      ","NA","NA","NA","NA"),ncolumns=8,file=FOUT)
      write(" ",file=FOUT)
      is.aliased<-is.aliased[2:l2]
      l2<-length(is.aliased)
      }
   }

#Affichage des Vi^3
for(i in 1:d){
   if(is.aliased[1]==FALSE){
      write(c("V",i,"^3","      ",coeff[c(1,l+1,2*l+1,3*l+1)]),ncolumns=8,file=FOUT)
      write(" ",file=FOUT)
      coeff<-coeff[c(2:l,(l+2):(2*l),(2*l+2):(3*l),(3*l+2):(4*l))]
      l<-0.25*length(coeff)
      is.aliased<-is.aliased[2:l2]
      l2<-length(is.aliased)
      }
   else{
      write(c("V",i,"^3","      ","NA","NA","NA","NA"),ncolumns=8,file=FOUT)
      write(" ",file=FOUT)
      is.aliased<-is.aliased[2:l2]
      l2<-length(is.aliased)
   }
}

#Affichage des ViVj
if (d>1){
  for(i in 1:(d-1)){
    for(j in (i+1):d){
      if(is.aliased[1]==FALSE){
         write(c("V",i,"V",j,"     ",coeff[c(1,l+1,2*l+1,3*l+1)]),ncolumns=9,file=FOUT)
         write(" ",file=FOUT)
         coeff<-coeff[c(2:l,(l+2):(2*l),(2*l+2):(3*l),(3*l+2):(4*l))]
         l<-0.25*length(coeff)
	 is.aliased<-is.aliased[2:l2]
	 l2<-length(is.aliased)
	 }
      else{
         write(c("V",i,"V",j,"     ","NA","NA","NA","NA"),ncolumns=9,file=FOUT)
         write(" ",file=FOUT)
	 is.aliased<-is.aliased[2:l2]
	 l2<-length(is.aliased)
      }
    }
  }
}
   
#Affichage des Vi^2Vj
if (d>1){
  for(i in 1:d){
    for(j in 1:d){
      if(i!=j){
         if(is.aliased[1]==FALSE){
	    write(c("V",i,"^2","V",j,"  ",coeff[c(1,l+1,2*l+1,3*l+1)]),ncolumns=10,file=FOUT)
	    write(" ",file=FOUT)
            coeff<-coeff[c(2:l,(l+2):(2*l),(2*l+2):(3*l),(3*l+2):(4*l))]
            l<-0.25*length(coeff)
	    is.aliased<-is.aliased[2:l2]
	    l2<-length(is.aliased)
	    }
	 else{
	    write(c("V",i,"^2","V",j,"  ","NA","NA","NA","NA"),ncolumns=10,file=FOUT)
	    write(" ",file=FOUT)
	    is.aliased<-is.aliased[2:l2]
	    l2<-length(is.aliased)
	    }
         }
      }
   }
}

#Affichage des ViVjVk
if (d>2){
  for(i in 1:(d-2)){
    for(j in (i+1):(d-1)){
      for(k in (j+1):d){
         if(is.aliased[1]==FALSE){
	    write(c("V",i,"V",j,"V",k," ",coeff[c(1,l+1,2*l+1,3*l+1)]),ncolumns=11,file=FOUT)
	    write(" ",file=FOUT)
            coeff<-coeff[c(2:l,(l+2):(2*l),(2*l+2):(3*l),(3*l+2):(4*l))]
            l<-0.25*length(coeff)
	    is.aliased<-is.aliased[2:l2]
	    l2<-length(is.aliased)
	    }
	 else{
	    write(c("V",i,"V",j,"V",k," ","NA","NA","NA","NA"),ncolumns=11,file=FOUT)
	    write(" ",file=FOUT)
	    is.aliased<-is.aliased[2:l2]
	    l2<-length(is.aliased)
	    }
         }
      }
   }
  }
}

#-----------------------------
#Affichage si degreformule=4 |
#-----------------------------

if(degreformule==4){

write("                  Estimation   Ecart-type   t-value   Pr(>|t|)",file=FOUT)
write(" ",file=FOUT)

#Affichage de la constante
if(is.aliased[1]==FALSE){
   write(c("Intercept","       ",coeff[c(1,l+1,2*l+1,3*l+1)]),ncolumns=6,file=FOUT)
   write(" ",file=FOUT)
   coeff<-coeff[c(2:l,(l+2):(2*l),(2*l+2):(3*l),(3*l+2):(4*l))]
   l<-0.25*length(coeff)
   is.aliased<-is.aliased[2:l2]
   l2<-length(is.aliased)
   }
else{
   write(c("Intercept","       ","NA","NA","NA","NA"),ncolumns=6,file=FOUT)
   write(" ",file=FOUT)
   is.aliased<-is.aliased[2:l2]
   l2<-length(is.aliased)
   }

#Affichage des Vi
for(i in 1:d){
   if(is.aliased[1]==FALSE){
      write(c("V",i,"             ",coeff[c(1,l+1,2*l+1,3*l+1)]),ncolumns=7,file=FOUT)
      write(" ",file=FOUT)
      coeff<-coeff[c(2:l,(l+2):(2*l),(2*l+2):(3*l),(3*l+2):(4*l))]
      l<-0.25*length(coeff)
      is.aliased<-is.aliased[2:l2]
      l2<-length(is.aliased)
      }
   else{
      write(c("V",i,"             ","NA","NA","NA","NA"),ncolumns=7,file=FOUT)
      write(" ",file=FOUT)
      is.aliased<-is.aliased[2:l2]
      l2<-length(is.aliased)
      }
   }

#Affichage des Vi^2
for(i in 1:d){
   if(is.aliased[1]==FALSE){
      write(c("V",i,"^2","          ",coeff[c(1,l+1,2*l+1,3*l+1)]),ncolumns=8,file=FOUT)
      write(" ",file=FOUT)
      coeff<-coeff[c(2:l,(l+2):(2*l),(2*l+2):(3*l),(3*l+2):(4*l))]
      l<-0.25*length(coeff)
      is.aliased<-is.aliased[2:l2]
      l2<-length(is.aliased)
      }
   else{
      write(c("V",i,"^2","          ","NA","NA","NA","NA"),ncolumns=8,file=FOUT)
      write(" ",file=FOUT)
      is.aliased<-is.aliased[2:l2]
      l2<-length(is.aliased)
      }
   }

#Affichage des Vi^3
for(i in 1:d){
   if(is.aliased[1]==FALSE){
      write(c("V",i,"^3","          ",coeff[c(1,l+1,2*l+1,3*l+1)]),ncolumns=8,file=FOUT)
      write(" ",file=FOUT)
      coeff<-coeff[c(2:l,(l+2):(2*l),(2*l+2):(3*l),(3*l+2):(4*l))]
      l<-0.25*length(coeff)
      is.aliased<-is.aliased[2:l2]
      l2<-length(is.aliased)
      }
   else{
      write(c("V",i,"^3","          ","NA","NA","NA","NA"),ncolumns=8,file=FOUT)
      write(" ",file=FOUT)
      is.aliased<-is.aliased[2:l2]
      l2<-length(is.aliased)
      }
   }

#Affichage des Vi^4
for(i in 1:d){
   if(is.aliased[1]==FALSE){
      write(c("V",i,"^4","          ",coeff[c(1,l+1,2*l+1,3*l+1)]),ncolumns=8,file=FOUT)
      write(" ",file=FOUT)
      coeff<-coeff[c(2:l,(l+2):(2*l),(2*l+2):(3*l),(3*l+2):(4*l))]
      l<-0.25*length(coeff)
      is.aliased<-is.aliased[2:l2]
      l2<-length(is.aliased)
      }
   else{
      write(c("V",i,"^4","          ","NA","NA","NA","NA"),ncolumns=8,file=FOUT)
      write(" ",file=FOUT)
      is.aliased<-is.aliased[2:l2]
      l2<-length(is.aliased)
      }
   }

#Affichage des ViVj
if (d>1){
  for(i in 1:(d-1)){
    for(j in (i+1):d){
      if(is.aliased[1]==FALSE){
         write(c("V",i,"V",j,"         ",coeff[c(1,l+1,2*l+1,3*l+1)]),ncolumns=9,file=FOUT)
         write(" ",file=FOUT)
         coeff<-coeff[c(2:l,(l+2):(2*l),(2*l+2):(3*l),(3*l+2):(4*l))]
         l<-0.25*length(coeff)
	 is.aliased<-is.aliased[2:l2]
	 l2<-length(is.aliased)
	 }
      else{
         write(c("V",i,"V",j,"         ","NA","NA","NA","NA"),ncolumns=9,file=FOUT)
         write(" ",file=FOUT)
	 is.aliased<-is.aliased[2:l2]
	 l2<-length(is.aliased)
	 }
      }
   }
}

#Affichage des Vi^2Vj
if (d>1){
  for(i in 1:d){
    for(j in 1:d){
      if(i!=j){
         if(is.aliased[1]==FALSE){
	    write(c("V",i,"^2","V",j,"      ",coeff[c(1,l+1,2*l+1,3*l+1)]),ncolumns=10,file=FOUT)
            write(" ",file=FOUT)
	    coeff<-coeff[c(2:l,(l+2):(2*l),(2*l+2):(3*l),(3*l+2):(4*l))]
            l<-0.25*length(coeff)
	    is.aliased<-is.aliased[2:l2]
	    l2<-length(is.aliased)
	    }
	 else{
	    write(c("V",i,"^2","V",j,"      ","NA","NA","NA","NA"),ncolumns=10,file=FOUT)
            write(" ",file=FOUT)
	    is.aliased<-is.aliased[2:l2]
	    l2<-length(is.aliased)
	    }
         }
      }
   }
}

#Affichage des Vi^2Vj^2
if (d>1){
  for(i in 1:(d-1)){
    for(j in (i+1):d){
      if(is.aliased[1]==FALSE){
         write(c("V",i,"^2","V",j,"^2","   ",coeff[c(1,l+1,2*l+1,3*l+1)]),ncolumns=11,file=FOUT)
         write(" ",file=FOUT)
         coeff<-coeff[c(2:l,(l+2):(2*l),(2*l+2):(3*l),(3*l+2):(4*l))]
         l<-0.25*length(coeff)
	 is.aliased<-is.aliased[2:l2]
	 l2<-length(is.aliased)
	 }
      else{
         write(c("V",i,"^2","V",j,"^2","   ","NA","NA","NA","NA"),ncolumns=11,file=FOUT)
         write(" ",file=FOUT)
	 is.aliased<-is.aliased[2:l2]
	 l2<-length(is.aliased)
	 }
      }
   }
}

#Affichage des Vi^3Vj
if (d>1){
  for(i in 1:d){
    for(j in 1:d){
      if(i!=j){
         if(is.aliased[1]==FALSE){
	    write(c("V",i,"^3","V",j,"      ",coeff[c(1,l+1,2*l+1,3*l+1)]),ncolumns=10,file=FOUT)
            write(" ",file=FOUT)
	    coeff<-coeff[c(2:l,(l+2):(2*l),(2*l+2):(3*l),(3*l+2):(4*l))]
            l<-0.25*length(coeff)
	    is.aliased<-is.aliased[2:l2]
	    l2<-length(is.aliased)
	    }
	 else{
	    write(c("V",i,"^3","V",j,"      ","NA","NA","NA","NA"),ncolumns=10,file=FOUT)
            write(" ",file=FOUT)
	    is.aliased<-is.aliased[2:l2]
	    l2<-length(is.aliased)
	    }
         }
      }
   }
}

#Affichage des ViVjVk
if (d>2){
  for(i in 1:(d-2)){
    for(j in (i+1):(d-1)){
      for(k in (j+1):d){
         if(is.aliased[1]==FALSE){
	    write(c("V",i,"V",j,"V",k,"     ",coeff[c(1,l+1,2*l+1,3*l+1)]),ncolumns=11,file=FOUT)
            write(" ",file=FOUT)
	    coeff<-coeff[c(2:l,(l+2):(2*l),(2*l+2):(3*l),(3*l+2):(4*l))]
            l<-0.25*length(coeff)
	    is.aliased<-is.aliased[2:l2]
	    l2<-length(is.aliased)
	    }
	 else{
	    write(c("V",i,"V",j,"V",k,"     ","NA","NA","NA","NA"),ncolumns=11,file=FOUT)
            write(" ",file=FOUT)
	    is.aliased<-is.aliased[2:l2]
	    l2<-length(is.aliased)
	    }
         }
      }
   }
}

#Affichage des Vi^2VjVk
if (d>2){
  for(i in 1:d){
    for(j in 1:(d-1)){
      for(k in (j+1):d){
         if((i!=j)&(i!=k)){
	    if(is.aliased[1]==FALSE){
	       write(c("V",i,"^2","V",j,"V",k,"  ",coeff[c(1,l+1,2*l+1,3*l+1)]),ncolumns=12,file=FOUT)
               write(" ",file=FOUT)
	       coeff<-coeff[c(2:l,(l+2):(2*l),(2*l+2):(3*l),(3*l+2):(4*l))]
               l<-0.25*length(coeff)
	       is.aliased<-is.aliased[2:l2]
	       l2<-length(is.aliased)
	       }
	    else{
	       write(c("V",i,"^2","V",j,"V",k,"  ","NA","NA","NA","NA"),ncolumns=12,file=FOUT)
               write(" ",file=FOUT)
	       is.aliased<-is.aliased[2:l2]
	       l2<-length(is.aliased)
	       }
            }
	 }
      }
   }
}

#Affichage des ViVjVkVL
if (d>3){
  for(i in 1:(d-3)){
    for(j in (i+1):(d-2)){
      for(k in (j+1):(d-1)){
         for(L in (k+1):d){
	    if(is.aliased[1]==FALSE){
	       write(c("V",i,"V",j,"V",k,"V",L," ",coeff[c(1,l+1,2*l+1,3*l+1)]),ncolumns=13,file=FOUT)
               write(" ",file=FOUT)
	       coeff<-coeff[c(2:l,(l+2):(2*l),(2*l+2):(3*l),(3*l+2):(4*l))]
               l<-0.25*length(coeff)
	       is.aliased<-is.aliased[2:l2]
	       l2<-length(is.aliased)
	       }
	    else{
	       write(c("V",i,"V",j,"V",k,"V",L," ","NA","NA","NA","NA"),ncolumns=13,file=FOUT)
               write(" ",file=FOUT)
	       is.aliased<-is.aliased[2:l2]
	       l2<-length(is.aliased)
	       }
            }
	 }
      }
   }
}     
}

}

#--------------------------------
#Affichage si forme="maformule" |
#--------------------------------

if(forme=="maformule"){

write("                  Estimation   Ecart-type   t-value   Pr(>|t|)",file=FOUT)
write(" ",file=FOUT)

coeff<-summary(modele)$coefficients
coeff<-format(coeff,digits=5)
l<-0.25*length(coeff)

is.aliased<-summary(modele)$aliased
l2<-length(is.aliased)

#Affichage de la constante
if(intercept==TRUE){
   if(is.aliased[1]==FALSE){
      write(c("Intercept","       ",coeff[c(1,l+1,2*l+1,3*l+1)]),ncolumns=6,file=FOUT)
      write(" ",file=FOUT)
      coeff<-coeff[c(2:l,(l+2):(2*l),(2*l+2):(3*l),(3*l+2):(4*l))]
      l<-0.25*length(coeff)
      is.aliased<-is.aliased[2:l2]
      l2<-length(is.aliased)
      }
   else{
      write(c("Intercept","       ","NA","NA","NA","NA"),ncolumns=6,file=FOUT)
      write(" ",file=FOUT)
      is.aliased<-is.aliased[2:l2]
      l2<-length(is.aliased)
      }
   }

#Boucle pour prendre les Vi
for(i in 1:d){
   if (tabA1[i]=="yes"){
      if(is.aliased[1]==FALSE){
         write(c("V",i,"             ",coeff[c(1,l+1,2*l+1,3*l+1)]),ncolumns=7,file=FOUT)
         write(" ",file=FOUT)
         coeff<-coeff[c(2:l,(l+2):(2*l),(2*l+2):(3*l),(3*l+2):(4*l))]
         l<-0.25*length(coeff)
	 is.aliased<-is.aliased[2:l2]
         l2<-length(is.aliased)
	 }
      else{
         write(c("V",i,"             ","NA","NA","NA","NA"),ncolumns=7,file=FOUT)
         write(" ",file=FOUT)
	 is.aliased<-is.aliased[2:l2]
         l2<-length(is.aliased)
	 }
      }
   }

#Boucle pour prendre les Vi^2
for(i in 1:d){
   if (tabA2[i]=="yes"){
      if(is.aliased[1]==FALSE){
         write(c("V",i,"^2","          ",coeff[c(1,l+1,2*l+1,3*l+1)]),ncolumns=8,file=FOUT)
         write(" ",file=FOUT)
         coeff<-coeff[c(2:l,(l+2):(2*l),(2*l+2):(3*l),(3*l+2):(4*l))]
         l<-0.25*length(coeff)
	 is.aliased<-is.aliased[2:l2]
	 l2<-length(is.aliased)
	 }
      else{
         write(c("V",i,"^2","          ","NA","NA","NA","NA"),ncolumns=8,file=FOUT)
         write(" ",file=FOUT)
	 is.aliased<-is.aliased[2:l2]
         l2<-length(is.aliased)
	 }
      }
   }

#Boucle pour prendre les Vi^3
for(i in 1:d){
   if (tabA3[i]=="yes"){
      if(is.aliased[1]==FALSE){
         write(c("V",i,"^3","          ",coeff[c(1,l+1,2*l+1,3*l+1)]),ncolumns=8,file=FOUT)
         write(" ",file=FOUT)
         coeff<-coeff[c(2:l,(l+2):(2*l),(2*l+2):(3*l),(3*l+2):(4*l))]
         l<-0.25*length(coeff)
	 is.aliased<-is.aliased[2:l2]
	 l2<-length(is.aliased)
	 }
      else{
         write(c("V",i,"^3","          ",coeff[c(1,l+1,2*l+1,3*l+1)]),ncolumns=8,file=FOUT)
         write(" ",file=FOUT)
	 is.aliased<-is.aliased[2:l2]
	 l2<-length(is.aliased)
	 }
      }
   }

#Boucle pour prendre les Vi^4
for(i in 1:d){
   if (tabA4[i]=="yes"){
      if(is.aliased[1]==FALSE){
         write(c("V",i,"^4","          ",coeff[c(1,l+1,2*l+1,3*l+1)]),ncolumns=8,file=FOUT)
         write(" ",file=FOUT)
         coeff<-coeff[c(2:l,(l+2):(2*l),(2*l+2):(3*l),(3*l+2):(4*l))]
         l<-0.25*length(coeff)
	 is.aliased<-is.aliased[2:l2]
	 l2<-length(is.aliased)
	 }
      else{
         write(c("V",i,"^4","          ",coeff[c(1,l+1,2*l+1,3*l+1)]),ncolumns=8,file=FOUT)
         write(" ",file=FOUT)
	 is.aliased<-is.aliased[2:l2]
	 l2<-length(is.aliased)
	 }
      }
   }

#Boucle pour prendre les ViVj
if(d>1){for(i in 1:(d-1)){
              for(j in (i+1):d){
                 if (tabB1[i,j]=="yes"){
                    if(is.aliased[1]==FALSE){
		       write(c("V",i,"V",j,"         ",coeff[c(1,l+1,2*l+1,3*l+1)]),ncolumns=9,file=FOUT)
                       write(" ",file=FOUT)
                       coeff<-coeff[c(2:l,(l+2):(2*l),(2*l+2):(3*l),(3*l+2):(4*l))]
                       l<-0.25*length(coeff)
		       is.aliased<-is.aliased[2:l2]
		       l2<-length(is.aliased)
		       }
		    else{
		       write(c("V",i,"V",j,"         ","NA","NA","NA","NA"),ncolumns=9,file=FOUT)
                       write(" ",file=FOUT)
		       is.aliased<-is.aliased[2:l2]
		       l2<-length(is.aliased)
		       }
                    }
		 }
	      }
}

#Boucle pour prendre les Vi^2Vj
if(d>1){for(i in 1:d){
              for(j in 1:d){
                 if(tabC2[i,j]=="yes"){
                   if (i!=j){
	              if(is.aliased[1]==FALSE){
		         write(c("V",i,"^2","V",j,"      ",coeff[c(1,l+1,2*l+1,3*l+1)]),ncolumns=10,file=FOUT)
                         write(" ",file=FOUT)
                         coeff<-coeff[c(2:l,(l+2):(2*l),(2*l+2):(3*l),(3*l+2):(4*l))]
                         l<-0.25*length(coeff)
			 is.aliased<-is.aliased[2:l2]
			 l2<-length(is.aliased)
			 }
		      else{
		         write(c("V",i,"^2","V",j,"      ","NA","NA","NA","NA"),ncolumns=10,file=FOUT)
                         write(" ",file=FOUT)
			 is.aliased<-is.aliased[2:l2]
			 l2<-length(is.aliased)
			 }
                      }
                   }
                }
	    }
}

#Boucle pour prendre les Vi^3Vj
if(d>1){for(i in 1:d){
              for(j in 1:d){
                 if (tabD2[i,j]=="yes"){
                    if (i!=j){
	               if(is.aliased[1]==FALSE){
		          write(c("V",i,"^3","V",j,"      ",coeff[c(1,l+1,2*l+1,3*l+1)]),ncolumns=10,file=FOUT)
                          write(" ",file=FOUT)
                          coeff<-coeff[c(2:l,(l+2):(2*l),(2*l+2):(3*l),(3*l+2):(4*l))]
                          l<-0.25*length(coeff)
			  is.aliased<-is.aliased[2:l2]
			  l2<-length(is.aliased)
			  }
		       else{
		          write(c("V",i,"^3","V",j,"      ","NA","NA","NA","NA"),ncolumns=10,file=FOUT)
                          write(" ",file=FOUT)
			  is.aliased<-is.aliased[2:l2]
			  l2<-length(is.aliased)
			  }
                       }
                    }
                 }
	      }
}

#Boucle pour prendre les Vi^2Vj^2
if(d>1){for(i in 1:(d-1)){
              for(j in (i+1):d){
                 if (tabD4[i,j]=="yes"){
                    if(is.aliased[1]==FALSE){
		       write(c("V",i,"^2","V",j,"^2","   ",coeff[c(1,l+1,2*l+1,3*l+1)]),ncolumns=11,file=FOUT)
                       write(" ",file=FOUT)
                       coeff<-coeff[c(2:l,(l+2):(2*l),(2*l+2):(3*l),(3*l+2):(4*l))]
                       l<-0.25*length(coeff)
		       is.aliased<-is.aliased[2:l2]
		       l2<-length(is.aliased)
		       }
		    else{
		       write(c("V",i,"^2","V",j,"^2","   ","NA","NA","NA","NA"),ncolumns=11,file=FOUT)
                       write(" ",file=FOUT)
		       is.aliased<-is.aliased[2:l2]
		       l2<-length(is.aliased)
		       }
                    }
                 }
	      }
}

#Boucle pour prendre les ViVjVk
if(d>2){for(i in 1:(d-2)){
              for(j in (i+1):(d-1)){
                 for(k in (j+1):d){
                    if (tabC1[i,j,k]=="yes"){
	               if(is.aliased[1]==FALSE){
		          write(c("V",i,"V",j,"V",k,"     ",coeff[c(1,l+1,2*l+1,3*l+1)]),ncolumns=11,file=FOUT)
                          write(" ",file=FOUT)
                          coeff<-coeff[c(2:l,(l+2):(2*l),(2*l+2):(3*l),(3*l+2):(4*l))]
                          l<-0.25*length(coeff)
			  is.aliased<-is.aliased[2:l2]
			  l2<-length(is.aliased)
			  }
		       else{
		          write(c("V",i,"V",j,"V",k,"     ","NA","NA","NA","NA"),ncolumns=11,file=FOUT)
                          write(" ",file=FOUT)
			  is.aliased<-is.aliased[2:l2]
			  l2<-length(is.aliased)
			  }
                       }
                    }
                 }
              }
}

#Boucle pour prendre les Vi^2VjVk
if(d>2){for(i in 1:d){
              for(j in 1:(d-1)){
                 for(k in (j+1):d){
	            if (tabD3[i,j,k]=="yes"){
	               if ((i!=j)&(i!=k)){
	                  if(is.aliased[1]==FALSE){
			     write(c("V",i,"^2","V",j,"V",k,"  ",coeff[c(1,l+1,2*l+1,3*l+1)]),ncolumns=12,file=FOUT)
                             write(" ",file=FOUT)
                             coeff<-coeff[c(2:l,(l+2):(2*l),(2*l+2):(3*l),(3*l+2):(4*l))]
                             l<-0.25*length(coeff)
			     is.aliased<-is.aliased[2:l2]
			     l2<-length(is.aliased)
			     }
			  else{
			     write(c("V",i,"^2","V",j,"V",k,"  ","NA","NA","NA","NA"),ncolumns=12,file=FOUT)
                             write(" ",file=FOUT)
			     is.aliased<-is.aliased[2:l2]
			     l2<-length(is.aliased)
			     }
                          }
                       }
                    }
                 }
	      }
}

#Boucle pour prendre les ViVjVkVL
if(d>3){for(i in 1:(d-3)){
              for(j in (i+1):(d-2)){
                 for(k in (j+1):(d-1)){
                    for(L in (k+1):d){
	               if (tabD1[i,j,k,L]=="yes"){
		          if(is.aliased[1]==FALSE){
			     write(c("V",i,"V",j,"V",k,"V",L," ",coeff[c(1,l+1,2*l+1,3*l+1)]),ncolumns=13,file=FOUT)
                             write(" ",file=FOUT)
                             coeff<-coeff[c(2:l,(l+2):(2*l),(2*l+2):(3*l),(3*l+2):(4*l))]
                             l<-0.25*length(coeff)
			     is.aliased<-is.aliased[2:l2]
			     l2<-length(is.aliased)
			     }
			  else{
			     write(c("V",i,"V",j,"V",k,"V",L," ",coeff[c(1,l+1,2*l+1,3*l+1)]),ncolumns=13,file=FOUT)
                             write(" ",file=FOUT)
			     is.aliased<-is.aliased[2:l2]
			     l2<-length(is.aliased)
			     }
                          }
	               }
                    }
                 }
              }
}

}

#-----------------------
#Affichage de l'erreur |
#-----------------------

write("--------------------------",file=FOUT)
write("| Estimation de l'erreur |",file=FOUT)
write("--------------------------",file=FOUT)
write(" ",file=FOUT)

write(c("R2                               ",resultatcriteres[1]),ncolumns=2,file=FOUT)
write(" ",file=FOUT)
write(c("R2 ajuste                        ",resultatcriteres[2]),ncolumns=2,file=FOUT)
write(" ",file=FOUT)
write(c("MSE                              ",resultatcriteres[3]),ncolumns=2,file=FOUT)
write(" ",file=FOUT)
write(c("NMSE                             ",resultatcriteres[4]),ncolumns=2,file=FOUT)
write(" ",file=FOUT)
write(c("SMSE                             ",resultatcriteres[5]),ncolumns=2,file=FOUT)
write(" ",file=FOUT)
write(c("MAE                              ",resultatcriteres[6]),ncolumns=2,file=FOUT)
write(" ",file=FOUT)
write(c("Biais                            ",resultatcriteres[7]),ncolumns=2,file=FOUT)
write(" ",file=FOUT)
write(c("Biais normalise                  ",resultatcriteres[8]),ncolumns=2,file=FOUT)
write(" ",file=FOUT)
write(c("Residu maximal                   ",resultatcriteres[9]),ncolumns=2,file=FOUT)
write(" ",file=FOUT)
write(c("Residu relatif en moyenne        ",resultatcriteres[10]),ncolumns=2,file=FOUT)
write(" ",file=FOUT)
write(c("Residu relatif en ecart-type     ",resultatcriteres[11]),ncolumns=2,file=FOUT)
write(" ",file=FOUT)

#----------------------------------
#Fin de l'affichage du modele GLM |
#----------------------------------

write("-----------------------------------------------",file=FOUT)
write("| Somme totale des carres - degre de liberte|",file=FOUT)
write("-----------------------------------------------",file=FOUT)
write(c(modele$null.deviance,"    ",modele$df.null),ncolumns=3,file=FOUT)
write(" ",file=FOUT)
write("----------------------------------------------------",file=FOUT)
write("| Somme des carres des residus - degre de liberte|",file=FOUT)
write("----------------------------------------------------",file=FOUT)
write(c(modele$deviance,"    ",modele$df.residual),ncolumns=3,file=FOUT)
write(" ",file=FOUT)

}

