affichestepwise<-function(modele,FOUT,resultatcriteres,bootstrap=FALSE,
                 e,estimation,restep){

#---------------------------------------------------------------
#Fonction permettant d'afficher les resultats quand on a       |
#utilise l'option stepwise                                     |
#--------------------------------------------------------------|
#Arguments                                                     |
#obligatoires : modele : liste contenant le modeleGLM          |
#               FOUT : nom du fichier dans lequel afficher     |
#               resultats                                      |
#               resultatcriteres : vecteur contenant les       |
#               valeurs des criteres de qualite                |
#               e : nombre d'echantillons bootstrap            |
#               estimation : vecteur contenant les resultats   |
#               du bootstrap                                   |
#               restep : matrice contenant les details des     |
#               etapes de l'algorithme stepwise                |
#                                                              |
#Argument                                                      |
#facultatif : bootstrap : indique si on a effectue le bootstrap|
#--------------------------------------------------------------|
#Sortie : la fonction affiche les resultats dans le fichier    |
#res-SURFER.out qui se trouve dans le repertoire de travail    |
#--------------------------------------------------------------|
#      Auteur : A. Ducellier et B. Iooss                       |
#--------------------------------------------------------------|

#-----------------------
#--------------------- |
#Affichage du modele | |
#--------------------- |
#-----------------------

write("-----------------",file=FOUT)
write("| ------------- |",file=FOUT)
write("| | Le modele | |",file=FOUT)
write("| ------------- |",file=FOUT)
write("-----------------",file=FOUT)
write(" ",file=FOUT)

#-----------------------------------------------
#Affichage des etapes de l'algorithme stepwise |
#-----------------------------------------------

write("---------------------------------------",file=FOUT)
write("| Les etapes de l'algorithme stepwise |",file=FOUT)
write("---------------------------------------",file=FOUT)
write(" ",file=FOUT)

restep<-format(restep)

write("Step         Df           Deviance     AIC",file=FOUT)
write(" ",file=FOUT)

dr<-dim(restep)[[1]]

for(i in 1:dr){
write(c(restep[i,1],restep[i,4],restep[i,5],restep[i,6]),ncolumns=4,file=FOUT)
write(" ",file=FOUT)
}

write("Step = terme elimine",file=FOUT)
write("Df = degres de liberte du modele",file=FOUT)
write("Deviance = somme des carres des residus",file=FOUT)
write("AIC = critere d'information d'Akaike",file=FOUT)
write(" ",file=FOUT)

#----------------------------
#Affichage des coefficients |
#----------------------------

write("-------------------------------------",file=FOUT)
write("| Les coefficients de la regression |",file=FOUT)
write("-------------------------------------",file=FOUT)
write(" ",file=FOUT)

write("          Estimation   Ecart-type   t-value   Pr(>|t|)",file=FOUT)
write(" ",file=FOUT)

coeff<-summary(modele)$coefficients
coeff<-format(coeff,digits=5)

d<-dimnames(coeff)[[1]]
d<-format(d)
l<-dim(coeff)[1]

for(i in 1:l){
write(c(d[i],coeff[i],coeff[l+i],coeff[2*l+i],coeff[3*l+i]),ncolumns=5,file=FOUT)
write(" ",file=FOUT)
}

#-----------------------
#Affichage de l'erreur |
#-----------------------

write("--------------------------",file=FOUT)
write("| Estimation de l'erreur |",file=FOUT)
write("--------------------------",file=FOUT)
write(" ",file=FOUT)

write(c("R2                               ",resultatcriteres[1]),ncolumns=2,file=FOUT)
write(" ",file=FOUT)
write(c("R2 ajuste                        ",resultatcriteres[2]),ncolumns=2,file=FOUT)
write(" ",file=FOUT)
write(c("MSE                              ",resultatcriteres[3]),ncolumns=2,file=FOUT)
write(" ",file=FOUT)
write(c("NMSE                             ",resultatcriteres[4]),ncolumns=2,file=FOUT)
write(" ",file=FOUT)
write(c("SMSE                             ",resultatcriteres[5]),ncolumns=2,file=FOUT)
write(" ",file=FOUT)
write(c("MAE                              ",resultatcriteres[6]),ncolumns=2,file=FOUT)
write(" ",file=FOUT)
write(c("Biais                            ",resultatcriteres[7]),ncolumns=2,file=FOUT)
write(" ",file=FOUT)
write(c("Biais normalise                  ",resultatcriteres[8]),ncolumns=2,file=FOUT)
write(" ",file=FOUT)
write(c("Residu maximal                   ",resultatcriteres[9]),ncolumns=2,file=FOUT)
write(" ",file=FOUT)
write(c("Residu relatif en moyenne        ",resultatcriteres[10]),ncolumns=2,file=FOUT)
write(" ",file=FOUT)
write(c("Residu relatif en ecart-type     ",resultatcriteres[11]),ncolumns=2,file=FOUT)
write(" ",file=FOUT)

#----------------------------------
#Fin de l'affichage du modele GLM |
#----------------------------------

write("-----------------------------------------------",file=FOUT)
write("| Somme totale des carres - degres de liberte |",file=FOUT)
write("-----------------------------------------------",file=FOUT)
write(c(modele$null.deviance,"    ",modele$df.null),ncolumns=3,file=FOUT)
write(" ",file=FOUT)
write("----------------------------------------------------",file=FOUT)
write("| Somme des carres des residus - degres de liberte |",file=FOUT)
write("----------------------------------------------------",file=FOUT)
write(c(modele$deviance,"    ",modele$df.residual),ncolumns=3,file=FOUT)
write(" ",file=FOUT)

#--------------------------
#------------------------ |
#Affichage du bootstrap | |
#------------------------ |
#--------------------------

if(bootstrap==TRUE){

write("-----------------",file=FOUT)
write("| ------------- |",file=FOUT)
write("| | Bootstrap | |",file=FOUT)
write("| ------------- |",file=FOUT)
write("-----------------",file=FOUT)
write(" ",file=FOUT)

if(e==1){write(c("Il reste",estimation[1],"echantillons bootstrap"),ncolumns=3,file=FOUT)
         write("Il faut reduire le nombre de coefficients dans la formule de regression pour pouvoir faire le bootstrap",file=FOUT)
         write(" ",file=FOUT)}

else{

write(c("Il reste",estimation[4*l+41],"echantillons bootstrap"),ncolumns=3,file=FOUT)
write(" ",file=FOUT)

#----------------------------
#Affichage des coefficients |
#----------------------------

write("-------------------------------",file=FOUT)
write("| Estimation des coefficients |",file=FOUT)
write("-------------------------------",file=FOUT)
write(" ",file=FOUT)

write("          Moyenne   Ecart-type   Intervalle de confiance a 90%",file=FOUT)
write(" ",file=FOUT)

for(i in 1:l){
write(c(d[i],estimation[i],estimation[l+i],estimation[2*l+i],estimation[3*l+i]),ncolumns=5,file=FOUT)
write(" ",file=FOUT)
}

#-----------------------------------
#Affichage des criteres de qualite |
#-----------------------------------

i<-4*l+1
ncr<-11 # nombre de criteres

write("--------------------------------------",file=FOUT)
write("| Estimation des criteres de qualite |",file=FOUT)
write("--------------------------------------",file=FOUT)
write(" ",file=FOUT)

write("                              Moyenne   Ecart-type   Intervalle de confiance a 90%",file=FOUT)
write(" ",file=FOUT)

write(c("Q2                              ",estimation[c(i,i+1*ncr,i+2*ncr,i+3*ncr)]),ncolumns=5,file=FOUT)
write(" ",file=FOUT)
write(c("Q2 ajuste                       ",estimation[c(i+1,i+1+1*ncr,i+1+2*ncr,i+1+3*ncr)]),ncolumns=5,file=FOUT)
write(" ",file=FOUT)
write(c("MSE                             ",estimation[c(i+2,i+2+1*ncr,i+2+2*ncr,i+2+3*ncr)]),ncolumns=5,file=FOUT)
write(" ",file=FOUT)
write(c("NMSE                            ",estimation[c(i+3,i+3+1*ncr,i+3+2*ncr,i+3+3*ncr)]),ncolumns=5,file=FOUT)
write(" ",file=FOUT)
write(c("SMSE                            ",estimation[c(i+4,i+4+1*ncr,i+4+2*ncr,i+4+3*ncr)]),ncolumns=5,file=FOUT)
write(" ",file=FOUT)
write(c("MAE                             ",estimation[c(i+5,i+5+1*ncr,i+5+2*ncr,i+5+3*ncr)]),ncolumns=5,file=FOUT)
write(" ",file=FOUT)
write(c("Biais                           ",estimation[c(i+6,i+6+1*ncr,i+6+2*ncr,i+6+3*ncr)]),ncolumns=5,file=FOUT)
write(" ",file=FOUT)
write(c("Biais normalise                 ",estimation[c(i+7,i+7+1*ncr,i+7+2*ncr,i+7+3*ncr)]),ncolumns=5,file=FOUT)
write(" ",file=FOUT)
write(c("Residu maximal                  ",estimation[c(i+8,i+8+1*ncr,i+8+2*ncr,i+8+3*ncr)]),ncolumns=5,file=FOUT)
write(" ",file=FOUT)
write(c("Residu relatif en moyenne       ",estimation[c(i+9,i+9+1*ncr,i+9+2*ncr,i+9+3*ncr)]),ncolumns=5,file=FOUT)
write(" ",file=FOUT)
write(c("Residu relatif en ecart-type    ",estimation[c(i+10,i+10+1*ncr,i+10+2*ncr,i+10+3*ncr)]),ncolumns=5,file=FOUT)
write(" ",file=FOUT)

}

}

}
