bootstrap<-function(x,y,B=50,methode=1,formule,famille,iteration=FALSE,
     niter=1,ncomp=ncomp,c=1,eps=0.1,g=5,n.trees=25000,
     interaction.depth=2,ntree=2000,nunits=1,maxitRN=1000,
     min.nugget=0,simplex.ntries=5,constantPG=0,
     nugget.estim=T,optim.method="BFGS",
     elimination=FALSE,logy=FALSE,affiche=TRUE){

#--------------------------------------------------
#Fonction permettant d'evaluer par bootstrap : la  |
#moyenne, l'ecart-type et l'intervalle de confiance|
#a 90% des criteres de qualite                     |
#Pour le modele GLM, cette fonction donne aussi la |
#moyenne, l'ecart-type et l'int. de conf. a 90% des|
#coeff de regression                               |
#--------------------------------------------------|
#Arguments                                         |
#obligatoires : x : matrice des variables d'entree |
#               y : variable de sortie             |
#                                                  |
#Arguments                                         |
#facultatifs :                                     |
#   B : nombre de repetitions du bootstrap         |
#                si K est ngatif, on intervertit  |
#                les roles des bases de test et    |
#                et d'apprentissage                |
#   methode : numero de la methode                 |
#     1=GLM, 2=PLS, 3=SVM, 4=BOOSTING, 5=RF, 6=RN, |
#     7=GAM, 8=PG, 9=PGdice                        |
#   formule : la formule utilisee                  |
#                       (GLM, GAM, PLS, PGdice)    |
#   famille : pour GLM et GAM, la fonction         |
#     de densite de y est de type "famille". Par   |
#     defaut, on suppose qu'elle est gaussienne    |
#   iteration : indique si on choisit le nombre    | 
#     d'iterations dans la regression. Si iteration|
#     = FALSE, le programme effectue les iterations|
#     jusqu'a ce qu'il y ait convergence           |
#   niter : indique le nombre maximal d'iterations |
#     a effectuer                                  |
#   ncomp : nb de composantes dans modele PLS      |
#   c, eps, g: les parametres des SVM              |
#   n.trees,interaction.depth: parametres boosting |
#     (nombre d'arbres et ordre des interactions)  |
#   ntree : nb d'arbres de la foret aleatoire      |
#   maxitRN : nombre max. d'iterations pour le RN  |
#   nunits : nb de noeuds dans la couche du RN     |
#   min.nugget : valeur min de la pepite pour PG   |
#   simplex.ntries : nb d'optim par simplex pour PG|
#   constantPG : 0 (defaut) = partie reg lineair PG|
#                1 : partie deter constante pour PG|
#   nugget.estim : T (defaut) =pepite estime PGdice|
#   optim.method : "BFGS" (defaut) : optim PGdice  |
#                  "gen" : algo genetique rgenoud  |
#   logy : indique si la variable de sortie est    |
#      passee en logarithme                        |
#   affiche : si TRUE, affiche le graphe du        |
#      leave-one-out                               |
#                                                  |
#Remarque : si on veut une estimation precise des  |
#intervalles de confiance, il faut prendre B = 1000|   
#---------------------------------------------------------------|
#Sortie : la moyenne, l'ecart-type et l'intervalle de confiance |
#         a 90% des coefficients de la regression GLM           |
#                                                               |
#         la moyenne, l'ecart-type et l'intervalle de confiance |
#         a 90% des criteres de qualite R2, R2 ajuste, MSE,NMSE,|
#         SMSE,SSD, biais, biais normalise residu maximal,residu|
#         relatif en moyenne, residu relatif en ecart-type)     |
#                                                               |
#         la fonction trace egalement les valeurs predites en   |
#         fonction des valeurs observees pour les echantillons  |
#         de test et d'apprentissage                            |
#---------------------------------------------------------------|
#      Auteur : B. Iooss et A. Ducellier                        |
#---------------------------------------------------------------|  

#------------------------------------------------
#Initialisations necessaires avant de commencer |
#------------------------------------------------
  ncriteres<-11
  s=0
  l=0

#Calcul du nombre de coefficients dans la formule de regression
  if (methode==1){
    if(iteration==FALSE) modele<-glm(formule,famille,data=data.frame(x,y))
    else modele<-glm(formule,famille,data=data.frame(x,y),maxit=niter)
#  if (methode==2) modele<-glm(formule,data=data.frame(x,y))

    l<-length(modele$coefficients)
    s<-modele$df.null-modele$df.residual+1
#Initialisation de la matrice des coefficients
    M<-matrix(0,nr=l,nc=B)}
  else l<-dim(x)[2]

#Initialisation de la matrice des criteres de qualite
  resultat<-matrix(0,nr=ncriteres,nc=B)

#Initialisation des vecteurs yreel et ypred
#Ces vecteurs contiendront les valeurs observees et predites des
#echantillons de test
  yreel<-matrix(Inf,nr=length(y),nc=B)
  ypred<-matrix(Inf,nr=length(y),nc=B)

#Initialisation des vecteurs yareel et yappr
#Ces vecteurs contiendront les valeurs observees et predites des 
#echantillons d'apprentissage
  yareel<-matrix(Inf,nr=length(y),nc=B)
  yappr<-matrix(Inf,nr=length(y),nc=B) 

#-------------------
#Boucle principale |
#-------------------
  for(i in 1:B){

#Sauvegarde des variables d'entree et de sortie
    xb<-x
    yb<-y

#Initialisation du vecteur indice
    indice<-rep(0,dim(x)[1])

#Selection de l'echantillon
    for(j in 1:dim(x)[1]){
      alea<-(dim(x)[1])*(runif(1))
      indice[j]<-round(0.5+alea)}

#Creation de l'echantillon d'apprentissage
    xba<-x[indice,]
    yba<-y[indice]
    xba<-matrix(xba,nr=dim(x)[1],dimnames=list(1:dim(x)[1],dimnames(x)[[2]]))
    yba<-matrix(yba,nr=dim(x)[1],dimnames=list(1:dim(x)[1],dimnames(y)[[2]]))

#Mise a l'infini des observations deja utilisees dans l'echantillon
#d'apprentissage pour ne pas les reutiliser
    xb[indice,]<-Inf
    yb[indice]<-Inf

#Creation de l'echantillon de test
    xbt<-matrix(c(xb[xb[,1]!=Inf]),nc=dim(x)[2])
    ybt<-yb[yb!=Inf]

    d<-dim(xbt)[1]

    xbt<-matrix(xbt,nr=d,dimnames=list(1:d,dimnames(x)[[2]]))
    ybt<-matrix(ybt,nr=d,dimnames=list(1:d,dimnames(y)[[2]]))

# -----------------------------
# Modele GLM
    if (methode==1){

#S'il reste trop peu de valeurs dans l'echantillon d'apprentissage
#le logiciel ne peut pas calculer tous les coefficients de la regression
#On va donc eliminer les echantillons de test pour lesquels la
#difference entre le degre de liberte total et le degre de liberte des residus
#plus 1 est strictement inferieure au nombre de coefficients demandes
#dans la formule de regression
      if (methode==1){
        if(iteration==FALSE) modele<-glm(formule,famille,data=data.frame(xba,y=yba))
        else modele<-glm(formule,famille,data=data.frame(xba,y=yba),maxit=niter)}
      if (methode==2) modele<-plsr(formule,data=data.frame(xba,y=yba),ncomp=ncomp,method="kernelpls")

      if(s<l){resultat[,i]=rep(Inf,ncriteres)
              M[,i]=rep(Inf,l)
              yreel<-yreel
              ypred<-ypred
              yareel<-yareel
              yappr<-yappr}
	
      else{
#Calcul des valeurs predites
        pred<-predictSR(modele,xbt,methode)
        appr<-predictSR(modele,xba,methode)
			
#Calcul de la matrice des coefficients
        for(j in 1:l) M[j,i]=modele$coefficients[j]}
      ldim<-l
    }
    else ldim<-0 # dim des coefficients du GLM quand le modele n'est pas GLM
  
# -----------------------------
#Methode PLS
    if (methode==2){
      modele<-plsr(formule,data=data.frame(xba,y=yba),ncomp=ncomp,method="kernelpls")
      l <- length(modele$coefficients)-1
    }
#Methode SVM
    if (methode==3){
      if (is.data.frame(xba)==TRUE){ modele <- svm(yba~.,data=xba,kernel="radial",cost=c,epsilon=eps,gamma=g)}
      else{ modele <- svm(xba,yba,kernel="radial",cost=c,epsilon=eps,gamma=g)}}
#Methode BOOSTING
    if (methode==4) modele <- gbm.fit(xba,yba,distribution="gaussian",n.trees=n.trees,interaction.depth=interaction.depth,verbose=FALSE)
#Methode FORETS ALEATOIRES
    if (methode==5) modele <- randomForest(xba,yba,ntree=ntree)
#Methode RESEAU de NEURONES
    if (methode==6) modele <- nnet(xba,yba,linout=TRUE,size=nunits,maxit=maxitRN,MaxNWts=nunits*nunits*dim(x)[2])
#Methode GAM
    if (methode==7){
      modele <- gam(formule,famille,data=data.frame(xba,y=yba))
      l <- dim(as.data.frame(modele$coefficients))[1]
    }
#Methode PROCESSUS GAUSSIEN
    if (methode==8) modele<-mlegp(xba,yba,constantMean=constantPG,min.nugget=min.nugget,simplex.ntries=simplex.ntries)
    
#Methode PGdice
    ys <- y
    y <- yba # car km marche qu'avec y
    if (methode==9) modele<-km(formule,as.data.frame(xba),as.data.frame(y),covtype="powexp",nugget.estim=nugget.estim,optim.method=optim.method)
    y <- ys
  
# -----------------------------

    pred<-predictSR(modele,xbt,methode)
    appr<-predictSR(modele,xba,methode)

# -----------------------------
    if (methode>1 | s>=l){
#Calcul des criteres de qualite
      pred <- as.vector(pred)
      ybt <- as.vector(ybt)
      if(logy==TRUE){vecteurcriteres<-criteres(exp(ybt),exp(pred),dim_x=l)}
      else{vecteurcriteres<-criteres(ybt,pred,dim_x=l)}
      for(j in 1:ncriteres){resultat[j,i]=vecteurcriteres[j]}

#Calcul de yreel et ypred
      for(j in 1:length(ybt)){yreel[j,i]<-ybt[j]
                              ypred[j,i]<-pred[j]}
			
#Calcul de yareel et ypred
      for(j in 1:length(yba)){yareel[j,i]<-yba[j]
                              yappr[j,i]<-appr[j]}
    }}

#------------------
#Fin de la boucle |
#------------------

#--------------------------------------
#Elimination de certains echantillons |
#--------------------------------------

  if (methode==1){N<-M}
  R<-resultat

#S'il reste trop peu d'observations differentes dans l'echantillon
#d'apprentissage, il se peut que certains coefficients n'aient pas 
#ete calcules. Ils sont alors marques NA (Non Available). On va
#donc eliminer les echantillons correspondant en mettant toutes
#les valeurs a l'infini  

  if (methode==1){
    for(i in 1:B){
      index<-is.na(M[,i])
      for(j in 1:l){
        if (index[j]==TRUE){
          N[,i]<-rep(Inf,ldim)
          R[,i]<-rep(Inf,ncriteres)
          yreel[,i]<-rep(Inf,length(y))
          ypred[,i]<-rep(Inf,length(y))
          yareel[,i]<-rep(Inf,length(y))
          yappr[,i]<-rep(Inf,length(y))}    
      }}}

#Si l'echantillon de test est trop petit, il se peut qu'on rencontre
#des difficultes pour calculer les criteres de qualite On va donc
#eliminer les echantillons pour lesquels un des criteres de qualite
#n'est pas disponible ou est infini. On elimine ces echantillons en 
#mettant toutes les valeurs a l'infini

  for(i in 1:B){
    index<-is.na(resultat[,i])
    for(j in 1:ncriteres){
      if(index[j]==TRUE){
        R[,i]<-rep(Inf,ncriteres)
        if (methode==1){ N[,i]<-rep(Inf,ldim)}
        yreel[,i]<-rep(Inf,length(y))
        ypred[,i]<-rep(Inf,length(y))
        yareel[,i]<-rep(Inf,length(y))
        yappr[,i]<-rep(Inf,length(y))
      }}}

  for(i in 1:B){
    for(j in 1:ncriteres){
      if(R[j,i]==Inf){
        R[,i]<-rep(Inf,ncriteres)
        if (methode==1){N[,i]<-rep(Inf,ldim)}
        yreel[,i]<-rep(Inf,length(y))
        ypred[,i]<-rep(Inf,length(y))
        yareel[,i]<-rep(Inf,length(y))
        yappr[,i]<-rep(Inf,length(y))
      }}}
	      
#--------------------------------------
#Elimination des valeurs trop grandes |
#--------------------------------------

#Il peut arriver que certains echantillons ne soient pas du tout 
#representatifs des donnees et donnent des valeurs completement
#fausses. On va eliminer les echantillons pour lesquels le R2 
#est negatif  

  if(elimination==TRUE){

    for(i in 1:B){
      if(R[1,i]<0){
        R[,i]<-rep(Inf,ncriteres)
        if (methode==1){N[,i]<-rep(Inf,ldim)}
        yreel[,i]<-rep(Inf,length(y))
        ypred[,i]<-rep(Inf,length(y))
        yareel[,i]<-rep(Inf,length(y))
        yappr[,i]<-rep(Inf,length(y))
      }}}

#On garde alors seulement les echantillons pour lesquels
#les coefficients et les criteres ont des valeurs finies
  
  R<-matrix(c(R[R!=Inf]),nr=ncriteres)    	      
  if (methode==1){N<-matrix(c(N[N!=Inf]),nr=l)}
  yreel<-c(yreel[yreel!=Inf])
  ypred<-c(ypred[ypred!=Inf])
  yareel<-c(yareel[yareel!=Inf])
  yappr<-c(yappr[yappr!=Inf])

#On calcule le nombre d'echantillons bootstrap restant apres elimination
  B2<-dim(R)[2]

#S'il ne reste plus d'echantillons, la fonction renvoie 0
  if(B2==0){return(c(B2))}
  else{

#S'il ne reste qu'un echantillon, on ne peut pas calculer
#l'ecart-type. La fonction renvoie 1
    if(B2==1) return(c(B2))

    else{

#---------------------------------------------------------
#Calcul des estimations des coefficients et des criteres |
#---------------------------------------------------------

#Initialisations
      m<-rep(0,ldim)
      S<-rep(0,ldim)
      alpha1<-rep(0,ldim)
      alpha2<-rep(0,ldim)
      mc=rep(0,ncriteres)
      Sc=rep(0,ncriteres)
      alpha1c<-rep(0,ncriteres)
      alpha2c<-rep(0,ncriteres)

#-----------------------------------
#Estimations pour les coefficients |
#-----------------------------------
      if (methode==1){
        for(i in 1:l){

#Calcul du vecteur des moyennes des coefficients
          m[i]=mean(N[i,])

#Calcul du vecteur des ecarts-type des coefficients
          S[i]=sqrt((1/(dim(N)[2]-1))*sum((N[i,]-m[i])^2))

#Calcul des intervalles de confiance des coefficients
          N.2<-N[i,]
          N.2<-sort(N.2)
          a1=round(0.05*length(N.2))
          a2=round(0.95*length(N.2))
          if(a1==0){a1<-1}
          alpha1[i]=N.2[a1]
          alpha2[i]=N.2[a2]
        }}

#-------------------------------
#Estimations pour les criteres |
#-------------------------------

      for(i in 1:ncriteres){

#Calcul du vecteur des moyennes des criteres
        mc[i]=mean(R[i,])

#Calcul du vecteur des ecarts-type des criteres
        Sc[i]=sqrt((1/(dim(R)[2]-1))*sum((R[i,]-mc[i])^2))

#Calcul des intervalles de confiance des criteres
        R.2<-R[i,]
        R.2<-sort(R.2)
        a1=round(0.05*length(R.2))
        a2=round(0.95*length(R.2))
        if(a1==0){a1<-1}
        alpha1c[i]=R.2[a1]
        alpha2c[i]=R.2[a2]
      }

#--------------------------------------------------------------
#Trace des valeurs predites en fonction des valeurs observees |
#--------------------------------------------------------------

      if(affiche==TRUE){

#Calcul de l'intervalle dans lequel on trace la courbe pour le test
        mt=min(c(yreel,ypred))
        Mt=max(c(yreel,ypred))

#Calcul de l'intervalle dans lequel on trace la courbe pour l'apprentissage
        ma=min(c(yareel,yappr))
        Ma=max(c(yareel,yappr))

        if(logy==TRUE){

#Trace de la courbe pour le test
          par(mfg=c(3,2))
          plot(yreel,yreel,col=2,type="l",main=("bootstrap"),sub=("prediction"),xlab="log(observation)",ylab="log(prediction)",ylim=c(mt,Mt))
          points(yreel,ypred)
          
#Trace de la courbe pour l'apprentissage
          par(mfg=c(3,1))
          plot(yareel,yareel,col=2,type="l",main=("bootstrap"),sub=("apprentissage"),xlab="log(observation)",ylab="log(prediction)",ylim=c(ma,Ma))
          points(yareel,yappr)}

        else{

#Trace de la courbe pour le test
          par(mfg=c(3,2))
          plot(yreel,yreel,col=2,type="l",main=("bootstrap"),sub=("prediction"),xlab="observation",ylab="prediction",ylim=c(mt,Mt))
          points(yreel,ypred)

#Trace de la courbe pour l'apprentissage
          par(mfg=c(3,1))
          plot(yareel,yareel,col=2,type="l",main=("bootstrap"),sub=("apprentissage"),xlab="observation",ylab="prediction",ylim=c(ma,Ma))
          points(yareel,yappr)
        }}

#Renvoi
      return(c(m,S,alpha1,alpha2,mc,Sc,alpha1c,alpha2c,B2))
    }
  }
}
