bornemaxIC<-function(x,nom=c(NA),kmax=1,nech=100,nboot=100,conf=0.95){

#-----------------------------------------------------------------------
#Calcul des bornes min et max de l'intervalle de confiance bootstrap   |
#sur les moyenne et ecart-type si on rajoute k nouveaux points         |
#(k varie de 1 a kmax)                                                 |
#----------------------------------------------------------------------|
#Arguments :                                                           |
#                                                                      |
#obligatoire : x : matrice des variables d'entree (format colonnes)    |
#                                                                      |
#facultatif : nom    : vecteur des noms de variable (7 caracteres max) |
#                     (par defaut : header du tableau x)               |
#             kmax   : nombre maximal de points que l'on ajoute        |
#             nech   : nombre de nouveaux echantillons par echantillon |
#             nboot  : nombre de repliques bootstrap                   |
#             conf   : niveau de confiance pour l'intervalle bootstrap |
#----------------------------------------------------------------------|
#Sortie : Graphes d'evolution des bornes de l'intervalle de confiance  |
#----------------------------------------------------------------------|
#Utilisation :                                                         |
#  -Au prealable, il faut allouer x sous R                             |
#    Exemple : mat<-read.table(file="MOX_DEI.txt",header=TRUE,dec=",") |
#            x<-mat[,2:length(mat)]                                    |
#            x<-x+runif(dim(x)[[1]]*dim(x)[[2]],min=-0.0005,max=0.0005)|
#            x<-read.table(file="nom_fichier")                         |
#                                                                      |
#  - Il faut le meme nombre de donnees dans chaque colonne             |
#----------------------------------------------------------------------|
#      Auteur : B. Iooss                                               |
#----------------------------------------------------------------------|  
  
# *********************************************************************
# FONCTION UTILISEE :
#      bootstats.R (intervalle de confiance bootstrap)
  
# *********************************************************************
# LIBRAIRIE REQUISE
  library(boot)
# *********************************************************************
  
# INITIALISATIONS

# Transformation de x en matrice
  if (is.matrix(x)==FALSE & is.vector(x)==TRUE) x<-matrix(x,ncol=1)
  
# Cacul des dimensions de x
  n<-dim(x)[1]
  np<-dim(x)[2]
  xx <- matrix(0,nr=n+kmax,nc=np)
  
# Verification des arguments
  if (nech <= 1) {
    print("nech doit etre > 1, on prendra nech=100")
    nech <- 100}
  if (nboot <= 1) {
    print("nboot doit etre > 1, on prendra nboot=100")
    nboot <- 100}
  if (kmax <= 1) {
    print("kmax doit etre > 0, on prendra kmax=1")
    kmax <- 1}
  
  moy <- matrix(0,nr=np,nc=5)
  moymean <- matrix(0,nr=kmax+1,nc=np)
  moybias <- matrix(0,nr=kmax+1,nc=np)  
  minmoycimin <- matrix(0,nr=kmax+1,nc=np)
  maxmoycimax <- matrix(0,nr=kmax+1,nc=np)
  ect <- matrix(0,nr=np,nc=5)
  ectmean <- matrix(0,nr=kmax+1,nc=np)
  ectbias <- matrix(0,nr=kmax+1,nc=np)
  minectcimin <- matrix(0,nr=kmax+1,nc=np)
  maxectcimax <- matrix(0,nr=kmax+1,nc=np)
  
# *********************************************************************
# BOUCLE sur la TAILLE d'ECHANTILLON
  i <- 1
  index <- NULL
  while (i <= kmax+1){ # On decale i de 1 pour qu'il varie de 0 a kmax
      
    moymean1 <- 0;moybias1 <- 0;moycimin1 <- 0;moycimax1 <- 0
    ectmoycimin1 <- 0;ectmoycimax1 <- 0
    ectmean1 <- 0;ectbias1 <- 0;ectcimin1 <- 0;ectcimax1 <- 0
    ectectcimin1 <- 0;ectectcimax1 <- 0
      
# BOUCLE sur les REPLIQUES
    for (iech in 1:nech){

# AJOUT DE POINTS
# Reechantillonnage dans la base initiale
      for (j in 1:np){
        for (ii in 1:n) xx[ii,j] <- x[ii,j]
        for (ii in 1:kmax) xx[n+ii,j] <- sample(x[,j],size=1)}

# Bootstrap sur la moyenne
      b <- boot(xx[1:n+i-1,],function(s,j){mean(as.data.frame(s)[j,])},R=nboot)
      moy <- bootstats(b,conf,"basic")[,1:5]
      moymean1 <- moymean1+moy[,1] # Moyenne echantillon original
      moybias1 <- moybias1+moy[,2] + moy[,1] # estimateur bootstrap
      moycimin1 <- moycimin1+moy[,4] # Intervalle de confiance
      moycimax1 <- moycimax1+moy[,5]
      ectmoycimin1 <- ectmoycimin1+moy[,4]^2
      ectmoycimax1 <- ectmoycimax1+moy[,5]^2

# Bootstrap sur l'ecart-type
      b <- boot(xx[1:n+i-1,],function(s,j){sd(as.data.frame(s)[j,])},R=nboot)
      ect <- bootstats(b,conf,"basic")[,1:5]
      ectmean1 <- ectmean1+ect[,1] # ecart-type echantillon original
      ectbias1 <- ectbias1+ect[,2] + ect[,1] # estimateur bootstrap
      ectcimin1 <- ectcimin1+ect[,4] # Intervalle de confiance
      ectcimax1 <- ectcimax1+ect[,5]
      ectectcimin1 <- ectectcimin1+ect[,4]^2
      ectectcimax1 <- ectectcimax1+ect[,5]^2
    }
    
    moymean[i,] <- moymean1/nech
    moybias[i,] <- moybias1/nech
    minmoycimin[i,] <- moycimin1/nech-2*(ectmoycimin1/nech-(moycimin1/nech)^2)
    maxmoycimax[i,] <- moycimax1/nech+2*(ectmoycimax1/nech-(moycimax1/nech)^2)
    
    ectmean[i,] <- ectmean1/nech
    ectbias[i,] <- ectbias1/nech
    minectcimin[i,] <- ectcimin1/nech-2*(ectectcimin1/nech-(ectcimin1/nech)^2)
    maxectcimax[i,] <- ectcimax1/nech+2*(ectectcimax1/nech-(ectcimax1/nech)^2)
    
    index <- c(index,i-1)
    i <- i+1
  }

# *********************************************************************
# GRAPHIQUES

  for (j in 1:np){
    x11()
    op <- par(mfcol=c(2,1))
    par(mfg=c(1,1))
    plot(index,moymean[index+1,j],pch=20,xlab="",ylab="Moyenne",ylim=c(min(minmoycimin[index+1,j]),max(maxmoycimax[index+1,j])),main="")
    lines(index,moybias[index+1,j])
    lines(index,minmoycimin[index+1,j])
    lines(index,maxmoycimax[index+1,j])
    title(c("Bornes a 2 e-t sur l'intervalle de confiance",nom[j]),sub="point : estimateur echantillon\n ligne : estimateur bootstrap + IC a 95%")
      par(mfg=c(2,1))
      plot(index,ectmean[index+1,j],pch=20,xlab="Nombre de points ajoutes",ylab="Ecart-type",ylim=c(min(minectcimin[index+1,j]),max(maxectcimax[index+1,j])))
      lines(index,ectbias[index+1,j])
      lines(index,minectcimin[index+1,j])
      lines(index,maxectcimax[index+1,j])
  }
}
