cobweb<-function(xy,scale="percentile",MeV="basic",Afourni){
#---------------------------------------------------------------------------|
#Fonction d'affichage du cobweb plot aussi appel parallel coordinates	    |
#---------------------------------------------------------------------------|
#Argument                                                                   |
#obligatoire : xy : matrice de plus de deux variables d'entre, classe en  |
#                    colonnes et une seule sortie                           |
#                                                                           |
#facultatifs : scale   : chaine de caractres, qui peut prendre les valeurs |
#                        "natural" ou "percentile"                          |
#              MeV     : chane de caractres qui peut prendre les valeurs  |
#                        "basic","modif", "perso"                           |
#              Afourni : empilement de listes d'instructions pour la mise   |
#                        en valeur.                                         |
#---------------------------------------------------------------------------|
#Sortie : trace le diagramme demand                                	    |
#---------------------------------------------------------------------------|
#      Auteur : P. Lemaitre                                                 |
#---------------------------------------------------------------------------|
  
####################################################################
## file contents #########
#	cobweb function
#	vachercher function
#	CheckA function
#	enq function
#	coloriage function
#	Nouveau function
#	CreerColStyl function
##############################################################


# cobweb I ################ Travail prliminaire #########################################################################
 
		naxes=dim(xy)[2]

# cobweb II ############ Vrification des entres ########################################################################

	if (scale!="natural" && scale!="percentile" )
	{stop("Verifiez votre valeur d'chelle, elle n'est pas comprise")}
	
	if (MeV!="basic" && MeV!="modif" && MeV!="perso" )
	{stop("Verifiez votre valeur de Mise en Valeur, elle n'est pas comprise")}
	
	if (MeV=="perso" )
	{ if (   CheckA(Afourni,naxes)==FALSE  ) 
		{stop("Verifiez votre valeur de A, elle n'est pas comprise")}
	}

# cobweb III ############## Travail sur les donnes #######################################################################
# cobweb III #on cre une nouvelle matrice, qui scale les variables ( et la rponse ) 

	if (scale=="natural")
	{xybis=matrix(NA,ncol=dim(xy)[2],nrow=dim(xy)[1])
	for (i in 1:dim(xy)[2]){xybis[,i]=(xy[,i]-min(xy[,i]))/(max(xy[,i]-min(xy[,i]))) }
	}
	
	if (scale=="percentile")
	{xybis=enq(xy)}



# cobweb IV ############### Association : couleur et zone d'arrive #######################################################

	if (MeV=="basic"){
		A=rbind(list("q",naxes,0,1,"c","black"),list("q",naxes,0,0.1,"c","red")) }
	else if (MeV=="modif"){
		A=coloriage(naxes) }
	else if (MeV=="perso") {
		A=Afourni     }

	CS=CreerColStyl(A,xy)
	
# cobweb V ################ Creation de la fenetre #######################################################################

	# cobweb V.1 ## on cre le vecteur qui contient les intituls des axes
	LEG=dimnames(xy)[[2]]

	# cobweb V.2 ## on cre la fenetre et les axes, ainsi que les lignes verticales
	plot(c(1:naxes),c(0,rep(1,naxes-1)), type="n",axes=FALSE,xlab="",ylab="")
	
		# cobweb V.2.a # axe horizontal
		axis(1,at=c(-3,naxes+3))
		axis(1,at=1:naxes,label=LEG,line=2)

		# cobweb V.2.b # axe vertical 
 		if (scale=="percentile") 
			{axis(2,seq(0,1,1/10))}
		if (scale=="natural")
		{ 	VDo=min(xy[,1])
			VDu=max(xy[,1])
			for (i in 2:naxes){
				VDo=c(VDo,min(xy[,i]))
				VDu=c(VDu,max(xy[,i])) 
			}
			VDo=format(VDo,digits=5)
			VDu=format(VDu,digits=5)
	     		axis(1,at=1:naxes,label=VDo,cex.axis=0.7,tick=FALSE,line=-1.3)
			axis(3,at=1:naxes,label=VDu,cex.axis=0.7,tick=FALSE,line=-1.3)
 		}

 		# cobweb V.2.c # trac des lignes verticales reprsentant les variables
		for (i in 1:naxes)
		{segments(i,0,i,1)}

# cobweb  VI ############### Traage des segments #######################################################################
	for (k in 1:dim(xy)[1]){ # donc k c'est l'individu
	 for (j in 1:(naxes-1)){# donc j c'est la variable
		if (CS[[1]][k]!=0)
		{segments(j,xybis[k,j],j+1,xybis[k,j+1],col=CS[[1]][k],lty=as.numeric(CS[[2]][k]),lwd=0.1)}
	 }#end j
	}# end k

}# end cobweb

##########################################################################################################################
##########################################################################################################################
## fonction vachercher #########

vachercher<-function(file,head)
{

######################################################################
# cette fonction aide l'utilisateur pour l'importation d'un fichier  #
# de donnes									   #
######################################################################

# vachercher I ############### Importer les donnes et les afficher ######################################################

	# vachercher I.1 ## on importe les donnes et on demande  l'user le nombre d'entes
		T=read.table(file,header=head)
		cat(" Dans le fichier, on a ", dim(T)[2]," variables et ",dim(T)[1]," individus \n")
		cat("Donnez le nombre d'entres : ")
		vae<-as.numeric(readline()) 
	
	# vachercher I.2 ## on affiche les noms des variables pour plus de claret
		for (i in (1:(vae))){
			cat ("la variable ",i," se nomme :",dimnames(T)[[2]][i],"\n")
		}
		for (i in (1:(dim(T)[2]-vae))){
			cat ("la sortie ",i," se nomme :",dimnames(T)[[2]][i+vae],"\n")
		}


# vachercher II ############### Partie travail sur les entres ##########################################################

	# vachercher II.1 ##  Slection

		# vachercher II.1.a # Est il ncessaire d'entrer dans la boucle ?
		cat("On va calculer les plages d'entres\n")
		cat("Est ce qu'on prend toutes les entres ?  \n")
		repen<-readline("(o,n) :")
			while(repen!="o" && repen!="n"){
				repen<-readline("rponse non comprise, recommencer svp(o,n) : ")
			}		
		ListeEnlever=0	

		# vachercher II.1.b # La boucle qui permet d'enlever des entres
		if (repen=="n")
		{	ContinuerEnlever=TRUE
		
			while(ContinuerEnlever){
				t=as.numeric(readline(" Quelle variable doit on soustraire ? " ))
				while ( !is.element(t,1:vae) ){
					t=as.numeric(readline(" La variable que vous avez demand n'est pas une entre. Recommencez \n " ))
				}
				cat("on va enlever l'entre :",dimnames(T)[[2]][t]," aka numro ",t,"\n")
				ListeEnlever=c(ListeEnlever,t)
				repenlev=readline("Est-ce tout ? (o,n)")
				while(repenlev!="o" && repenlev!="n"){
					repenlev<-readline("rponse non comprise, recommencer svp(o,n) : ")
				}

				if (repenlev=="o")
				{	ListeEnlever=ListeEnlever[2:length(ListeEnlever)]
					ContinuerEnlever=FALSE
					nbren=vae-length(ListeEnlever)}
				}
			}

		# vachercher II.1.c # Si l'on enlve pas d'entres
		else 
		{cat("On prends donc toutes les entres \n")
		ListeEnlever="onenleverien"
		}

	# vachercher II.2 ##  Permutation
	
		# vachercher II.2.a # Est il ncessaire d'entrer dans la boucle ?
		cat("Doit on permuter des entres ?  \n")
		permut<-readline("(o,n) :")
			while(permut!="o" && permut!="n"){
				permut<-readline("rponse non comprise, recommencer svp(o,n) : ")
			}

		# vachercher II.2.b # La boucle qui permet de permuter des entres
		if (permut=="o")
		{	ContinuerPermut=TRUE
			ListePermut=0
			while(ContinuerPermut)
			{
				va1<-as.numeric(readline("Quelle variable permuter? (premier choix) \n"))
				va2<-as.numeric(readline("Quelle variable permuter?	(second choix) \n"))
				while( is.element(va1,ListeEnlever) || is.element(va2,ListeEnlever) || !is.element(va1,1:vae)|| !is.element(va2,1:vae)){
					cat("\nLa premire variable est  retirer :",is.element(va1,ListeEnlever), "\n La seconde variable est  retirer :",is.element(va2,ListeEnlever), "\n La premire variable n'est pas une entre :",!is.element(va1,1:vae), "\n La seconde variable n'est pas une entre :",,!is.element(va2,1:vae),"\n")
					cat(" Vous avez demand  retirer l'une des variables que vous voquez, ou alors vous demandez une variable qui n'est pas une entre \n")
					va1<-as.numeric(readline("Recommencez (premier choix) \n"))
					va2<-as.numeric(readline("Recommencez	(second choix) \n"))
				}
				ListePermut=c(ListePermut,va1,va2)
				cat("on va permuter les entres :", dimnames(T)[[2]][va1] ," et :", dimnames(T)[[2]][va2] ," resp numros ",va1," et ",va2,"\n")
				fini=readline("Est-ce tout ? (o,n)")
				while(fini!="o" && fini!="n"){
					fini<-readline("rponse non comprise, recommencer svp(o,n) : ")
				}
				if (fini=="o"){
					ContinuerPermut=FALSE
					ListePermut=ListePermut[2:length(ListePermut)]
				} # end if fini

			} # end while continuer  permuter

		} # end if on permute
		# vachercher II.2.c # Si l'on ne permute pas

		else 
		{cat("On ne permute donc pas \n")
		ListePermut="pasdepermutation"
		}



# vachercher III ###############  Slection de la sortie ################################################################
	cat("Quelle sortie prends on ? (numero) \n")
	repso<-as.numeric(readline(""))
		while(!is.element(repso,1:dim(T)[2]-vae)){
			repso<-as.numeric(readline("rponse non comprise, recommencer svp : "))
		}

# vachercher IV ###############   Concatnation #########################################################################

	# vachercher IV.1 ##  Cration des variables 

		# vachercher IV.1.a # travail sur xy qui sera la matrice de sortie
		xy=matrix(NA,dim(T)[1],vae+1)
		for (i in 1:vae){
			xy[,i]=T[,i]
		}
		colnames(xy)=c(dimnames(T)[[2]][1:vae],"cettecolonneesttemporaire")

		# vachercher IV.1.b # listevardef contient les numros des variables qu'on gardera
		listevardef=(1:vae)[!(1:vae)%in%ListeEnlever]	

		# vachercher IV.1.c # listenomvardef contient les nom des variables qu'on gardera
		listenomvardef="vide"					
		for (i in listevardef){
			listenomvardef=c(listenomvardef,dimnames(T)[[2]][i])
		}
		listenomvardef=listenomvardef[2:length(listenomvardef)] 


	# vachercher IV.2 ##  Permutation effective  des entres voulues

		if (ListePermut[1]!="pasdepermutation")
		{
			for (i in 1:(length(ListePermut)/2))
			{
			a1=ListePermut[2*i-1]
			a2=ListePermut[2*i]
			xtemp=xy[,a1]
			nomtemp=dimnames(xy)[[2]][a1]

			xy[,a1]=xy[,a2]
			dimnames(xy)[[2]][a1]=dimnames(xy)[[2]][a2]
	
			xy[,a2]=xtemp
			dimnames(xy)[[2]][a2]=nomtemp	
			}
		}

		else {}


	# vachercher IV.3 ##  Suppression effective des entres voulues

		# vachercher IV.3.a # dans ce cas on supprime toutes les colonnes
					  # dont le nom n'apparait pas dans la liste prvue  cet effet
		if (ListeEnlever!="onenleverien") 
		{	le=0
			for (i in 1:(dim(xy)[2]))
			{	
				if (!is.element(dimnames(xy)[[2]][i],listenomvardef))
				{le=c(le,i)}
			}
			le=le[-1]
			xy=xy[,-le]
		}
		
		# vachercher IV.3.b # dans ce cas on enlve la dernire colonne
					  # de xy qui est temporaire
		else(xy=xy[,-dim(xy)[2]]) 


	# vachercher IV.4 ##  Ajout de la sortie
		xy=cbind(xy,T[,repso+vae])
		dimnames(xy)[[2]][dim(xy)[2]]=dimnames(T)[[2]][repso+vae]

# vachercher V ############### Renvoi de la matrice et Fin de la fonction ##############################################

	return(xy)

}  # end vachercher



########################################################################################################################
########################################################################################################################
## fonction CheckA #########

CheckA<-function (A,naxes){
######################################################################
# cette fonction vrifie que le A fourni a bien le format standart : #
# list(qv,as.numeric(v),as.numeric(va1),as.numeric(va2),ct,ct2)	   #
######################################################################

# CheckA I ############### Initialisation des variables locales ########################################################
	checked=TRUE
	p=c("black","lightpink1","green3","red","magenta","yellow","blue","gray","cyan","orange")
	palette(p)


# CheckA II ############### Traitement dans le cas : A ne contient qu'une seule liste ##################################

	if ( dim(as.matrix(A))[1]==6 && dim(as.matrix(A))[2]==1 )
	{

	# CheckA II.1 ## Cration de variables locales
		N1=TRUE	## Ces deux boolens sont l pour tester l'appartenance des
		N2=TRUE	## valeurs 3 et 4  des valeurs numriques

	# CheckA II.2 ## Vrification sur la premire valeur
		if ( A[[1]]!="q" &&  A[[1]]!="v")
		{ warning("Il y a un problme sur la premire valeur de la ligne 1 : vous devez mettre q pour travail sur les quantiles et v pour travail sur les valeurs \n")
		  checked=FALSE}

	# CheckA II.3 ## Vrification sur la seconde valeur
		if ( ( !is.element(A[[2]],1:naxes) && is.numeric(A[[2]]) ) || (A[[2]]!="s" && !is.numeric(A[[2]]) ) )
		{ warning("Il y a un problme sur la seconde valeur de la ligne 1 : vous devez mettre s pour la sortie ou un chiffre compris entre 1 et le nombre d'axes \n")
		  checked=FALSE}
	# CheckA II.4 ## Vrification sur la troisime et quatrime valeur
		if (!is.numeric(A[[3]]) )
		{ warning("Il y a un problme sur la 3ime valeur de la ligne 1 : ce n'est pas un chiffre ")
		  N1=FALSE
		  checked=FALSE
		}
		if (!is.numeric(A[[4]]) )
		 {warning("Il y a un problme sur la 4ime valeur de la ligne 1 : ce n'est pas un chiffre ")
		 N2=FALSE	
		 checked=FALSE
		}
		if(N1==TRUE && N2==TRUE ) {
			if (A[[3]]>A[[4]] )
			{ warning("Il y a un problme sur les 3imes et 4imes valeurs de la ligne 1 : la 4ime doit etre suprieure  la 3ime\n")
			  checked=FALSE}
		}
	# CheckA II.5 ## Vrification sur la cinquime valeur puis affectation de la 6eme valeur
		if (is.character(A[[5]]) )
		{
			if ((A[[5]] != "c") && (A[[5]] != "t") ) 
			{ warning("Il y a un problme sur la cinquime valeur de la ligne 1 : vous devez mettre c pour couleur ou t pour trac \n")
			    checked=FALSE
			}
			if (A[[5]]=="c")
			{	if ( !is.element(A[[6]],colours()) && !is.element(A[[6]],1:length(palette())))		
 					{warning("La couleur de la ligne 1 n'a pas t comprise\n")
				  	checked=FALSE
				}
			} 
			if (A[[5]]=="t")
			{	if( !is.element(A[[6]],1:6)  ) 		
 					{warning("Le trac de la ligne 1 n'a pas t compris\n")
					 checked=FALSE
				}
			} 
		} # end si la 5e valeur est un character
		else
		{warning("La 5e valeur n'est pas un character. Peut etre avez vous oubli les crochets.\n")
		 checked=FALSE
		}

	} # end if taille=1


# CheckA III ############### Traitement dans le cas : A contient plusieurs listes ##################################

	else if ( dim(as.matrix(A))[2]==6 )
	{	for (i in 1:dim(A)[1])  # pour chaque liste
		{    
	# CheckA III.1 ## Cration de variables locales
			N1=TRUE		## Ces deux boolens sont l pour tester l'appartenance des
			N2=TRUE		## valeurs 3 et 4  des valeurs numriques
	# CheckA III.2 ## Vrification sur la premire valeur
			if ( A[,1][[i]]!="q" &&  A[,1][[i]]!="v")
			{ warning("Il y a un problme sur la premire valeur de la ligne ",i," : vous devez mettre q pour travail sur les quantiles et v pour travail sur les valeurs \n")
			  checked=FALSE}
	# CheckA III.3 ## Vrification de la seconde valeur
			if ( (!is.element(A[,2][[i]],1:naxes) && is.numeric(A[,2][[i]]) ) || ( A[,2][[i]]!="s" && !is.numeric(A[,2][[i]]) ) )
			{ warning("Il y a un problme sur la seconde valeur de la ligne ",i," : vous devez mettre s pour la sortie ou un chiffre compris entre 1 et le nombre d'axes \n")
			   checked=FALSE}
	# CheckA III.4 ## Vrification des troisime et quatrime valeurs
			if (!is.numeric(A[,3][[i]]) )
			{ warning("Il y a un problme sur la 3ime valeur de la ligne ",i," : ce n'est pas un chiffre ")
			  N1=FALSE
			  checked=FALSE}

			if (!is.numeric(A[,4][[i]]) )
			{ warning("Il y a un problme sur la 4ime valeur de la ligne",i," : ce n'est pas un chiffre ")
			  N2=FALSE	
			  checked=FALSE}


			if (N1==TRUE && N2==TRUE ) {
			if (A[,3][[i]]>A[,4][[i]] )
			{ warning("Il y a un problme sur les 3imes et 4imes valeurs de la ligne",i," : la 4ime doit etre suprieure  la 3ime\n")
			  checked=FALSE}
			}
	# CheckA III.5 ## Vrification de la cinquime valeur et affectation de la 6ime
			if (is.character(A[,5][[i]]) ){

				if ((A[,5][[i]] != "c") && (A[,5][[i]] != "t") ) 
				{ warning("Il y a un problme sur la cinquime valeur de la ligne",i," : vous devez mettre c pour couleur ou t pour trac \n")
				  checked=FALSE}
				if (A[,5][[i]]=="c")
				{
					if ( !is.element(A[,6][[i]],colours()) && !is.element(A[,6][[i]],1:length(palette())))		
 					{warning("La couleur de la ligne",i," n'a pas t comprise\n")
					 checked=FALSE}
				} 
				if (A[,5][[i]]=="t")
				{
					if( !is.element(A[,6][[i]],0:6)  ) 		
 						{warning("Le trac de la ligne",i," n'a pas t compris\n")
						 checked=FALSE}
				} 

			} # end si la 5e valeur est un character
			else
			{warning("La 5e valeur de la ligne",i," n'est pas un character. Peut etre avez vous oubli les crochets.\n")
			 checked=FALSE}

		} # end pour chaque liste
	} # end if taille >=2


# CheckA IV ############### Traitement dans le cas : A est de taille non conventionnelle ##################################

	else 
	{ warning (" Il y a un problme avec la taille de votre liste") 
	cat("Il se trouve que votre liste est de taille =",dim(as.matrix(A))[1]," ",dim(as.matrix(A))[2],"\n")
	cat("Le problme vient peut tre de l \n") 
	checked=FALSE
	}

# CheckA V ############### Renvoi du boolean et Fin de la fonction  #######################################################


	return(checked)
} # end CheckA

##########################################################################################################################
##########################################################################################################################
## fonction enq #########

enq<-function(X){
######################################################################################
# cette fonction sert  obtenir,  partir d'une matrice X , une matrice			 #
# Xtransfo telle que la valeur de chaque lment de Xtransfo soit le pourcentile     #
# correspondant dans la colonne de cette valeur							 #
######################################################################################

# enq I ############### Cration de la variable de travail  ###############################################################

	Xtransfo=matrix(NA,dim(X)[1],dim(X)[2])

# enq II ############### Calcul sur la variable de travail  ###############################################################

	# enq II.1 ## Cration des boucles
	for (i in 1:dim(X)[2])  # pour chaque colonne
	{
  		Xrank=rank(X[,i])
		for (j in 1:dim(X)[1]) # pour chaque individu
 		{

	# enq II.2 ## affectation des valeurs
			Xtransfo[j,i]=Xrank[j]/dim(X)[1]
	# enq II.3 ## troncature  la deuxime dcimale suprieure
			if (abs(round(Xtransfo[j,i],2)-Xtransfo[j,i]) > 1e-15 )
			{
				if ( Xtransfo[j,i]-round(Xtransfo[j,i],2) >0 ) {Xtransfo[j,i]=round(Xtransfo[j,i],2)+0.01}
				else if ( Xtransfo[j,i]-round(Xtransfo[j,i],2) <0 ) {Xtransfo[j,i]=round(Xtransfo[j,i],2)}	
			}

		}#end pour chaque individu
	}# end pour chaque colonne

# enq III ############### Renvoi de la matrice et Fin de la fonction  ####################################################

return(Xtransfo)
}# end enq 


##########################################################################################################################
##########################################################################################################################
## fonction coloriage #########

coloriage <- function(naxes)
{
###############################################################################
# Cette fonction sert  dterminer le choix de l'utilisateur quand aux listes	#
# 	de mise en valeur.	 								#					
# Elle a besoin du nombre de variables (au sens large, c'est  dire 		#
#	variables + sorties ) et du paramtre scale					#
###############################################################################

# coloriage I ############### Dfinition des variables locales et rglages des paramtres  ###############################

	# coloriage I.1 # coloriages par dfaut 
		{A=list("q",naxes,0,1,"c","black")} 
	# coloriage I.1 # Rglages couleurs 
		p=c("black","lightpink1","green3","red","magenta","yellow","blue","gray","cyan","orange")
		palette(p)
	# coloriage I.1 # Initialisation boucle 											
		fini=FALSE



# coloriage II ############### Boucle de poursuite du programme  ########################################################

	while (fini==FALSE)
	{

	# coloriage II.1 #  Question sur le choix : Nouveau, Prdfini ou Quitter ?
 		REP.npq <- readline("Que voulez vous faire ? Crer un nouveau coloriage(n), prendre un prdfini(p) ou quitter(q) ?: \n")
			while ((REP.npq != "n") && (REP.npq != "p")&& (REP.npq != "q")) 	{	
 				cat("Veuillez choisir entre 'n' 'p' ou 'q' !! \n")
				REP.npq <- readline("\n") 
			}

	# coloriage II.2 #  Si le choix est Nouveau
		if (REP.npq == "n")
		{
			A1=Nouveau(naxes)
			A=rbind(A,A1)
			cat=("nouvelle mise en valeur cre \n")
		}

	# coloriage II.3 #  Si le choix est Prdfini
		if (REP.npq == "p")
		{

			cat("Nous proposons :\n")
			cat("    -de mettre en rouge le 10-pourcentile final  (1) \n")
			cat("    -de colorier les entres d'une variable en arc en ciel  (2) \n")
			cat("    -de tracer uniquement le 10-pourcentile final en rouge et les valeurs suprieures au 90-pourcentile en vert  (3) \n")
			cat("    -de tracer en pointill les valeurs de sortie entre 10 et 12  (4)\n")
			cat("    -de tracer en bleu les 40  60% de valeurs de sortie  (5)\n")
			cat("    -de tracer uniquement un 10 pourcentile de sortie en rouge (6) \n")


			REP.pred <- readline()
			while (!is.element(REP.pred,1:6) )	{	
 				cat("Veuillez choisir dans les options \n")
				REP.pred <- readline("\n") }
		
			if  (REP.pred==1){
			## le 10% de plus petites valeurs de sortie en rouge
			A1=list("q",naxes,0,0.1,"c","red")}
	
			if  (REP.pred==2){
			##arc en ciel pourcentile
			a=as.numeric(readline("quelle variable ?"))
			while (!is.element(a,rep(1:naxes))){
				cat(" votre variable n'a pas t comprise\n")
				a=as.numeric(readline("recommencez\n"))}
			Aa=list("q",a,0,0.1,"c",1)
			Ab=list("q",a,.1,.2,"c",2)
			Ac=list("q",a,.2,.3,"c",3)
			Ad=list("q",a,.3,.4,"c",4)
			Af=list("q",a,.4,.5,"c",5)
			Ag=list("q",a,.5,.6,"c",6)
			Ah=list("q",a,.6,.7,"c",7)
			Ai=list("q",a,.7,.8,"c",8)
			Aj=list("q",a,.8,.9,"c",9)
			Ak=list("q",a,.9,1,"c",10)
			A1=rbind(Aa,Ab,Ac,Ad,Af,Ag,Ah,Ai,Aj,Ak)
			}

			if  (REP.pred==3){
			##uniquement le .9-1 en vert et le 0-.1 en rouge
			Aa=list("q",naxes,0,1,"t",0) #attention, car les lignes ne seront pas traces mme si on a chang la couleur prcdemment
			Ab=list("q",naxes,0,.1,"c","red")
			Ac=list("q",naxes,0,.1,"t",1)
			Ad=list("q",naxes,.9,1,"c","green")
			Ae=list("q",naxes,.9,1,"t",1)
			A1=rbind(Aa,Ab,Ac,Ad,Ae)
			}
		
			if  (REP.pred==4){
			## en pointill entre 10 et 12 pour la sortie
			A1=list("v",naxes,10,12,"t",3)
			}

			if  (REP.pred==5){
			## en bleu de .4  .6 en sortie
			A1=list("q",naxes,.4,.6,"c","blue")
			}

			if  (REP.pred==6){
			##uniquement un 10 pourcentile en rouge
			a=as.numeric(readline("quelle 10 pourcentile ? ( pour de 0  .1 taper 1, et pour de .5  .6 taper 6)"))
			while (!is.element(a,rep(1:10))){
				cat(" votre choix n'a pas t compris\n")
				a=as.numeric(readline("recommencez\n"))}

			Aa=list("q",naxes,0,1,"t",0) #attention, car les lignes ne seront pas traces mme si on a chang la couleur prcdemment
			Ab=list("q",naxes,(a-1)/10,a/10,"c","red")
			Ac=list("q",naxes,(a-1)/10,a/10,"t",1)
			A1=rbind(Aa,Ab,Ac)
			}
				
			A=rbind(A,A1)
			cat=("nouvelle mise en valeur prdfinie cre \n")
	} ## fin de prdfini


	# coloriage II.4 #  Si le choix est Quitter :
		if (REP.npq == "q") {fini=TRUE}	
	
	} # on a termin npq


# coloriage III ############### Renvoi de la liste de paramtres A et fin du programme  ##################################

	return (A)
} # end coloriage

##########################################################################################################################
##########################################################################################################################

## fonction Nouveau #########

Nouveau<-function(naxes) 
{
###############################################################################
# Cette fonction sert  guider l'utilisateur pour crer une liste de mise en	#	 
#	valeur au bont format 									#
# Elle a besoin du nombre de variables (au sens large, c'est  dire 		#
#	variables + sorties ) sur lesquelles cette mise en valeur va porter	#
###############################################################################



# Nouveau I ###############  travail sur quantile ou valeur ? ########################################################

	qv <- readline("Voulez vous mettre en vidence des quantiles(q) ou des valeurs(v) ?: \n")
		while ((qv != "q") && (qv != "v")) 	{	
 			cat("Veuillez choisir entre 'q' ou 'v' !! \n")
			ANSWER.qv<- readline("\n") 
		}


# Nouveau II ###############  travail sur quelle variable ? ########################################################

	v <- readline("Sur quelle variable (ou sortie) voulez vous votre condition ? (tapez le chiffre ou s pour sortie): \n")
	if (v=="s") {v=naxes}
	while ( !is.element(v,1: naxes) ) 	
	{	
 			cat("Veuillez choisir un entier entre 1 et le nombre de var ou appuyer sur s !! \n")
			v<- readline("\n")
			if (v=="s") {v=naxes} 
	}

# Nouveau III ###############  travail sur quelle valeurs ? ########################################################

	# Nouveau III.1 #  Saisie des valeurs et vrification de saisie de valeurs numriques
		va1 <- as.numeric(readline("Dans quelle plage de valeurs voulez vous votre modification ? (tapez le premier chiffre): \n"))
		va2 <- as.numeric(readline("tapez le second chiffre: \n"))

 		while (is.na(va1) || is.na(va1)) 	{	
 				cat("Vous devez saisir des valeurs, pas autre chose :! \n")
				cat("Veuillez re saisir deux valeurs numriques \n")
				va1 <- as.numeric(readline("Premiere valeur(entre 0 et 1): \n"))
				va2 <- as.numeric(readline("Seconde valeur(entre 0 et 1): \n"))
		}

	# Nouveau III.2 #  Vrification dans le cas quantile
		if (qv=="q")
		{ 	while ( (va1<0) || (va2>1) || (va2<va1)) 	{	
 				cat("Vous avez choisi des quantiles, vos valeurs ne sont pas pertinantes !! \n")
				cat("Veuillez re saisir deux valeurs entre 0 et 1 , la premire plus petite ou gale  la seconde \n")
				va1 <- as.numeric(readline("Premiere valeur(entre 0 et 1): \n"))
				va2 <- as.numeric(readline("Seconde valeur(entre 0 et 1): \n"))
			}
 		}

	# Nouveau III.3 #  Vrification dans le cas pourcentile
		if (qv=="v")
		{ 	while ( va2<va1  ) 	{	
 				cat("Vous avez choisi vos valeurs qui ne sont pas pertinantes !! \n")
				cat("Veuillez re saisir deux valeurs numriques\n")
				va1 <- as.numeric(readline("Premiere valeur: \n"))
				va2 <- as.numeric(readline("Seconde valeur : \n"))
			}
 		}

# Nouveau IV ################# Mise en valeur ###########################################################################

	# Nouveau IV.1 #  Slection Couleur ou trac ?
		ct <- readline("Voulez vous changer la couleur ou le trac ? (tapez c ou t ): \n")
		while ((ct != "c") && (ct != "t") ) 	{	
 			cat("Veuillez choisir c ou t !! \n")
			ct<- readline("\n") 
		}

	# Nouveau IV.2 #  Slection  proprement parler de la mise en valeur
		# Nouveau IV.2.a # Dans le cas d'une couleur
			if (ct=="c"){
				ct2 <- readline("tapez la couleur dsire en toutes lettres ou en chiffres: \n") 
				while ( !is.element(ct2,colours()) && !is.element(ct2,1:length(palette()))  ) 	{	
 					cat("Couleur non comprise, recommencer \n")
					ct2 <- readline("\n") 
				}
			} # end if couleur

		# Nouveau IV.2.b # Dans le cas d'un trac
			if (ct=="t"){
				ct2 <- readline("tapez le trac dsire par un numro entre 0 et 6 : \n")
				while ( !is.element(ct2,0:6)  ) 	{	
 					cat("trac non compris, recommencer \n")
					ct2<- readline("\n") 
				}
			} # end if trac

# Nouveau V ################# Concatnation des valeurs, renvoi de la concatnation et fin de la fonction ####################

	ANSWER=list(qv,as.numeric(v),as.numeric(va1),as.numeric(va2),ct,ct2)
	return(ANSWER)
} # end Nouveau

##########################################################################################################################
##########################################################################################################################
## fonction CreerColStyl #########

CreerColStyl=function(A,xy){
#####################################################################################
# Cette fonction reoit deux arguments : 								#
# 	A est un empilement de listes formates de la bonne faon (cf fonction CheckA)#
# 	xy est une matrice										#
# La fonction cre une structure qui contient deux vecteurs , dont la taille est	#
#	le nombre de lignes de xy.									#
# La premier vecteur contient la couleur qui sera utilise pour tracer chaque		#
#	individu												#
# Le second vecteur contient le style de ligne qui sera utilis pour tracer chaque 	#
#	individu												#
# Il est  noter que la priorit sera accorde  la dernire entre de la liste,	#
#	dans le cas o un individu satisfait  plusieurs conditions de mises en valeur# 
#####################################################################################




# CreerColStyl I ############ Creation des variables locales ############################################################

	naxes=dim(xy)[2]
	nombredindiv=dim(xy)[1]
	colk=matrix(1,nombredindiv) # par dfaut on trace tout en noir
	stylek=matrix(1,nombredindiv) # par dfaut c'est la ligne droite

# CreerColStyl II ########### si la taille de A est de un ###############################################################

	if ( dim(as.matrix(A))[1]==6 && dim(as.matrix(A))[2]==1 ) 
	{
		if (A[2][[1]]=="s"){A[2][[1]]=naxes}

	# CreerColStyl II.1 # si l'on traite de quantiles
		if (A[1][[1]]=="q")
		{
			Xrank=rank(xy[,A[2][[1]]])
			for (k in 1:nombredindiv){ # pour chaque individu
				if (  (A[3][[1]]*nombredindiv<=Xrank[k]) && (A[4][[1]]*nombredindiv>=Xrank[k]) ) {
					if (A[5][[1]]=="c"){
					colk[k]=A[6][[1]]
					}
					if (A[5][[1]]=="t"){
					stylek[k]=A[6][[1]]
					}
				}
			}# end for indiv	
		}# end of quantile

	# CreerColStyl II.2 # si l'on traite de valeurs
		if (A[1][[1]]=="v") 
		{
			for (k in 1:nombredindiv){
				if ((A[3][[1]]<=xy[k,A[2][[1]]]) && (A[4][[1]]>=xy[k,A[2][[1]]])){	
					if (A[5][[1]]=="c"){
					colk[k]=A[6][[1]]
					}
					if (A[5][[1]]=="t"){
					stylek[k]=A[6][[1]]
					}
				}
			}# end for indiv
		}# end of valeur	
}# end if taille =1



# CreerColStyl III ########### si la taille de A est suprieure  2 ##################################################

	else if ( dim(as.matrix(A))[2]==6 ) 
	{
		for (i in 1:dim(A)[1])  # pour chaque mise en valeur
		{	if (A[,2][[i]]=="s") {A[,2][[i]]=naxes}

	# CreerColStyl III.1 # si l'on traite de quantiles
			if (A[i,1]=="q")
			{
				for (k in 1:nombredindiv) # pour chaque individu
				{
				Xrank=rank(xy[,A[,2][[i]]])

					if (( A[,3][[i]]*nombredindiv<=Xrank[k]) && (A[,4][[i]]*nombredindiv>=Xrank[k]) )
					{	
						if (A[i,5]=="c"){
						colk[k]=A[,6][[i]]
						}
						if (A[i,5]=="t"){
						stylek[k]=A[,6][[i]]
						}
					}	
				}	# end pour chaque individu
			}

	# CreerColStyl III.2 # si l'on traite de valeurs

			if (A[i,1]=="v")
			{
				for (k in 1:nombredindiv)
				{
					if ((A[,3][[i]]<=xy[k,A[,2][[i]]]) && (A[,4][[i]]>=xy[k,A[,2][[i]]]))
					{	
						if (A[i,5]=="c"){
						colk[k]=A[,6][[i]]
						}
						if (A[i,5]=="t"){
						stylek[k]=A[,6][[i]]
						}
					}	
				}	
			}

		} # end for lignes de A is pour chaque mise en valeur
	} # end if taille >=2

# CreerColStyl IV ################# Concatnation des vecters, renvoi de la liste et fin de la fonction ####################

	return(list(colk,stylek))
} # End CreerColStyl


#########################################################################################################################
#########################################################################################################################
### fin du fichier cobweb

