contributions<-function(x,y,forme="maformule",degre=1,famille=gaussian,
              intercept=TRUE,
              a1=rep("no",dim(x)[2]),
              a2=rep("no",dim(x)[2]),
              a3=rep("no",dim(x)[2]),
              a4=rep("no",dim(x)[2]),
              b1=array("no",dim=c(dim(x)[2],dim(x)[2])),
	      c1=array("no",dim=c(dim(x)[2],dim(x)[2],dim(x)[2])),
	      c2=array("no",dim=c(dim(x)[2],dim(x)[2])),
	      d1=array("no",dim=c(dim(x)[2],dim(x)[2],dim(x)[2],dim(x)[2])),
              d2=array("no",dim=c(dim(x)[2],dim(x)[2])),
	      d3=array("no",dim=c(dim(x)[2],dim(x)[2],dim(x)[2])),
	      d4=array("no",dim=c(dim(x)[2],dim(x)[2]))){

#---------------------------------------------------------------------------
#Fonction permettant de calculer les contributions des effets des          |
#coefficients a la variation interclasse (modele GLM)                      |
#                                                                          |
#La fonction commence par transformer la matrice x dont les colonnes sont  |
#les Vi en une matrice dont les colonnes sont les Vi, les Vi^2, les        |
#ViVj ... suivant ce qu'on a precise en option                             |
#                                                                          |
#La fonction calcule alors les contributions des effets des coefficients   |
#de la formule de regression. C'est pourquoi on precise de quelle facon    |
#celle-ci a ete creee et les options choisies pour la creer                |
#                                                                          |  
#--------------------------------------------------------------------------|
#Arguments                                                                 |
#obligatoires : x : la matrice contenant les variables d'entree            |
#               y : la variable de sortie                                  |
#                                                                          |
#Arguments                                                                 |
#facultatifs : forme : sert a preciser si la formule de regression a ete   |
#              creee avec "creationformule" ou avec "maformule"            |
#              degre : precise le degre des interactions si on a choisi    |
#              "creationformule"                                           |
#              famille : le programme suppose que la fonction de densite   |
#              de y est de type "famille". Par defaut, on suppose qu'elle  |
#              est gaussienne                                              |
#              intercept : precise si on a inclu un intercept dans la      |
#              formule si on a choisi "maformule"                          |
#              a1 : precise si on a inclu les Vi dans la formule si on a   |
#              choisi "maformule"                                          |
#              a2 : precise si on a inclu les Vi^2 dans la formule si on a |
#              choisi "maformule"                                          |
#              a3 : precise si on a inclu les Vi^3 dans la formule si on a |
#              choisi "maformule"                                          |
#              a4 : precise si on a inclu les Vi^4 dans la formule si on a |
#              choisi "maformule"                                          |
#              b1 : precise si on a inclu les ViVj dans la formule si on a |
#              choisi "maformule"                                          |
#              c1 : precise si on a inclu les ViVjVk dans la formule si on |
#              a choisi "maformule"                                        |
#              c2 : precise si on a inclu les Vi^2Vj dans la formule si on |
#              a choisi "maformule"                                        |
#              d1 : precise si on a inclu les ViVjVkVl dans la formule si  |
#              on a choisi "maformule"                                     |
#              d2 : precise si on a inclu les Vi^3Vj dans la formule si on |
#              a choisi "maformule"                                        |
#              d3 : precise si on a inclu les Vi^VjVk dans la formule si   |
#              on a choisi "maformule"                                     |
#              d4 : precise si on a inclu les Vi^2Vj^2 dans la formule si  |
#              on a choisi "maformule"                                     |
#                                                                          |
#Il faut definir au moins un des arguments facultatifs sinon la fonction   |
#ne calcule rien                                                           |
#--------------------------------------------------------------------------|
#Sortie : les contributions de chacun des effets                           |
#--------------------------------------------------------------------------|
#      Auteur : A. Ducellier et B. Iooss                                   |
#--------------------------------------------------------------------------|  

  dd <- dim(x)[2]
#------------------------------------------   
#---------------------------------------- |
#Transformation de la variable d'entree | |
#---------------------------------------- |
#------------------------------------------

#--------------------------------------------
#Transformation quand forme=creationformule |
#--------------------------------------------

if(forme=="creationformule"){

#Initialisation de la matrice xbis
xbis<-c()

#Boucle pour les Vi
if(degre>0){
for(i in 1:dd){
   xbis<-matrix(c(xbis,x[,i]),nr=dim(x)[1])}
}

#Boucle pour les Vi^2
if(degre>1){
for(i in 1:dd){
   xbis<-matrix(c(xbis,x[,i]*x[,i]),nr=dim(x)[1])}
}

#Boucle pour les Vi^3
if(degre>2){
for(i in 1:dd){
   xbis<-matrix(c(xbis,x[,i]*x[,i]*x[,i]),nr=dim(x)[1])}
}

#Boucle pour les Vi^4
if(degre>3){
for(i in 1:dd){
   xbis<-matrix(c(xbis,x[,i]*x[,i]*x[,i]*x[,i]),nr=dim(x)[1])}
}

#Boucle pour les ViVj
if(degre>1 & dd>1){
for(i in 1:(dd-1)){
   for(j in (i+1):dd){
      xbis<-matrix(c(xbis,x[,i]*x[,j]),nr=dim(x)[1])}}
}

#Boucle pour les Vi^2Vj
if(degre>2 & dd>1){
for(i in 1:dd){
   for(j in 1:dd){
      if(i==j) xbis=xbis    
      else xbis<-matrix(c(xbis,x[,i]*x[,i]*x[,j]),nr=dim(x)[1])}}
}

#Boucle pour les Vi^2Vj^2
if(degre>3 & dd>1){
for(i in 1:(dd-1)){
   for(j in (i+1):dd){
      xbis<-matrix(c(xbis,x[,i]*x[,i]*x[,j]*x[,j]),nr=dim(x)[1])}}
}   

#Boucle pour les Vi^3Vj
if(degre>3 & dd>1){
for(i in 1:dd){
   for(j in 1:dd){
      if(i==j) xbis=xbis
      else xbis<-matrix(c(xbis,x[,i]*x[,i]*x[,i]*x[,j]),nr=dim(x)[1])}}
}

#Boucle pour les ViVjVk
if(degre>2 & dd>2){
for(i in 1:(dd-2)){
   for(j in (i+1):(dd-1)){
      for(k in (j+1):dd){
         xbis<-matrix(c(xbis,x[,i]*x[,j]*x[,k]),nr=dim(x)[1])}}}
}

#Boucle pour les Vi^2VjVk
if(degre>3 & dd>2){
for(i in 1:dd){
   for(j in 1:(dd-1)){
      for(k in (j+1):dd){
         if(i==j|i==k) xbis=xbis
	 else xbis<-matrix(c(xbis,x[,i]*x[,i]*x[,j]*x[,k]),nr=dim(x)[1])}}}
}

#Boucle pour les ViVjVkVl
if(degre>3 & dd>3){   
for(i in 1:(dd-3)){
   for(j in (i+1):(dd-2)){
      for(k in (j+1):(dd-1)){
         for(l in (k+1):dd){
	    xbis<-matrix(c(xbis,x[,i]*x[,j]*x[,k]*x[,l]),nr=dim(x)[1])}}}}
} 

}

#--------------------------------------
#Transformation quand forme=maformule |
#--------------------------------------

if(forme=="maformule"){

#Initialisation de la matrice xbis
  xbis<-c() 

#Boucle pour les Vi
  for(i in 1:dd){
    if (a1[i]=="yes"){
      xbis<-matrix(c(xbis,x[,i]),nr=dim(x)[1])
    }
  }

#Boucle pour les Vi^2
  for(i in 1:dd){
    if (a2[i]=="yes"){
      xbis<-matrix(c(xbis,x[,i]*x[,i]),nr=dim(x)[1])
    }
  }

#Boucle pour les Vi^3
  for(i in 1:dd){
    if (a3[i]=="yes"){
      xbis<-matrix(c(xbis,x[,i]*x[,i]*x[,i]),nr=dim(x)[1])
    }
  }

#Boucle pour les Vi^4
  for(i in 1:dd){
    if (a4[i]=="yes"){
      xbis<-matrix(c(xbis,x[,i]*x[,i]*x[,i]*x[,i]),nr=dim(x)[1])
    }
  }

#Boucle pour les ViVj
  if(dd>1){for(i in 1:(dd-1)){
              for(j in (i+1):dd){
                 if (b1[i,j]=="yes"){
                    xbis<-matrix(c(xbis,x[,i]*x[,j]),nr=dim(x)[1])
                 }
              }
	   }
  }

#Boucle pour les Vi^2Vj
  if(dd>1){for(i in 1:dd){
              for(j in 1:dd){
                 if(c2[i,j]=="yes"){
                   if (i==j){
	           xbis=xbis
	           }
	           else xbis<-matrix(c(xbis,x[,i]*x[,i]*x[,j]),nr=dim(x)[1])
                 }
              }
           }
  }

#Boucle pour les Vi^3Vj
  if(dd>1){for(i in 1:dd){
              for(j in 1:dd){
                 if (d2[i,j]=="yes"){
                    if (i==j){
	            xbis=xbis
	            }
	            else xbis<-matrix(c(xbis,x[,i]*x[,i]*x[,i]*x[,j]),nr=dim(x)[1])
                 }
              }
           }
  }

#Boucle pour les Vi^2Vj^2
  if(dd>1){for(i in 1:(dd-1)){
              for(j in (i+1):dd){
                 if (d4[i,j]=="yes"){
                    xbis<-matrix(c(xbis,x[,i]*x[,i]*x[,j]*x[,j]),nr=dim(x)[1])
                 }
              }
	   }
  }

#Boucle pour les ViVjVk
  if(dd>2){for(i in 1:(dd-2)){
              for(j in (i+1):(dd-1)){
                 for(k in (j+1):dd){
                    if (c1[i,j,k]=="yes"){
	               xbis<-matrix(c(xbis,x[,i]*x[,j]*x[,k]),nr=dim(x)[1])
                    }
		 }
              }
           }
  }

#Boucle pour les Vi^2VjVk
  if(dd>2){for(i in 1:dd){
              for(j in 1:(dd-1)){
                 for(k in (j+1):dd){
	            if (d3[i,j,k]=="yes"){
	               if (i==j|i==k){
	               xbis=xbis
	               }
	               else xbis<-matrix(c(xbis,x[,i]*x[,i]*x[,j]*x[,k]),nr=dim(x)[1])
                    }                 
		 }
              }
           }
  }

#Boucle pour les ViVjVkVl
  if(dd>3){for(i in 1:(dd-3)){
              for(j in (i+1):(dd-2)){
                 for(k in (j+1):(dd-1)){
                    for(l in (k+1):dd){
	               if (d1[i,j,k,l]=="yes"){
		          xbis<-matrix(c(xbis,x[,i]*x[,j]*x[,k]*x[,l]),nr=dim(x)[1])
                       }
	            }
                 }
              }
           }
  }

}

#----------------------------
#-------------------------- |
#Calcul des contributions | |
#-------------------------- |
#----------------------------

#------------------
#Calcul du modele |
#------------------

#Calcul de la formule
if(forme=="maformule"){
  formule=maformule(dd,intercept=intercept,a1=a1,a2=a2,a3=a3,a4=a4,b1=b1,c1=c1,c2=c2,d1=d1,d2=d2,d3=d3,d4=d4)}
else{
  formule=creationformule(dd,interaction.depth=degre)}

#Calcul du modele
modele<-glm(formule,famille,data=data.frame(x,y))

#--------------------------
#Calcul des contributions |
#--------------------------

#Calcul du vecteur des coefficients
l<-length(modele$coefficients)

if(forme=="creationformule")
{A<-modele$coefficients[2:l]}
else
{if(intercept==TRUE)
{A<-modele$coefficients[2:l]}
else
{A<-modele$coefficients}
}

#Initialisation du vecteur des contributions
T<-rep(0,dim(xbis)[2])

#Calcul du vecteur des contributions
for(i in 1:dim(xbis)[2]){
T[i]<-(dim(xbis)[1]-1)*A[i]*cov(xbis[,i],y)
}

#---------------------------------------
#Calcul de l'indice de conditionnement |
#---------------------------------------

C<-cor(xbis)
valeurspropres<-eigen(C)$values
idc<-max(valeurspropres)/min(valeurspropres)

#Renvoi
return(c(T,C,idc))

}
