correlationS<-function(x,nom=c(NA)){

#--------------------------------------------------------------
#Cette fonction permet de calculer les correlations entre     |
#les variables x et les correlations entre les rangs des x    |
#-------------------------------------------------------------|
#Argument                                                     |
#obligatoire : x : la matrice des variables d'entree          |
#                                                             |
#Argument                                                     |
#facultatif : nom : vecteur des noms des variables d'entree   |
#             (6 caracteres)                                  |
#-------------------------------------------------------------|
#Sortie : la matrice de correlations                          |
#         l'indice de conditionnement                         |
#         la matrice de correlations des rangs                |
#                                                             |
#Les resultats (correlations simples + indice de condit.)     |
#s'affichent dans un fichier nomme "res-CORRELATIONS.out"     |
#qui se trouve dans le repertoire de travail                  |
#                                                             | 
#2 fichiers "matcorrel.dat" et "matcorrelrank.dat" contenant  |
#les matrices de correlation simple et de correlation sur les |
#rangs sont aussi crees                                       |
#-------------------------------------------------------------|
#      Auteur : B. Iooss et A. Ducellier                      |
#-------------------------------------------------------------|  

#Calcul des dimensions de x
p<-dim(x)[1]
n<-dim(x)[2]

#Calcul des correlations
C<-cor(x)
Crank<-cor(x,method="spearman")

#Calcul de l'indice de conditionnement
valeurspropres<-eigen(C)$values
valeurspropres<-sort(valeurspropres)
idc<-valeurspropres[n]/valeurspropres[1]

Cf<-format(C,digits=4)

#----------------------------------
#Creation du fichier de resultats |
#---------------------------------- 
FOUT <- file("res-CORRELATIONS.out","w")
FOUT1 <- file("matcorrel.dat","w")
FOUT2 <- file("matcorrelrank.dat","w")

#-----------
#Affichage |
#-----------

#Affichage si le nom n'est pas defini
if(is.na(nom[1])==TRUE){

  write("----------------------------",file=FOUT)
  write("| Matrice des correlations |",file=FOUT)
  write("----------------------------",file=FOUT)
  write(" ",file=FOUT)

  write(c("       ",dimnames(x)[[2]]),ncolumns=(n+1),file=FOUT)
  write(" ",file=FOUT)

  for(i in 1:n){
    write(c(dimnames(x)[[2]][i],"   ",Cf[i,]),ncolumns=(n+2),file=FOUT)
    write(" ",file=FOUT)
  }
  write.table(C,file=FOUT1,row.names=FALSE,col.names=FALSE)
  write.table(Crank,file=FOUT2,row.names=FALSE,col.names=FALSE)
}

#Affichage si le nom est defini
else{

  write("----------------------------",file=FOUT)
  write("| Matrice des correlations |",file=FOUT)
  write("----------------------------",file=FOUT)
  write(" ",file=FOUT)

  write(c("          ",nom),ncolumns=(n+1),file=FOUT)
  write(" ",file=FOUT)

  for(i in 1:n){
    write(c(nom[i],"   ",C[i,]),ncolumns=(n+2),file=FOUT)
    write(" ",file=FOUT)
  }
  write.table(C,file=FOUT1,row.names=FALSE,col.names=FALSE)
  write.table(Crank,file=FOUT2,row.names=FALSE,col.names=FALSE)
}

write("-----------------------------",file=FOUT)
write("| Indice de conditionnement |",file=FOUT)
write("-----------------------------",file=FOUT)
write(idc,file=FOUT)
write(" ",file=FOUT)

#------------------------------------------------
#Fermeture et affichage du fichier de resultats |
#------------------------------------------------

close(FOUT)
close(FOUT1)
close(FOUT2)

if (Sys.getenv("OS")!="Windows_NT"){ pager_OS="cat"}
else{ pager_OS=""}
file.show("res-CORRELATIONS.out",pager=pager_OS)

}
