creationformule<-function(dimx,interaction.depth=1,
                          gam=F,xnom="V",ynom="y"){

#-------------------------------------------------------------------------
#Fonction permettant de generer une formule de regression a partir       |
#du nombre de parametres dimx                                            |
#                                                                        |
#Pour interaction.depth = 1, la formule est de la forme y ~ V1+V2+...    |
#                                                                        |
#Pour interaction.depth = 2, la formule est de la forme y ~ (V1+V2+...)^2|
# + I(V1^2) + ...                                                        |
#                                                                        |
#Pour interaction.depth = 3, la formule est de la forme y ~ (V1+V2+...)^3|
#+ I(V1^2) + ... + I(V1^3) + ... + I(Vi^2):Vj                            |
#                                                                        |
#Pour interaction.depth = 4, la formule est de la forme (V1+V2+...)^4 +  |
#I(V1^2) + ... + I(V1^3)+ ... + I(V1^4) + ... + I(Vi^2):Vj +             |
#I(Vi^2):I(Vj^2) + I(Vi^3):Vj + I(Vi^2):Vj:Vk                            |
#                                                                        |
#Il est important que la formule obtenue a la fin soit de la             |
#forme :                                                                 |
#y ~ 1 + V1 + ... + I(V1^2) + ...                                        |
#sinon les fonctions glm et predict.glm utilisees dans "cross",          |
#"bootstrap" et "smurfer" ne marcheront pas                              |
#                                                                        |
#------------------------------------------------------------------------|
#Argument                                                                |
#obligatoire : dimx : la dimension des variables d'entree                |
#                                                                        |
#Arguments                                                               |
#facultatifs : interaction.depth : l'ordre des interactions considerees  |
#              gam : si TRUE, on introduit des splines s()               |
#              xnom : le "nom" des parametres d'entree                   |
#              ynom : le "nom" de la variable de sortie                  |
#                                                                        |
#Par defaut, la formule est de la forme :                                |
#y ~ 1 + V1 + V2 + ... + Vn                                              |
#------------------------------------------------------------------------|
#Sortie : un objet de type formule                                       |
#------------------------------------------------------------------------|
#      Auteur : A. Ducellier, B. Iooss et P-M. Pair                      |
#------------------------------------------------------------------------|  

#Initialisation
  f=paste(ynom,"~(")

#Boucle rajoutant le terme (V1 + V2 + .. + Vn)^d ( d = 1, 2, 3 ou 4 )
  for(i in 1:dimx) f <- paste(f,xnom,as.character(i),"+",sep="")

#On enleve le dernier "+" inutile
  f=substr(f,1,nchar(f)-1)

# Fin de la creation de ce premier terme
  if (interaction.depth>1){
    f <- paste( f , ")^" , as.character(interaction.depth) , sep="")
  }
  else if (interaction.depth==1){
    f=paste(f,")",sep="")
  }
     
#Boucle pour prendre les Vi^2, Vi^3, Vi^4
  if (interaction.depth>1){
    for(j in 2:interaction.depth){
      for(i in 1:dimx){
        f <- paste(f,"+I(",xnom,as.character(i),"^",as.character(j),")",sep="")
      }
    }
  }
  else f=f

#Boucle pour prendre les Vi^2Vj
  if (interaction.depth>2 & dimx>1){
    for(i in 1:dimx){
      for(j in 1:dimx){
         if (i==j){
           f=f
	 }
	 else f <- paste(f,"+I(",xnom,as.character(i),"^2):",xnom,as.character(j),sep="")
       }
    }
  }

#Boucle pour prendre les Vi^2Vj^2
  if (interaction.depth>3 & dimx>1){
    for(i in 1:(dimx-1)){
      for(j in (i+1):dimx){
        f <- paste(f,"+I(",xnom,as.character(i),"^2):I(",xnom,as.character(j),"^2)",sep="")
      }
    }
  }
  else f=f

#Boucle pour prendre les Vi^3Vj
  if (interaction.depth>3 & dimx>1){
    for(i in 1:dimx){
      for(j in 1:dimx){
        if (i==j){
          f=f
        }
        else f <- paste(f,"+I(",xnom,as.character(i),"^3):",xnom,as.character(j),sep="")
      }
    }
  }
 
#Boucle pour prendre les Vi^2VjVk
  if (interaction.depth>3 & dimx>2){
    for(i in 1:dimx){
      for(j in 1:(dimx-1)){
	 for(k in (j+1):dimx){
           if(i==j|i==k){
             f=f
           }
           else f <- paste(f,"+I(",xnom,as.character(i),"^2):",xnom,as.character(j),":",xnom,as.character(k),sep="")
         }
       }
    }
  }
  else f=f

  if (gam == T){
# GAM : boucle pour prendre en compte les s(Vi)
    for(i in 1:dimx) f <- paste(f,"+s(",xnom,as.character(i),")",sep="")
    
# GAM : boucle pour prendre en compte les s(Vi,Vj)
    if (interaction.depth>1 & dimx>1){
      for(i in 1:(dimx-1)){
        for(j in (i+1):dimx){
          f <- paste(f,"+s(",xnom,as.character(i),",",xnom,as.character(j),")",sep="")
        }
      }
    }

# GAM : boucle pour prendre en compte les s(Vi,Vj,Vk)
    if (interaction.depth>2 & dimx>2){
      for(i in 1:(dimx-2)){
        for(j in (i+1):dimx-1){
          for(k in (j+1):dimx){
            f <- paste(f,"+s(",xnom,as.character(i),",",xnom,as.character(j),",",xnom,as.character(k),")",sep="")
          }
        }
      }
    }

# GAM : boucle pour prendre en compte les s(Vi,Vj,Vk,Vl)
    if (interaction.depth>3 & dimx>3){
      for(i in 1:(dimx-3)){
        for(j in (i+1):dimx-2){
          for(k in (j+1):dimx-1){
            for(l in (k+1):dimx){
              f <- paste(f,"+s(",xnom,as.character(i),",",xnom,as.character(j),",",xnom,as.character(k),",",xnom,as.character(l),")",sep="")
            }
          }
        }
      }
    }
  }
  
#Transformation en formule et renvoi
  return(as.formula(f))

}
