effectS <- function(f=a,modele=b,methode=0,dim_x,x=NULL,
                logx=rep(FALSE,dim_x),logy=FALSE,S=1000,R=100,
                LHS=FALSE,lois=rep(0,dim_x),
                paramlois=array(0,dim=c(4,dim_x)),
                tronq=rep(F,dim_x),paramtronq=array(0,dim=c(2,dim_x)),...){

#---------------------------------------------------------------------------|
#Fonction pour calculer les effets elementaires                             |
#---------------------------------------------------------------------------|
#Arguments obligatoires dans le cas d'un modele sous forme d'objet R :      |
#       modele : objet contenant le modele                                  |
#       methode : numero de la methode, 0 si fonction utilisateur           |
#       dim_x : nombre de parametres d'entree dans le modele                |
#                                                                           |
#Arguments obligatoires dans le cas d'une fonction :                        |
#       f : objet representant la fonction (prealablement declaree)         |
#       dim_x : nombre de parametres d'entree dans le modele                |
#                                                                           |
#Arguments facultatifs :                                                    |
#             x : matrice de variables d'entree (pour calculer les bornes   |
#               de la loi uniforme si necessaire)                           |
#             logx : vecteur logx=c(TRUE,TRUE,...,TRUE), correspondant aux  |
#               parametres d'entree transformees en log lors de             |
#               l'ajustement du modele                                      |
#             logy : mettre logy=TRUE si la variable de sortie a ete        |
#               transformee en log lors de l'ajustement du modele           |
#             S : nombre de simulations pour l'integration MC               |
#             R : nombre de pas de discretisations dans le calcul des effets|
#             LHS : TRUE pour utiliser un echantillonnage LHS lors du       |
#               calcul des indices de Sobol                                 |
#               Par defaut : FALSE pour un echantillonnage simple           |
#             lois : vecteur contenant les types de distribution de proba   |
#               pour chaque entree                                          |
#               0=uniforme ; 1=normale ; 2=lognormale ; 3=weibull           | 
#               4=exponentielle ; 5=beta ; 6=triangulaire ; 7=trapezoidale  |
#               10=gumbel                                                   |
#               Par defaut, on prend la loi uniforme                        |
#             paramlois : tableau avec les parametres de chaque loi (max=4) |
#               pour chaque entree (range par colonne) :                    |
#               (min,max,0,0) pour uniforme (par dfaut : min=0, max=1)     |
#               (moy,ecart-type,0,0) pour normale,                          |
#               (moy du log, ecart-type du log,0,0) pour lognormale,        |
#               (forme,echelle,0,0) pour Weibull,                           |
#               (lambda,0,0,0) pour exponentielle,                          |
#               (shape1, shape2,0,0) pour beta,                             |
#               (min,mode,max,0) pour triangulaire,                         |
#               (min,mode1,mode2,max) pour trapezoidale,                    |
#               (mode,echelle,0,0) pour Gumbel                              |
#                Par defaut, on a la loi uniforme avec (min,max) calcules a |
#                partir des parametres d'entree x                           |
#             tronq : vecteur pour selectionner ou non une loi tronquee     |
#               TRUE pour loi tronquee, FALSE sinon (par dfaut)            |
#             paramtronq : tableau avec les parametres de troncature de     |
#               chaque loi : (min,max) range par colonne                    |
#---------------------------------------------------------------------------|
#Sortie : 1 graphe avec tous les effets elementaires superposes             |
#         La matrice des effets elementaires EE                             |
#---------------------------------------------------------------------------|
#      Auteur : B. Iooss                                                    |
#---------------------------------------------------------------------------|  

  EE <<- array(0,dim=c(R,dim_x))
  xc <- seq(1:R)

# Pour definir les bornes de la loi uniforme si necessaire
  for(i in 1:dim_x){
    if (lois[i]==0 & paramlois[1,i]==0 & paramlois[2,i]==0){
      if (logx[i]==FALSE){
        paramlois[1,i]<-min(x[,i])
        paramlois[2,i]<-max(x[,i])}
      else{
        paramlois[1,i]<-exp(min(x[,i]))
        paramlois[2,i]<-exp(max(x[,i]))}}}


#--------------------------
#Boucle sur les parametres |
#--------------------------

  for (i in 1:dim_x){

    x <- samplingLHS(dim_x=1,N=R,lois=lois,paramlois=paramlois) 
    x <- sort(x)
	# solution de facilite, on veut juste prendre les quantiles des lois

    for (k in 1:R){ # boucle sur la discretisation
 
      if (LHS==FALSE) X1<-samplingSimple(dim_x=dim_x,N=S,lois=lois,paramlois=paramlois)
      else X1<-samplingLHS(dim_x=dim_x,N=S,lois=lois,paramlois=paramlois)
	# solution de facilite, on a besoin d'un LHS de dimension dim_x-1

      X1[,i] <- x[k] # on fixe la valeur du parametre Xi

      for (j in 1:dim_x) if (logx[j]==TRUE) X1[,j]=log(X1[,j])

#-------------------------------
#Calcul de la moyenne de y en k |
#-------------------------------

      if (methode>0) pred <- predictSR(modele,X1,methode)
      else pred <- f(X1,...)
      if (logy) pred <- exp(pred)

      EE[k,i] <<- mean(pred)
    }
  }

# Graphe

  x11()
  matplot(xc,EE,type="l",lty=1:dim_x,main="Elementary effects")
  legend(x=1,y=max(EE),lty=c(1:dim_x),col=c(1:dim_x),legend=c(1:dim_x))

}
