leaveoneout<-function(x,y,methode=1,formule,famille=gaussian,
   iteration=FALSE,niter=1,ncomp=1,c=1,eps=0.1,g=5,
   n.trees=25000,interaction.depth=2,ntree=2000,nunits=1,
   maxitRN=1000, min.nugget=0,simplex.ntries=5,constantPG=0,
   nugget.estim=T,optim.method="BFGS",
   logy=FALSE,affiche=TRUE){

#--------------------------------------------------
#Fonction permettant d'effectuer le leave-one-out  |
#sur n'importe quelle methode                      |
#--------------------------------------------------|
#Arguments                                         |
#obligatoires : x :la matrice des variables d'entre|
#               y : la variable de sortie          |
#                                                  |
#Arguments                                         |
#facultatifs :                                     |
#   methode : numero de la methode                 |
#     1=GLM, 2=PLS, 3=SVM, 4=BOOSTING, 5=RF, 6=RN, |
#     7=GAM, 8=PG, 9=PGdice                        |
#   formule : la formule utilisee                  |
#                       (GLM, GAM, PLS, PGdice)    |
#   famille : pour GLM et GAM, la fonction         |
#     de densite de y est de type "famille". Par   |
#     defaut, on suppose qu'elle est gaussienne    |
#   iteration : indique si on choisit le nombre    | 
#     d'iterations dans la regression. Si iteration|
#     = FALSE, le programme effectue les iterations|
#     jusqu'a ce qu'il y ait convergence           |
#   niter : indique le nombre maximal d'iterations |
#     a effectuer                                  |
#   ncomp : nb de composantes dans modele PLS      |
#   c, eps, g: les parametres des SVM              |
#   n.trees,interaction.depth: parametres boosting |
#     (nombre d'arbres et ordre des interactions)  |
#   ntree : nb d'arbres de la foret aleatoire      |
#   maxitRN : nombre max. d'iterations pour le RN  |
#   nunits : nb de noeuds dans la couche du RN     |
#   min.nugget : valeur min de la pepite pour PG   |
#   simplex.ntries : nb d'optim par simplex pour PG|
#   constantPG : 0 (defaut) = partie reg lineair PG|
#                1 : partie deter constante pour PG|
#   nugget.estim : T (defaut) =pepite estime PGdice|
#   optim.method : "BFGS" (defaut) : optim PGdice  |
#                  "gen" : algo genetique rgenoud  |
#   logy : indique si la variable de sortie est    |
#      passee en logarithme                        |
#   affiche : si TRUE, affiche le graphe du        |
#      leave-one-out                               |
#-------------------------------------------------------------|
#Sortie : les criteres de qualite : R2, R2 ajuste, MSE, NMSE, |
#         SMSE, SSD, Biais, Biais normalise, Residu max,Residu|
#         relatif en moyenne, Residu relatif en ecart type    |
#         pour l'apprentissage et la prediction               |
#                                                             |
#         la fonction trace pour chaque point du leave-one-out|
#         la valeur predite / la valeur observee              |
#-------------------------------------------------------------|
#      Auteur : B. Iooss                                      |
#-------------------------------------------------------------|  

#------------------------------------------------
#Initialisations necessaires avant de commencer |
#------------------------------------------------
	
#Initialisation des vecteurs de criteres de qualite
#Les noms sont de la forme : "sigle du critere" + "v" pour vectoriel

r2v<-c();r2av<-c();msev<-c();nmsev<-c();smsev<-c();ssdv<-c();nvsv<-c()
bv<-c();bnv<-c();rmv<-c();mrrv<-c();srrv<-c();r2va<-c();r2ava<-c();mseva<-c()
nmseva<-c();smseva<-c();ssdva<-c();nvsva<-c();bva<-c();bnva<-c();rmva<-c()
mrrva<-c();srrva<-c()

#Initialisation des vecteurs yareel et yappr
#Ces vecteurs contiendront les valeurs observees et predites
#des echantillons d'apprentissage
yareel<-c()
yappr<-c()

#Initialisation des vecteurs yreel et ypred
#Ces vecteurs contiendront les valeurs observees et predites
#des echantillons de test
yreel<-c()
ypred<-c()

#-------------------
#Boucle principale |
#-------------------
l=0
s=0

for (i in 1:dim(x)[1]){
    
#Formation des echantillons de test et d'apprentissage
  index <- ((1:dim(x)[1]) == i)
  xa<-x[!index,]
  xt<-matrix(x[index,],nr=1,nc=dim(xa)[[2]],dimnames=list(c(1),dimnames(xa)[[2]]))
  yt<-y[index]
  ya<-y[!index]

# Methode GLM
  if (methode==1){
    if(iteration==FALSE){modele<-glm(formule,famille,data=data.frame(xa,y=ya))}
    else{modele<-glm(formule,famille,data=data.frame(xa,y=ya),maxit=niter)}
    l<-length(modele$coefficients)
  }    
  else l<-dim(x)[2] # dimension des entrees pour criteres (R2 ajuste)

#Methode PLS
  if (methode==2){
    modele<-plsr(formule,ncomp=ncomp,data=data.frame(xa,y=ya),method="kernelpls")
    l <- length(modele$coefficients)-1
  }

#Methode SVM
  if (methode==3){
    if (is.data.frame(xa)==TRUE){ modele<-svm(ya~.,data=xa,kernel="radial",cost=c,epsilon=eps,gamma=g)}
    else{ modele<-svm(xa,ya,kernel="radial",cost=c,epsilon=eps,gamma=g)}}
    
#Methode BOOSTING
  if (methode==4) modele<-gbm.fit(xa,ya,distribution="gaussian",n.trees=n.trees,interaction.depth=interaction.depth,verbose=FALSE)
  
#Methode FORETS ALEATOIRES
  if (methode==5) modele<-randomForest(xa,ya,ntree=ntree)
  
#Methode RESEAU de NEURONES
  if (methode==6) modele<-nnet(xa,ya,linout=TRUE,size=nunits,maxit=maxitRN,MaxNWts=nunits*nunits*dim(x)[2])
    
#Methode GAM
  if (methode==7){
    modele<-gam(formule,famille,data=data.frame(xa,y=ya))
    l <- dim(as.data.frame(modele$coefficients))[1]
  }

#Methode PROCESSUS GAUSSIEN
  if (methode==8) modele<-mlegp(xa,ya,constantMean=constantPG,min.nugget=min.nugget,simplex.ntries=simplex.ntries)
  
#Methode PGdice
  ys <- y
  y <- ya # car km marche qu'avec y
  if (methode==9) modele<-km(formule,as.data.frame(xa),as.data.frame(y),covtype="powexp",nugget.estim=nugget.estim,optim.method=optim.method)
  y <- ys
  
#Prediction
  pred<-predictSR(modele,xt,methode)
  appr<-predictSR(modele,xa,methode)

# **********************************************
#Calcul des criteres de qualite (base d'apprentissage)
  if(logy==TRUE){resultat <- criteres(exp(ya),exp(appr),dim_x=l)}
  else{resultat <- criteres(ya,appr,dim_x=l)}

  r2va<-c(r2va,resultat[1])
  r2ava<-c(r2ava,resultat[2])
  mseva<-c(mseva,resultat[3])
  nmseva<-c(nmseva,resultat[4])
  smseva<-c(smseva,resultat[5])
  ssdva<-c(ssdva,resultat[6])
  bva<-c(bva,resultat[7])
  bnva<-c(bnva,resultat[8])
  rmva<-c(rmva,resultat[9]) 
  mrrva<-c(mrrva,resultat[10])
  srrva<-c(srrva,resultat[11])

#Calcul de yreel et ypred
  yreel<-c(yreel,yt)
  ypred<-c(ypred,pred)

#Calcul de yareel et yappr
  yareel<-c(yareel,ya)
  yappr<-c(yappr,appr)
}
#------------------
#Fin de la boucle |
#------------------

#Calcul des criteres de qualite (base de test)
if(logy==TRUE){resultat <- criteres(exp(yreel),exp(ypred),dim_x=l)}
else{resultat <- criteres(yreel,ypred,dim_x=l)}

r2v<-c(r2v,resultat[1])
r2av<-c(r2av,resultat[2])
msev<-c(msev,resultat[3])
nmsev<-c(nmsev,resultat[4])
smsev<-c(smsev,resultat[5])
ssdv<-c(ssdv,resultat[6])
bv<-c(bv,resultat[7])
bnv<-c(bnv,resultat[8])
rmv<-c(rmv,resultat[9]) 
mrrv<-c(mrrv,resultat[10])
srrv<-c(srrv,resultat[11])
 
#--------------------------------------------------------------
#Trace des valeurs predites en fonction des valeurs observees |
#--------------------------------------------------------------

if(affiche==TRUE){

#Calcul de l'intervalle dans lequel on trace la courbe pour le test
  mt=min(c(yreel,ypred))
  Mt=max(c(yreel,ypred))

  if(logy==TRUE){

#Courbe de test
    par(mfg=c(1,2))
    plot(yreel,yreel,col=2,type="l",main=("leave-one-out"),sub=("prediction"),xlab="log(observation)",ylab="log(prediction)",ylim=c(mt,Mt))
    points(yreel,ypred)}

  else{

#Courbe de test
    par(mfg=c(1,2))
    plot(yreel,yreel,col=2,type="l",main=("leave-one-out"),sub=("prediction"),xlab="observation",ylab="prediction",ylim=c(mt,Mt))
    points(yreel,ypred)
}}

#Renvoi des criteres de qualite pour le test et l'apprentissage

return(c(mean(r2v),mean(r2av),mean(msev),mean(nmsev),mean(smsev),mean(ssdv),mean(bv),mean(bnv),mean(rmv),mean(mrrv),mean(srrv),mean(r2va),mean(r2ava),mean(mseva),mean(nmseva),mean(smseva),mean(ssdva),mean(bva),mean(bnva),mean(rmva),mean(mrrva),mean(srrva),min(r2v),min(r2av),min(msev),min(nmsev),min(smsev),min(ssdv),min(bv),min(bnv),min(rmv),min(mrrv),min(srrv),min(r2va),min(r2ava),min(mseva),min(nmseva),min(smseva),min(ssdva),min(bva),min(bnva),min(rmva),min(mrrva),min(srrva),max(r2v),max(r2av),max(msev),max(nmsev),max(smsev),max(ssdv),max(bv),max(bnv),max(rmv),max(mrrv),max(srrv),max(r2va),max(r2ava),max(mseva),max(nmseva),max(smseva),max(ssdva),max(bva),max(bnva),max(rmva),max(mrrva),max(srrva),sqrt(var(r2v)),sqrt(var(r2av)),sqrt(var(msev)),sqrt(var(nmsev)),sqrt(var(smsev)),sqrt(var(ssdv)),sqrt(var(bv)),sqrt(var(bnv)),sqrt(var(rmv)),sqrt(var(mrrv)),sqrt(var(srrv)),sqrt(var(r2va)),sqrt(var(r2ava)),sqrt(var(mseva)),sqrt(var(nmseva)),sqrt(var(smseva)),sqrt(var(ssdva)),sqrt(var(bva)),sqrt(var(bnva)),sqrt(var(rmva)),sqrt(var(mrrva)),sqrt(var(srrva)),1))

}
