maformule<-function(dimx,xnom="V",ynom="y",
                    intercept=TRUE,
		    a1=rep("no",dimx),
                    a2=rep("no",dimx),
                    a3=rep("no",dimx),
		    a4=rep("no",dimx),
                    b1=array("no",dim=c(dimx,dimx)),
		    c1=array("no",dim=c(dimx,dimx,dimx)),
		    c2=array("no",dim=c(dimx,dimx)),
		    d1=array("no",dim=c(dimx,dimx,dimx,dimx)),
		    d2=array("no",dim=c(dimx,dimx)),
		    d3=array("no",dim=c(dimx,dimx,dimx)),
		    d4=array("no",dim=c(dimx,dimx)),
                    agam=rep("no",dimx),
                    bgam=array("no",dim=c(dimx,dimx)),
                    cgam=array("no",dim=c(dimx,dimx,dimx)),
                    dgam=array("no",dim=c(dimx,dimx,dimx,dimx))){
		    
#----------------------------------------------------------------
#Fonction permettant de generer une formule de regression a     |
#partir du nombre de parametres dimx.                           |
#                                                               |
#On peut choisir d'y inclure au choix les termes de la forme Vi,|
#Vi^2, Vi^3, Vi^4, ViVj, ViVjVk, Vi^2Vj, ViVjVkVl, Vi^3Vj,      |
#Vi^2VjVk, Vi^2Vj^2                                             |
#                                                               |
#Il est important que la formule obtenue a la fin soit de la    |
#forme :                                                        |
#y ~ 1 + V1 + ... + I(V1^2) + ...                               |
#sinon les fonctions glm et predict.glm utilisees dans "cross", |
#"bootstrap" et "smurfer" ne marcheront pas                     |
#                                                               |
#---------------------------------------------------------------|
#Argument                                                       |
#obligatoire : dimx : la dimension des variables d'entree       |
#                                                               |
#Arguments                                                      |
#facultatifs : xnom : le "nom" des parametres d'entree          |
#              ynom : le "nom" de la variable de sortie y       |
#              intercept : si FALSE, la formule de regression   |
#              ne contiendra pas de constante                   |
#              a1 : terme servant a introduire les Vi           |
#              a2 : terme servant a introduire les Vi^2         |
#              a3 : terme servant a introduire les Vi^3         |
#              a4 : terme servant a introduire les Vi^4         |
#              b1 : terme servant a introduire les ViVj         |
#              c1 : terme servant a introduire les ViVjVk       |
#              c2 : terme servant a introduire les Vi^2Vj       |
#              d1 : terme servant a introduire les ViVjVkVl     |
#              d2 : terme servant a introduire les Vi^3Vj       |
#              d3 : terme servant a introduire les Vi^2VjVk     |
#              d4 : terme servant a introduire les Vi^2Vj^2     |
#              agam : terme pour introduire les s(Vi)           |
#              bgam : terme pour introduire les s(Vi,Vj)        |
#              cgam : terme pour introduire les s(Vi,Vj,Vk)     |
#              dgam : terme pour introduire les s(Vi,Vj,Vk,Vl)  |
#                                                               |
#Par defaut, la formule obtenue sera :                          |
#y ~ 1                                                          | 
#---------------------------------------------------------------|
#Sortie : un objet de type formule                              |
#---------------------------------------------------------------|
#      Auteur : B. Iooss                                        |
#---------------------------------------------------------------|  
#Pour inclure une constante
if(intercept==TRUE){f=paste(ynom,"~1+")}
else{f=paste(ynom,"~0+")}

#Boucle pour prendre les Vi
for(i in 1:dimx){
   if (a1[i]=="yes"){
      f=paste(f,xnom,as.character(i),"+",sep="")
   }
   else f=f
}

#Boucle pour prendre les s(Vi)
for(i in 1:dimx){
   if (agam[i]=="yes"){
      f=paste(f,"s(",xnom,as.character(i),")+",sep="")
   }
   else f=f
}

#Boucle pour prendre les Vi^2
for(i in 1:dimx){
   if (a2[i]=="yes"){
      f=paste(f,"I(",xnom,as.character(i),"^2)","+",sep="")
   }
   else f=f
}

#Boucle pour prendre les Vi^3
for(i in 1:dimx){
   if (a3[i]=="yes"){
      f=paste(f,"I(",xnom,as.character(i),"^3)","+",sep="")
   }
   else f=f
}

#Boucle pour prendre les Vi^4
for(i in 1:dimx){
   if (a4[i]=="yes"){
      f=paste(f,"I(",xnom,as.character(i),"^4)","+",sep="")
   }
   else f=f
}

#Boucle pour prendre les ViVj
if(dimx>1){for(i in 1:(dimx-1)){
              for(j in (i+1):dimx){
                 if (b1[i,j]=="yes"){
                    f=paste(f,xnom,as.character(i),":",xnom,as.character(j),"+",sep="")
                 }
                 else f=f
              }
	   }
}

#Boucle pour prendre les s(Vi,Vj)
if(dimx>1){for(i in 1:(dimx-1)){
               for(j in (i+1):dimx){
                 if (bgam[i,j]=="yes"){
                    f=paste(f,"s(",xnom,as.character(i),",",xnom,as.character(j),")+",sep="")
                  }
                 else f=f
               }
             }
         }

#Boucle pour prendre les ViVjVk
if(dimx>2){for(i in 1:(dimx-2)){
              for(j in (i+1):(dimx-1)){
                 for(k in (j+1):dimx){
                    if (c1[i,j,k]=="yes"){
	               f=paste(f,xnom,as.character(i),":",xnom,as.character(j),":",xnom,as.character(k),"+",sep="")
                    }
	            else f=f
                 }
              }
           }
}

#Boucle pour prendre les s(Vi,Vj,Vk)
if(dimx>2){for(i in 1:(dimx-2)){
              for(j in (i+1):(dimx-1)){
                 for(k in (j+1):dimx){
                    if (cgam[i,j,k]=="yes"){
	               f=paste(f,"s(",xnom,as.character(i),",",xnom,as.character(j),",",xnom,as.character(k),")+",sep="")
                    }
	            else f=f
                 }
              }
           }
         }

#Boucle pour prendre les Vi^2Vj
if(dimx>1){for(i in 1:dimx){
              for(j in 1:dimx){
                 if(c2[i,j]=="yes"){
                   if (i==j){
	           f=f
	           }
	           else f=paste(f,"I(",xnom,as.character(i),"^2):",xnom,as.character(j),"+",sep="")
                 }
                 else f=f
              }
           }
}

#Boucle pour prendre les ViVjVkVl
if(dimx>3 & dimx<40){for(i in 1:(dimx-3)){
              for(j in (i+1):(dimx-2)){
                 for(k in (j+1):(dimx-1)){
                    for(l in (k+1):dimx){
	               if (dgam[i,j,k,l]=="yes"){
		          f=paste(f,xnom,as.character(i),":",xnom,as.character(j),":",xnom,as.character(k),":",xnom,as.character(l),"+",sep="")
                       }
	               else f=f
	            }
                 }
              }
           }
}

#Boucle pour prendre les s(Vi,Vj,Vk,Vl)
if(dimx>3 & dimx<40){for(i in 1:(dimx-3)){
              for(j in (i+1):(dimx-2)){
                 for(k in (j+1):(dimx-1)){
                    for(l in (k+1):dimx){
	               if (d1[i,j,k,l]=="yes"){
		          f=paste(f,"s(",xnom,as.character(i),",",xnom,as.character(j),",",xnom,as.character(k),",",xnom,as.character(l),")+",sep="")
                       }
	               else f=f
	            }
                 }
              }
           }
                   }

#Boucle pour prendre les Vi^3Vj
if(dimx>1){for(i in 1:dimx){
              for(j in 1:dimx){
                 if (d2[i,j]=="yes"){
                    if (i==j){
	               f=f
	            }
	            else f=paste(f,"I(",xnom,as.character(i),"^3):",xnom,as.character(j),"+",sep="")
                 }
                 else f=f
              }
           }
}

#Boucle pour prendre les Vi^2VjVk
if(dimx>2){for(i in 1:dimx){
              for(j in 1:(dimx-1)){
                 for(k in (j+1):dimx){
	            if (d3[i,j,k]=="yes"){
	               if (i==j|i==k){
	                  f=f
	               }
	               else f=paste(f,"I(",xnom,as.character(i),"^2):",xnom,as.character(j),":",xnom,as.character(k),"+",sep="")
                    }
	            else f=f
                 }
              }
           }
}

#Boucle pour prendre les Vi^2Vj^2
if(dimx>1){for(i in 1:(dimx-1)){
              for(j in (i+1):dimx){
                 if (d4[i,j]=="yes"){
                    f=paste(f,"I(",xnom,as.character(i),"^2):I(",xnom,as.character(j),"^2)+",sep="")
                 }
                 else f=f
              }
	   }
}

#Suppression du "+" a la fin de la formule
f=substr(f,1,nchar(f)-1)

#Transformation en formule et renvoi
return(as.formula(f))

}
