moyenne<-function(x,nom=c(NA)){

#-----------------------------------------------------------------------
#Fonction calculant la moyenne, l'ecart-type, le minimum et le maximum |
#des colonnes de x                                                     |
#----------------------------------------------------------------------|
#Argument                                                              |
#obligatoire : x : matrice des variables d'entree                      |
#                                                                      |
#Argument                                                              |
#facultatif : nom : vecteur des noms des variables (6 caracteres)      |
#----------------------------------------------------------------------|
#Sortie : la moyenne, l'ecart-type, le minimum et le maximum des       |
#         colonnes de la matrice sont affiches dans un fichier nomme   |
#         res-MOYENNE.out                                              |
#----------------------------------------------------------------------|
#      Auteur : B. Iooss et A. Ducellier                               |
#----------------------------------------------------------------------|  

#Cacul des dimensions de x
p<-dim(x)[1]
n<-dim(x)[2]

#Initialisations
moyenne<-rep(0,n)
ecarttype<-rep(0,n)
minimum<-rep(0,n)
maximum<-rep(0,n)

#Boucle
for(i in 1:n){

#Calcul de la moyenne
moyenne[i]=mean(x[,i])

#Calcul de l'ecart-type
ecarttype[i]=sqrt((sum((x[,i]-mean(x[,i]))^2))/(p-1))

#Calcul du minimum
minimum[i]=min(x[,i])

#Calcul du maximum
maximum[i]=max(x[,i])

}

#Mise en forme pour l'affichage
A<-c(moyenne,ecarttype,minimum,maximum)
A<-format(A,digits=6)

moyenne<-A[1:n]
ecarttype<-A[(n+1):(2*n)]
minimum<-A[(2*n+1):(3*n)]
maximum<-A[(3*n+1):(4*n)]

#Creation du fichier de resultats
FOUT <- file("res-MOYENNE.out","w")

#Affichage si le nom n'est pas defini
if(is.na(nom[1])==TRUE){

write("--------------------------------",file=FOUT)
write("| Statistiques sur les donnees |",file=FOUT)
write("--------------------------------",file=FOUT)
write(" ",file=FOUT)
write(c("       Moyenne         Ecart-type      Minimum         Maximum"),file=FOUT)
write(" ",file=FOUT)

for(i in 1:n){
write(c(dimnames(x)[[2]][i],"   ",moyenne[i],"   ",ecarttype[i],"   ",minimum[i],"   ",maximum[i]),ncolumns=9,file=FOUT)
write(" ",file=FOUT)
}

}

#Affichage si le nom est defini
else{

write("--------------------------------",file=FOUT)
write("| Statistiques sur les donnees |",file=FOUT)
write("--------------------------------",file=FOUT)
write(" ",file=FOUT)
write(c("           Moyenne         Ecart-type      Minimum         Maximum"),file=FOUT)
write(" ",file=FOUT)

for(i in 1:n){
write(c(nom[i],"   ",moyenne[i],"   ",ecarttype[i],"   ",minimum[i],"   ",maximum[i]),ncolumns=9,file=FOUT)
write(" ",file=FOUT)
}

}

#Fermeture du fichier de resultats
close(FOUT)

# Affichage du fichier de resultat
if (Sys.getenv("OS")!="Windows_NT"){ pager_OS="cat"}
else{ pager_OS=""}

file.show("res-MOYENNE.out",pager=pager_OS)

}
