samplingSimplecontraint<-function(dim_x,nom=c(NA),N=1,lois=rep(0,dim_x),
                  paramlois=array(0,dim=c(4,dim_x)),contraint=rep(0,dim_x),
                  tronq=rep(F,dim_x),paramtronq=array(0,dim=c(2,dim_x))){

#---------------------------------------------------------------------------|
#Tirage aleatoire simple de N jeux de parametres avec prise en compte de    |
#contraintes entre eux (croissance ou decroissance)                         |
#  Petelet et al., 2009. http://fr.arxiv.org/abs/0909.0329                  |
#---------------------------------------------------------------------------|
#Arguments obligatoires :                                                   |
#               dim_x : nombre de parametres d'entree dans le modele        |
#                                                                           |
#Arguments facultatifs :                                                    |
#             nom : vecteur des noms des parametres simules                 |
#               (defini comme : c("V1","V2","V3",...))                      |
#             N : nombre de jeux de simulations                             |
#             lois : vecteur contenant les types de distribution de proba   |
#               pour chaque entree                                          |
#               0=uniforme ; 1=normale ; 2=lognormale ; 3=weibull           | 
#               4=exponentielle ; 5=beta ; 6=triangulaire ; 7=trapezoidale  |
#               10=gumbel                                                   |
#               Par defaut, on prend la loi uniforme                        |
#             paramlois : tableau avec les parametres de chaque loi (max=4) |
#               pour chaque entree (range par colonne) :                    |
#               (min,max,0,0) pour uniforme (par dfaut : min=0, max=1)     |
#               (moy,ecart-type,0,0) pour normale,                          |
#               (moy du log, ecart-type du log,0,0) pour lognormale,        |
#               (forme,echelle,0,0) pour Weibull,                           |
#               (lambda,0,0,0) pour exponentielle,                          |
#               (shape1, shape2,0,0) pour beta,                             |
#               (min,mode,max,0) pour triangulaire,                         |
#               (min,mode1,mode2,max) pour trapezoidale,                    |
#               (mode,echelle,0,0) pour Gumbel                              |
#             tronq : vecteur pour selectionner ou non une loi tronquee     |
#               TRUE pour loi tronquee, FALSE sinon (par dfaut)            |
#             paramtronq : tableau avec les parametres de troncature de     |
#               chaque loi : (min,max) range par colonne                    |
#             contraint : vecteur contenant les types de contraintes a      |
#               imposer pour chaque Vi                                      |
#               0=tirage normal ; 1=contrainte de croissance ; -1=contrainte|
#               de decroissance.                                            |
#               Par defaut, on realise un tirage normal.                    |
#---------------------------------------------------------------------------|
#Sortie : la matrice des N simulations des dim_x parametres                 |
#---------------------------------------------------------------------------|
#      Auteur : B. Iooss                                                    |
#---------------------------------------------------------------------------|

# !!! WARNING !!!! : la troncature et les contraintes ne s'appliquent pas 
#                    a la loi trapezoidale

# *********************************************************************
# FONCTION UTILISEE :
#      truncated.R (fonctions pour lois tronquees)
# *********************************************************************
# LIBRAIRIES REQUISES (A INSTALLER)
  library(triangle)
  library(evd) # Gumbel
# *********************************************************************

  for(i in 1:dim_x){ # si le nom du x n'est pas defini
    if(is.na(nom[i])==TRUE) nom[i]<-paste("V",i,sep="")}
  
# on definit les matrices et vecteurs
  x<-matrix(0,nr=N,nc=dim_x,dimnames=list(1:N,nom))
  min_c <- rep(NA,N)
  max_c <- rep(NA,N)
	
#on complete les bornes des lois uniformes
  for (i in 1:dim_x) {
	if (lois[i]==0 && paramlois[1,i]>=paramlois[2,i]) paramlois[2,i]<-1
	else if (lois[i]==6 && paramlois[1,i]>=paramlois[3,i]) {
		paramlois[2,i]<-0.5
		paramlois[3,i]<-1
	}
	else if (lois[i]==7 && paramlois[1,i]>=paramlois[4,i]) {
		paramlois[2,i]=0.25
		paramlois[3,i]=0.75
		paramlois[4,i]=1
	}}

# la 1ere variable ne peut pas etre contrainte
  contraint[1] <- 0

  for(i in 1:dim_x){

# prise en compte des contraintes pour tronquer les lois
    if (contraint[i] == 1) min_c <- x[,i-1]
    if (contraint[i] == -1) max_c <- x[,i-1]

    if (lois[i]==0){ # loi uniforme
      if (tronq[i]==F){
        if (contraint[i]==0) x[,i] <- runif(N,min=paramlois[1,i],max=paramlois[2,i])
        if (contraint[i]==1) 
          for (j in 1:N) x[j,i] <-  r.trunc.distr(1,'unif',min=paramlois[1,i],max=paramlois[2,i],trunc.int=c(min_c[j],1e12))
        if (contraint[i]==-1)
          for (j in 1:N) x[j,i] <-  r.trunc.distr(1,'unif',min=paramlois[1,i],max=paramlois[2,i],trunc.int=c(-1e9,max_c[j]))}
      else{
        for (j in 1:N) 
          x[j,i] <-  r.trunc.distr(1,'unif',min=paramlois[1,i],max=paramlois[2,i],trunc.int=c(max(paramtronq[1,i],min_c[j],na.rm=T),max=min(paramtronq[2,i],max_c[j],na.rm=T)))}}

    if (lois[i]==1){ # loi normale
      if (tronq[i]==F){
        if (contraint[i]==0) x[,i] <- rnorm(N,mean=paramlois[1,i],sd=paramlois[2,i])
        if (contraint[i]==1) 
          for (j in 1:N) x[j,i] <-  r.trunc.distr(1,'norm',mean=paramlois[1,i],sd=paramlois[2,i],trunc.int=c(min_c[j],1e12))
        if (contraint[i]==-1)
          for (j in 1:N) x[j,i] <-  r.trunc.distr(1,'norm',mean=paramlois[1,i],sd=paramlois[2,i],trunc.int=c(-1e9,max_c[j]))}
      else{
        for (j in 1:N) 
          x[j,i] <-  r.trunc.distr(1,'norm',mean=paramlois[1,i],sd=paramlois[2,i],trunc.int=c(max(paramtronq[1,i],min_c[j],na.rm=T),max=min(paramtronq[2,i],max_c[j],na.rm=T)))}}

    if (lois[i]==2){ # loi lognormale
      if (tronq[i]==F){
        if (contraint[i]==0) x[,i] <- rlnorm(N,meanlog=paramlois[1,i],sdlog=paramlois[2,i])
        if (contraint[i]==1) 
          for (j in 1:N) x[j,i] <-  r.trunc.distr(1,'lnorm',meanlog=paramlois[1,i],sdlog=paramlois[2,i],trunc.int=c(min_c[j],1e12))
        if (contraint[i]==-1)
          for (j in 1:N) x[j,i] <-  r.trunc.distr(1,'lnorm',meanlog=paramlois[1,i],sdlog=paramlois[2,i],trunc.int=c(-1e9,max_c[j]))}
      else{
        for (j in 1:N) 
          x[j,i] <-  r.trunc.distr(1,'lnorm',meanlog=paramlois[1,i],sdlog=paramlois[2,i],trunc.int=c(max(paramtronq[1,i],min_c[j],na.rm=T),max=min(paramtronq[2,i],max_c[j],na.rm=T)))}}

    if (lois[i]==3){ # loi weibull
      if (tronq[i]==F){
        if (contraint[i]==0) x[,i] <- rweibull(N,shape=paramlois[1,i],scale=paramlois[2,i])
        if (contraint[i]==1) 
          for (j in 1:N) x[j,i] <-  r.trunc.distr(1,'weibull',shape=paramlois[1,i],scale=paramlois[2,i],trunc.int=c(min_c[j],1e12))
        if (contraint[i]==-1)
          for (j in 1:N) x[j,i] <-  r.trunc.distr(1,'weibull',shape=paramlois[1,i],scale=paramlois[2,i],trunc.int=c(-1e9,max_c[j]))}
      else{
        for (j in 1:N) 
          x[j,i] <-  r.trunc.distr(1,'weibull',shape=paramlois[1,i],scale=paramlois[2,i],trunc.int=c(max(paramtronq[1,i],min_c[j],na.rm=T),max=min(paramtronq[2,i],max_c[j],na.rm=T)))}}

    if (lois[i]==10){ # loi gumbel
      if (tronq[i]==F){
        if (contraint[i]==0) x[,i] <- rgumbel(N,loc=paramlois[1,i],scale=paramlois[2,i])
        if (contraint[i]==1) 
          for (j in 1:N) x[j,i] <-  r.trunc.distr(1,'gumbel',loc=paramlois[1,i],scale=paramlois[2,i],trunc.int=c(min_c[j],1e12))
        if (contraint[i]==-1)
          for (j in 1:N) x[j,i] <-  r.trunc.distr(1,'gumbel',loc=paramlois[1,i],scale=paramlois[2,i],trunc.int=c(-1e9,max_c[j]))}
      else{
        for (j in 1:N) 
          x[j,i] <-  r.trunc.distr(1,'gumbel',loc=paramlois[1,i],scale=paramlois[2,i],trunc.int=c(max(paramtronq[1,i],min_c[j],na.rm=T),max=min(paramtronq[2,i],max_c[j],na.rm=T)))}}

    if (lois[i]==4){ # loi exponentielle
      if (tronq[i]==F){
        if (contraint[i]==0) x[,i] <- rexp(N,rate=paramlois[1,i])
        if (contraint[i]==1) 
          for (j in 1:N) x[j,i] <-  r.trunc.distr(1,'exp',rate=paramlois[1,i],trunc.int=c(min_c[j],1e12))
        if (contraint[i]==-1)
          for (j in 1:N) x[j,i] <-  r.trunc.distr(1,'exp',rate=paramlois[1,i],trunc.int=c(-1e9,max_c[j]))}
      else{
        for (j in 1:N) 
          x[j,i] <-  r.trunc.distr(1,'exp',rate=paramlois[1,i],trunc.int=c(max(paramtronq[1,i],min_c[j],na.rm=T),max=min(paramtronq[2,i],max_c[j],na.rm=T)))}}

    if (lois[i]==5){ # loi beta
      if (tronq[i]==F){
        if (contraint[i]==0) x[,i] <- rbeta(N,shape1=paramlois[1,i],shape2=paramlois[2,i])
        if (contraint[i]==1) 
          for (j in 1:N) x[j,i] <-  r.trunc.distr(1,'beta',shape1=paramlois[1,i],shape2=paramlois[2,i],trunc.int=c(min_c[j],1e12))
        if (contraint[i]==-1)
          for (j in 1:N) x[j,i] <-  r.trunc.distr(1,'beta',shape1=paramlois[1,i],shape2=paramlois[2,i],trunc.int=c(-1e9,max_c[j]))}
      else{
        for (j in 1:N) 
          x[j,i] <-  r.trunc.distr(1,'beta',shape1=paramlois[1,i],shape2=paramlois[2,i],trunc.int=c(max(paramtronq[1,i],min_c[j],na.rm=T),max=min(paramtronq[2,i],max_c[j],na.rm=T)))}}

    if (lois[i]==6){ # loi triangulaire
      if (tronq[i]==F){
        if (contraint[i]==0) x[,i] <- rtriangle(N,a=paramlois[1,i],b=paramlois[3,i],c=paramlois[2,i])
        if (contraint[i]==1) 
          for (j in 1:N) x[j,i] <-  r.trunc.distr(1,'triangle',a=paramlois[1,i],b=paramlois[3,i],c=paramlois[2,i],trunc.int=c(min_c[j],1e12))
        if (contraint[i]==-1)
          for (j in 1:N) x[j,i] <-  r.trunc.distr(1,'triangle',a=paramlois[1,i],b=paramlois[3,i],c=paramlois[2,i],trunc.int=c(-1e9,max_c[j]))}
      else{
        for (j in 1:N) 
          x[j,i] <-  r.trunc.distr(1,'triangle',a=paramlois[1,i],b=paramlois[3,i],c=paramlois[2,i],trunc.int=c(max(paramtronq[1,i],min_c[j],na.rm=T),max=min(paramtronq[2,i],max_c[j],na.rm=T)))}}

    if (lois[i]==7) { # loi trapezoidale
      norm<-1/(paramlois[2,i]-paramlois[1,i])
      normp<-1/(paramlois[4,i]-paramlois[3,i])
      j<-1
      while (j<=N) {
        aux<-runif(1,min=max(paramlois[1,i],min_c[j],na.rm=T),max=min(paramlois[4,i],max_c[j],na.rm=T))
        auxp<-runif(1)
        if (aux<=paramlois[2,i]) {
          if (auxp<=norm*(aux-paramlois[1,i])) {
            x[j,i]<-aux
            j<-j+1}}
        else if (aux>=paramlois[3,i]) {
          if (auxp<=normp*(paramlois[4,i]-aux)) {
            x[j,i]<-aux
            j<-j+1}}
        else {
          x[j,i]<-aux
          j<-j+1}}}
  }

  return(x)
}
