# SMURFER Version 1.0 - 10/01/2010

smurfer<-function(x,y,
        methodes=c(rep(TRUE,9)),
        logx=rep(FALSE,dim(x)[2]),logy=FALSE,affiche_file=TRUE,
        affiche=TRUE,loo=FALSE,vc=FALSE,N=10,bootstrap=FALSE,B=50,
        elimination=FALSE,xtest=NULL,ytest=NULL,sobol=FALSE,S=10000,
        R=1,Sij=TRUE,LHS=FALSE,lois=rep(0,dim(x)[2]),
        paramlois=array(0,dim=c(4,dim(x)[2])),
        tronq=rep(F,dim(x)[2]),paramtronq=array(0,dim=c(2,dim(x)[2])),
        cmin=1,cmax=101,cpas=50,epsmin=0.01,epsmax=0.51,epspas=0.25,
        gmin=0.5,gmax=10.5,gpas=5,SVMexplore=TRUE,critere="R2",
        n.trees=25000,interaction.depth=2,ntree=2000,
        maxitRN=1000,nunits=5,
        min.nugget=0,simplex.ntries=5,constantPG=0,
        nugget.estim=T,optim.method="BFGS",
        forme="creationformule",degreformule=1,famille=gaussian,
        stepwise=FALSE,penaltyAIC=2,trace=0,iteration=FALSE,niter=1,
        intercept=TRUE,
        tabA1=rep("no",dim(x)[2]),tabA2=rep("no",dim(x)[2]),
        tabA3=rep("no",dim(x)[2]),tabA4=rep("no",dim(x)[2]),
        tabB1=array("no",dim=c(dim(x)[2],dim(x)[2])),
        tabC1=array("no",dim=c(dim(x)[2],dim(x)[2],dim(x)[2])),
        tabC2=array("no",dim=c(dim(x)[2],dim(x)[2])),
        tabD1=array("no",dim=rep(dim(x)[2],4)),
        tabD2=array("no",dim=c(dim(x)[2],dim(x)[2])),
        tabD3=array("no",dim=c(dim(x)[2],dim(x)[2],dim(x)[2])),
        tabD4=array("no",dim=c(dim(x)[2],dim(x)[2])),
        tabAgam=rep("no",dim(x)[2]),
        tabBgam=array("no",dim=c(dim(x)[2],dim(x)[2])),
        tabCgam=array("no",dim=c(dim(x)[2],dim(x)[2],dim(x)[2])),
        tabDgam=array("no",dim=rep(dim(x)[2],4))
                  ){

#-------------------------------------------------------------------
#Fonction applicant diverses mehodes de regression sur un jeu de    |
#donnees afin de creer une surface de reponse. Les methodes sont :  |
#  GLM (forme polynomiale jusqu'au 4eme degre), regression PLS, SVM,|
#  boosting, forets aleatoires, reseaux de neurones (RN), GAM, PG   |
#Les performances sont evaluees par leave-one-out, validation       |
#croisee et/ou bootstrap                                            |
#                                                                   |
#Les indices de sensibilite (Sobol) peuvent etre calcules a l'aide  |
#des surfaces de reponse.                                           |
#                                                                   |
#-------------------------------------------------------------------|
#Arguments                                                          |
#obligatoires : x : matrice des variables d'entree                  |
#               y : vecteur de la variable de sortie                |
#                                                                   |
#Arguments facultatifs :                                            |
#      methodes: Tableau de booleens indiquant les methodes a       |
#        utiliser ou pas : GLM, PLS, SVM, BOOSTING, RF, RN, GAM, PG |
#                          PGdice                                   |
#      logx : si logx=c(TRUE,TRUE,...,TRUE), le programme           |
#        transformera les variables d'entree Vi en log(Vi)          |
#      logy : si logy=TRUE, le programme transformera la variable   |
#        de sortie y en log(y)                                      |
#      affiche_file : si TRUE, affiche automatiquement les fichiers |
#        de sortie "res-SURFER.out" et res-SOBOL.out" a l'ecran     |
#      affiche : si TRUE, cree les graphes de la validation croisee |
#        et du bootstrap                                            |
#        Sous Unix, gv est lance pour les afficher automatiquement  |
#        Sous Windows, l'affichage n'est pas automatique            |
#      loo : si loo=TRUE, le leave-one-out sera applique pour       |
#        evaluer les criteres de qualite de la surface de reponse   |
#      vc : si vc=FALSE, le programme n'effectuera pas la           |
#        validation croisee                                         |
#      N : nombre de repetitions de la validation croisee           |
#        si N est ngatif, on intervertit les rles des bases de    |
#        test et d'apprentissage (en conservant |N| intervalles)    |
#      bootstrap : si bootstrap=FALSE, le programme n'effectuera    |
#        pas le bootstrap                                           |
#      B : le nombre de repetitions du bootstrap                    |
#      elimination : si TRUE, permet d'eliminer dans le bootstrap   |
#        les echantillons pour lesquels R2 < 0                      |
#      xtest : matrice des variables d'entree de la base de test    |
#      ytest : vecteur de la variable de sortie de la base de test  |
#      sobol : si TRUE, calcul des indices de Sobol                 |
#      S : nombres de simulations effectuees pour calculer les      |
#        indices de sensibilite de Sobol                            |
#      R : nombre de repetitions du calcul des indices de Sobol     |
#      Sij : TRUE pour calculer les indices d'ordre 2, FALSE sinon  |
#      LHS : TRUE pour utiliser un echantillonnage LHS lors du      |
#        calcul des indices de Sobol                                |
#        Par defaut : FALSE pour un echantillonnage simple          |
#      lois : vecteur contenant les types de distribution de proba  |
#        pour chaque entree                                         |
#        0=uniforme ; 1=normale ; 2=lognormale ; 3=weibull          | 
#        4=exponentielle ; 5=beta ; 6=triangulaire ; 7=trapezoidale |
#        10=gumbel                                                  |
#        Par defaut, on prend la loi uniforme                       |
#      paramlois : tableau avec les parametres de chaque loi (max=4)|
#        pour chaque entree (range par colonne) :                   |
#        (min,max,0,0) pour uniforme (par dfaut : min=0, max=1)    |
#        (moy,ecart-type,0,0) pour normale,                         |
#        (moy du log, ecart-type du log,0,0) pour lognormale,       |
#        (forme,echelle,0,0) pour Weibull,                          |
#        (lambda,0,0,0) pour exponentielle,                         |
#        (shape1, shape2,0,0) pour beta,                            |
#        (min,mode,max,0) pour triangulaire,                        |
#        (min,mode1,mode2,max) pour trapezoidale,                   |
#        (mode,echelle,0,0) pour Gumbel                             |
#        Par defaut, on a la loi uniforme avec (min,max) calcules a |
#        partir des parametres d'entree x                           |
#      tronq : vecteur pour selectionner ou non une loi tronquee    |
#        TRUE pour loi tronquee, FALSE sinon (par dfaut)           |
#      paramtronq : tableau avec les parametres de troncature de    |
#        chaque loi : (min,max) range par colonne                   |
#      cmin,cmax,cpas,epsmin,epsmax,epspas,gmin,gmax,gpas :         |
#        les plages d'exploration des SVM : cout, marge et portee   |
#      SVMexplore: Si FALSE, alors on n'explore pas les plages des  |
#        parametres. On prend les valeurs minimales                 |
#      critere: Critere pour la selection des parametres SVM:       |
#        R2,R2ajuste,MSE,NMSE,SMSE,MAE,B,BN,RM,RRM ou RRE           |
#      n.trees, interaction.depth : les parametres du boosting      |
#        (nombre d'arbres et ordre des interactions)                |
#      ntree : nombre d'arbres de la foret aleatoire                |
#      maxitRN : nombre max. d'iterations pour l'apprentissage du RN|
#      nunits : nombre de noeuds dans la couche cachee du RN        |
#      min.nugget : valeur minimum de la pepite pour PG (mlegp)     |
#      simplex.ntries : nb d'optim par simplex pour PG (mlegp)      |
#      constantPG : 0 (defaut) = partie regress lineaire PG (mlegp) |
#                   1 : partie deterministe constante pour PG       |
#      nugget.estim : T (defaut) = effet de pepite estime PGdice    |
#      optim.method : "BFGS" (defaut) : optimisation PGdice par BFGS|
#                     "gen" : algorithme genetique de rgenoud       |
#      forme : sert a choisir la facon dont est cree la formule de  |
#        regression pour les GLM, PLS, GAM, PGdice                  |
#        On peut choisir "creationformule" ou "maformule"           |
#      degreformule : le degre de la formule dans le cas ou on      |
#        choisit "creationformule"                                  |
#      famille : pour GLM et GAM le programme suppose que la        |
#        fonction de densite de y est de type "famille"             |
#        Par defaut, on suppose qu'elle est gaussienne de fonction  |
#         lien l'identite (famille=gaussian(link="identity"))       |
#      stepwise : pour GLM, si TRUE, permet de construire un modele |
#        optimal en eliminant certains termes. Dans ce cas, on ne   |
#        pourra calculer ni les contributions des effets, ni Sobol  |
#        Si stepwise est utilise pour GLM, on ne peut pas appliquer |
#        sobol pour PLS et PGdice (car meme formule)                |
#      penaltyAIC : multiple du nb de degre de liberte pour AIC dans|
#        le stepwise (par defaut = 2 ; si > 2 -> selection + dure   |
#      trace : si trace = 1, le programme affiche les etapes de     |
#        l'algorithme stepwise. Par defaut, trace = 0 et le         |
#        programme n'affiche rien                                   |
#      iteration : indique si on choisit le nombre d'iterations dans|
#        la regression. Si iteration = FALSE, le programme effectue |
#        les iterations jusqu'a ce qu'il y ait convergence          |
#      niter : indique le nombre maximal d'iterations a effectuer   |
#      intercept : si intercept=TRUE, il y a une constante dans la  |
#        formule de regression (GLM, GAM, PGdice) si on a choisi    |
#                                                   "maformule"     |
#      tabA1 : permet d'inclure les Vi dans "maformule"             |
#      tabA2 : permet d'inclure les Vi^2 dans "maformule"           |
#      tabA3 : permet d'inclure les Vi^3 dans "maformule"           |
#      tabA4 : permet d'inclure les Vi^4 dans "maformule"           |
#      tabB1 : permet d'inclure les ViVj dans "maformule"           |
#      tabC1 : permet d'inclure les ViVjVk dans "maformule"         |
#      tabC2 : permet d'inclure les Vi^2Vj dans "maformule"         |
#      tabD1 : permet d'inclure les ViVjVkVl dans "maformule"       |
#      tabD2 : permet d'inclure les Vi^3Vj dans "maformule"         |
#      tabD3 : permet d'inclure les Vi^2VjVk dans "maformule"       |
#      tabD4 : permet d'inclure les Vi^2Vj^2 dans "maformule"       |
#      tabAgam : pour GAM inclut les s(Vi) dans "maformule"         |
#      tabBgam : pour GAM inclut les s(Vi,Vj) dans "maformule"      |
#      tabCgam : pour GAM inclut les s(Vi,Vj,Vk) dans "maformule"   |
#      tabDgam : pour GAM inclut les s(Vi,Vj,Vk,Vl) dans "maformule"|
#                                                                   |
#Par defaut, la formule de regression utilisee par le programme     |
#        sera y ~ 1                                                 |
#-------------------------------------------------------------------|
#Sortie : Pour chaque methode, la fonction renvoie les  parametres  |
#         des modeles obtenus et les criteres de qualite du modele :|
#         R2: coefficicient de determination                        |
#         R2ajuste : R2 non biaise (corrige du nombre de ddl)       |
#           mesure valide pour GLM, PLS et GAM                      |
#         MSE: Mean Square Error                                    |
#         NMSE: Normalized Mean Square Error                        |
#         SMSE: Standardized Mean Square Error                      |
#         MAE: Mean Absolute Error                                  |
#         B: biais,                                                 |
#         BN: biais normalise                                       |
#         RM: residu maximum                                        |
#         RRM: residu relatif en moyenne                            |
#         RRE: residu relatif en ecart-type                         |
#                                                                   |
#         Par leave-one-out, on a des criteres moyens de prediction |
#         et des statistiques pour les criteres d'apprentissage     |
#         La valeur du Q2 est mise dans une variable externe Q2     |
#         Par validation croisee, on a la moyenne, l'ecart-type et  |
#         les min et max des criteres de qualite obtenus pour       |
#         l'apprentissage et la prediction                          |
#         Par bootstrap, on a la moyenne, l'ecart-type et           |
#         l'intervalle de confiance a 90% des criteres de qualite   |
#         pour la prediction                                        |
#         La fct peut calculer les criteres sur une base de test    |
#         Si on n'a pas les resultats de la base de test, les       |
#         fichiers "resGLM-test.dat', "resPLS-test.dat", ...        |
#         contiennent les predictions des modeles sur la base test  |
#                                                                   |
#         La fonction trace egalement les valeurs predites en fct   |
#         des valeurs observees pour les echantillons de test et    |
#         d'apprentissage pour le leave-one-out, la validation      |
#         croisee et le bootstrap                                   |
#                                                                   |
#         Pour le modele GLM, on a en plus :                        |
#            - les coefficients de la regression : Estimation,      |
#              Ecart-type, t-value et Pr(>|t|)                      |
#            - la variance totale et son degre de liberte           |
#            - la variance residuelle et son degre de liberte       |
#            - les contributions des effets et des residus          |
#            - l'indice de conditionnement de la matrice de         |
#              correlation des effets                               |
#            - la moyenne, l'ecart-type et l'intervalle de confiance|
#              a 90% des coefficients de la regression par bootstrap|
#         Pour SVM, si on explore les plages des parametres SVM, la |
#         fct enregistre les criteres (r2m,msem,nmsem,smsem,ssdm,bn,|
#         bnm,rmm,mrrm,srrm) dans le fichier "res-exploreSVM.Rdata" |
#                                                                   |
#         Tous ces resultats sont dans un fichier "res-SURFER.out"  |
#         (fichier ecrase) dans le repertoire de travail            |
#         Les sorties graphiques se trouvent dans les fichiers      |
#         compar*.ps : compar.ps si pas de vc ni de bootstrap,      |
#           sinon compar1.ps, compar2.ps, ..., compar8.ps           |
#                                                                   |
#         De plus, les modeles sont renvoyes dans des objets R      |
#         modeleGLM, modelePLS, modeleSVM, modeleBOOSTING, modeleRF |
#         modeleRN, modeleGAM, modelePG, modelePGdice               |
#         Ceux-ci sont utilisables sous la meme session R           |
#         Avant de quitter la session R, les objets peuvent etre    |
#         sauvegardes dans un fichier a l'aide de la commande "save"|
#         Pour les recuperer dans une autre session, utiliser "load"|
#         Les parametres dim_x, dim1_x, dimnames_x, dimnames1_x     |
#         sont aussi conserves                                      |
#                                                                   |
#         Les resultats des calculs Sobol sont dans le fichier      |
#         "res-SOBOL.out" (fichier ecrase) du repertoire de travail |
#-------------------------------------------------------------------|
#      Auteur : B. Iooss et P-M. Pair                               |
#-------------------------------------------------------------------|

# dechargement du package gam (risque d'interaction avec mgcv)
if ("package:gam" %in% search()) detach(package:gam)

#----------------------------------
#Creation du fichier de resultats 
#---------------------------------- 
FOUT <- file("res-SURFER.out","w")

if (affiche){
   ps.options(horizontal=FALSE)

   if (loo==FALSE & vc==FALSE & bootstrap==FALSE){ # une page par modele
     postscript(file="compar.ps")
     oldpar=par
     par(mfcol=c(3,3))}}
      
#----------------------------------
#Initialisation : sauvegarde de x et y dans des variables auxiliaires
xbak<-x
ybak<-y
dim_x<<-dim(x)[2]
ndata<-dim(x)[1]
dimnames_x<<-dimnames(x)[[2]]

# qualitatif: indique si certaines variables sont de type qualitatif. Dans ce cas, x DOIT etre un
# data.frame ou les colonnes correspondant aux variables qualitatives DOIVENT etre de type factor.
qualitatif=is.data.frame(x);

#-------------------------------------
#Transformation des variables en log 
#-------------------------------------

#Transformation de la matrice x

listnotlog=NULL; cpt=0
for(i in 1:dim_x){ 
  if(logx[i]==TRUE){
    for(j in 1:ndata){ 
      if (x[j,i]<=0){ listnotlog=append(listnotlog,i)
                      logx[i]<-FALSE; cpt=1
                      break}
      if (logx[i]==FALSE) break}}
  if(logx[i]==TRUE){ 
    x[,i]<-log(x[,i])
    if (is.null(xtest)==FALSE){ xtest[,i]<-log(xtest[,i])}}}
if (cpt==1){write(c("Impossible de passer au log les parametres d'entree x suivants :",listnotlog),file=FOUT)}

x<-matrix(x,nr=ndata,nc=dim_x,dimnames=list(1:ndata,dimnames_x))
    
#Transformation de la variable y
if(logy==TRUE){
  for(j in 1:ndata){
    if (y[j]<=0 & logy==TRUE){ write("Impossible de passer au log la variable y",file=FOUT)
		  logy<-FALSE
		  break}}}
if(logy==TRUE){ 
  y<-log(y)
  if (is.null(ytest)==FALSE) ytest<-log(ytest)}

#----------------------------------------------------------------
#Creation de la formule de regression polynomiale (GLM, PLS, PGdice)
#----------------------------------------------------------------
if (methodes[1]==TRUE | methodes[2]==TRUE | methodes[9]==TRUE){

# On transforme les noms des variables en V1, V2, V3, ...
  xnames <- NULL
  for (i in 1:dim_x) xnames <- c(xnames,paste("V",as.character(i),sep=""))
  x <- matrix(c(x),nr=dim(x)[1],nc=dim(x)[2],dimnames=list(1:dim(x)[1],xnames))

  if(forme=="maformule") formule <- maformule(dimx=dim_x,intercept=intercept,a1=tabA1,a2=tabA2,a3=tabA3,a4=tabA4,b1=tabB1,c1=tabC1,c2=tabC2,d1=tabD1,d2=tabD2,d3=tabD3,d4=tabD4)

  else formule <- creationformule(dimx=dim_x,interaction.depth=degreformule)}

#-------------
#----------- |
#Programme | |
#----------- |
#-------------

#---------------------------------------------------------------------
# Evaluation des donnees par la methode GLM
#---------------------------------------------------------------------

if (methodes[1]==TRUE){
#		Par defaut: loi uniforme sur [0,1]                          |

#Initialisation du modele
  modeleGLM<-NaN

#Modele optimal : elimination des termes inutiles par l'algorithme "stepwise"
  if(stepwise==TRUE){
    GLMexhaustif=glm(formule,famille,data=data.frame(x,y))
    modeleGLM<-step(GLMexhaustif,trace=trace,k=penaltyAIC)
    formule<-as.formula(modeleGLM)
    restep<-modeleGLM[[31]]
    restep<-as.matrix(restep)}
  else { 
    if(iteration==FALSE){ modeleGLM<-glm(formule,famille,data=data.frame(x,y))}
    else{ modeleGLM<-glm(formule,famille,data=data.frame(x,y),maxit=niter)}}

# Redefinition a cause du stepwise
  dim1_x <<- length(modeleGLM$coefficients)-1 # nb de parametres du modele
  dimnames1_x <<- names(modeleGLM$coefficients)[2:dim1_x]
  
#Calcul des criteres de qualite
  pred<-predictSR(modeleGLM,data.frame(x),methode=1)
  if(logy==TRUE){resultatcriteres<-criteres(exp(y),exp(pred),dim_x=dim1_x)}
  else{resultatcriteres<-criteres(y,pred,dim_x=dim1_x)}
  resultatcriteres<-format(resultatcriteres,digits=5)

#Calcul sur la base de test
  if (is.null(xtest)==FALSE){
    predtest<-predictSR(modeleGLM,data.frame(xtest),methode=1)
    if (is.null(ytest)==FALSE){
      if(logy==TRUE){resultattest<-criteres(exp(ytest),exp(predtest),dim_x=dim1_x)}
      else{resultattest<-criteres(ytest,predtest,dim_x=dim1_x)}
      resultattest<-format(resultattest,digits=5)}}

#Parametrage de l'affichage des courbes
  if (affiche) {
    if (loo== TRUE  | vc==TRUE | bootstrap==TRUE){ # une page par modele
      postscript(file="compar1.ps")
      oldpar=par
      par(mfcol=c(3,2))
    }
    if (logy==TRUE){
      plot(y,y,col=2,type="l",main=("Modele GLM"),xlab="log(observation)",ylab="log(prediction)")
      points(y,pred)}
    else{
      plot(y,y,col=2,type="l",main=("Modele GLM"),xlab="observation",ylab="prediction")
      points(y,pred)}
  }

#--------------------
#Leave-one-out

  if(loo==TRUE){
    resultat1<-leaveoneout(x,y,methode=1,formule=formule,famille=famille,iteration=iteration,niter=niter,logy=logy,affiche=affiche)
    r<-length(resultat1)
    resultat1<-format(resultat1,digits=5)}

#--------------------
#Validation croisee

  if(vc==TRUE){
    resultat2<-cross(x,y,K=N,methode=1,formule=formule,famille=famille,iteration=iteration,niter=niter,logy=logy,affiche=affiche)
    r<-length(resultat2)
    resultat2<-format(resultat2,digits=5)}

#-----------
#Bootstrap 

  if(bootstrap==TRUE){
    estimation<-bootstrap(x,y,B=B,methode=1,formule=formule,famille=famille,iteration=iteration,niter=niter,elimination=TRUE,logy=logy,affiche=affiche)
    estimation<-format(estimation,digits=5)
    e<-length(estimation)}

#-------------------------------------
#Calcul des contributions des effets 
#-------------------------------------

  lien<-modeleGLM$family[2]
  if(stepwise==FALSE & lien=="identity"){

#Quand il n'y a qu'un seul coefficient (hors intercept) dans la formule
#de regression, on ne peut pas calculer les contributions

#Cas ou forme="maformule"
      if(forme=="maformule"){
        l<-length(modeleGLM$coefficients)

#Cas ou on a un seul coefficient (hors intercept)
#Calculer les contributions n'a donc aucun sens

        if(((intercept==TRUE)&(l<3))|((intercept==FALSE)&(l==1))){
          pred<-predict(modeleGLM,data.frame(x))
          effet<-sum((pred-mean(y))^2)
	  residus<-sum((pred-y)^2)
  	  total<-residus+effet
 	  pourcentages.effet<-100*(effet/total)
	  pourcentage.residus<-100*(residus/total)
	  idc<-NA
	  cor<-NA}

#Cas ou on a plusieurs coefficients
        else{
	  rescontrib<-contributions(x,y,forme=forme,degre=degreformule,famille=famille,intercept=intercept,a1=tabA1,a2=tabA2,a3=tabA3,a4=tabA4,b1=tabB1,c1=tabC1,c2=tabC2,d1=tabD1,d2=tabD2,d3=tabD3,d4=tabD4)
	  l<-length(modeleGLM$coefficients)
	  if(intercept==TRUE){l<-l-1}
	  effet<-rescontrib[1:l]
	  cor<-rescontrib[(l+1):(l+l^2)]
	  idc<-rescontrib[l+l^2+1]

#Calcul des residus
	  pred<-predict(modeleGLM,data.frame(x))
	  residus<-sum((pred-y)^2)

#Calcul des pourcentages correspondant
	  total<-residus+sum(effet,na.rm=TRUE)
 	  pourcentages.effet<-100*(effet/total)
	  pourcentage.residus<-100*(residus/total)}}

#Cas ou forme="creationformule"
      if(forme=="creationformule"){
        l<-length(modeleGLM$coefficients)

#Cas ou on a un seul coefficient (hors intercept)
#Calculer les contributions n'a donc aucun sens
        if(l<3){
      	  pred<-predict(modeleGLM,data.frame(x))
      	  effet<-sum((pred-mean(y))^2)
      	  residus<-sum((pred-y)^2)
      	  total<-residus+effet
      	  pourcentages.effet<-100*(effet/total)
          pourcentage.residus<-100*(residus/total)
      	  idc<-NA
      	  cor<-NA}

#Cas ou on a plusieurs coefficients
        else{
      	  rescontrib<-contributions(x,y,forme=forme,degre=degreformule,famille=famille,intercept=intercept,a1=tabA1,a2=tabA2,a3=tabA3,a4=tabA4,b1=tabB1,c1=tabC1,c2=tabC2,d1=tabD1,d2=tabD2,d3=tabD3,d4=tabD4)
      	  l<-length(modeleGLM$coefficients)
      	  l<-l-1
      	  effet<-rescontrib[1:l]
      	  cor<-rescontrib[(l+1):(l+l^2)]
      	  idc<-rescontrib[l+l^2+1]

#Calcul des residus
      	  pred<-predict(modeleGLM,data.frame(x))
      	  residus<-sum((pred-y)^2)

#Calcul des pourcentages correspondant
      	  total<-residus+sum(effet,na.rm=TRUE)
      	  pourcentages.effet<-100*(effet/total)
      	  pourcentage.residus<-100*(residus/total)}}

      A<-c(effet,pourcentages.effet,residus,pourcentage.residus)
      A<-format(A,digits=5)
      lA<-length(A)
      le<-(lA/2)-1
      effet<-A[1:le]
      pourcentages.effet<-A[(le+1):(2*le)]
      residus<-A[2*le+1]
      pourcentage.residus<-A[2*le+2]
      cor<-format(cor,digits=4)}

#-------------------------
#Affichage du modele GLM 
  write("",file=FOUT)
  write("**************************** Modele GLM ***************************",file=FOUT)
  write("",file=FOUT)
  if(stepwise==FALSE){ affichemodeleGLM(x,y,FOUT=FOUT,modele=modeleGLM,resultatcriteres=resultatcriteres,forme=forme,degreformule=degreformule,intercept=intercept,tabA1=tabA1,tabA2=tabA2,tabA3=tabA3,tabA4=tabA4,tabB1=tabB1,tabC1=tabC1,tabC2=tabC2,tabD1=tabD1,tabD2=tabD2,tabD3=tabD3,tabD4=tabD4)}
  else{ affichestepwise(modele=modeleGLM,FOUT=FOUT,resultatcriteres=resultatcriteres,bootstrap=bootstrap,e=e,estimation=estimation,restep=restep)}

#----------------------------------------
#Affichage des contributions des effets 
  if(stepwise==FALSE & lien=="identity"){ affichecontributions(x,y,FOUT=FOUT,modeleGLM=modeleGLM,forme=forme,degreformule=degreformule,idc=idc,cor=cor,effet=effet,pourcentages.effet=pourcentages.effet,residus=residus,pourcentage.residus=pourcentage.residus,intercept=intercept,tabA1=tabA1,tabA2=tabA2,tabA3=tabA3,tabA4=tabA4,tabB1=tabB1,tabC1=tabC1,tabC2=tabC2,tabD1=tabD1,tabD2=tabD2,tabD3=tabD3,tabD4=tabD4)}

#------------------------------------
#Affichage des leave-one-out, validation croisee et bootstrap 
  if(loo==TRUE){ afficheloo(x,y,FOUT=FOUT,resultat=resultat1)}
  if(vc==TRUE){ affichevc(x,y,FOUT=FOUT,r=r,resultat=resultat2,K=N)}
  if(bootstrap==TRUE & stepwise==FALSE){ affichebootstrap(x,y,FOUT=FOUT,modele=modeleGLM,forme=forme,degreformule=degreformule,e=e,estimation=estimation,intercept=intercept,tabA1=tabA1,tabA2=tabA2,tabA3=tabA3,tabA4=tabA4,tabB1=tabB1,tabC1=tabC1,tabC2=tabC2,tabD1=tabD1,tabD2=tabD2,tabD3=tabD3,tabD4=tabD4)}

#------------------------------------
#Affichage de la base de test
  if(is.null(xtest)==FALSE){
    if(is.null(ytest)==FALSE){ affichetest(FOUT=FOUT,resultatcriteres=resultattest)}
    else{
      FTEST <- file("resGLM-test.dat","w")
      write(predtest,ncolumns=1,file=FTEST)
      close(FTEST)}}

#-------------------------------------------------
#Assignement du modele en sortie de la fonction
  modeleGLM<<-modeleGLM

  print("Evaluation GLM terminee");}

#---------------------------------------------------------------------
# Evaluation des donnees par la methode PLS
#---------------------------------------------------------------------

if (methodes[2]==TRUE){

#Initialisation du modele
# Attention : la formule peut avoir ete modifie par l'algo stepwise du GLM
  modelePLS<-NaN

  if (methodes[1]==TRUE) modele<-modeleGLM
  else modele<-glm(formule,data=data.frame(x,y))
  ncomp=length(modele$coefficients)-1
  dim1_x <<- ncomp # nb de parametres du modele

# Attention, le modele optimal est construit sur les valeurs de x et y, qui peuvent avoir ete passees au log. A garder en tete.
  modelePLS<-plsr(formule,ncomp=ncomp,data=data.frame(x,y),method="kernelpls")
    
#Calcul des criteres de qualite
  pred<-predict(modelePLS,data.frame(x),cumulative=FALSE,comps=modelePLS$ncomp)
  
  if(logy==TRUE) resultatcriteres<-criteres(exp(y),exp(pred),dim_x=dim1_x)
  else resultatcriteres<-criteres(y,pred,dim_x=dim1_x)
  resultatcriteres<-format(resultatcriteres,digits=5)
  
#Calcul sur la base de test
  if (is.null(xtest)==FALSE){
    predtest<-predict(modelePLS,data.frame(xtest),comps=modelePLS$ncomp)
    if (is.null(ytest)==FALSE){
      if(logy==TRUE){resultattest<-criteres(exp(ytest),exp(predtest),dim_x=dim1_x)}
      else{resultattest<-criteres(ytest,predtest,dim_x=dim1_x)}
      resultattest<-format(resultattest,digits=5)}}

#Parametrage de l'affichage des courbes
  if (affiche) {
    if (loo==TRUE | vc==TRUE | bootstrap==TRUE){ # une page par modele
      postscript(file="compar2.ps")
      oldpar=par
      par(mfcol=c(3,2))
    }

    if (logy==TRUE){
      plot(y,y,col=2,type="l",main=("Modele PLS"),xlab="log(observation)",ylab="log(prediction)")
      points(y,pred)}
    else{
      plot(y,y,col=2,type="l",main=("Modele PLS"),xlab="observation",ylab="prediction")
      points(y,pred)}
  }

#--------------------
#Leave-one-out

  if(loo==TRUE){
    resultat1<-leaveoneout(x,y,methode=2,formule=formule,ncomp=ncomp,logy=logy,affiche=affiche)
    r<-length(resultat1)
    resultat1<-format(resultat1,digits=5)}

#--------------------
#Validation croisee

  if(vc==TRUE){
    resultat2<-cross(x,y,K=N,methode=2,formule=formule,ncomp=ncomp,logy=logy,affiche=affiche)
    r<-length(resultat2)
    resultat2<-format(resultat2,digits=5)}

#-----------
#Bootstrap 

  if(bootstrap==TRUE){
    estimation<-bootstrap(x,y,B=B,methode=2,formule=formule,ncomp=ncomp,logy=logy,affiche=affiche)
    estimation<-format(estimation,digits=5)
    e<-length(estimation)}

#-------------------------
#Affichage du modele PLS
  write("",file=FOUT)
  write("**************************** Modele PLS ***************************",file=FOUT)
  write("",file=FOUT)
  write("------------------------------------------",file=FOUT)
  write("Les coefficients de la premiere composante",file=FOUT)
  write("------------------------------------------",file=FOUT)
  write.table(data.frame(modelePLS$coefficients[1:ncomp]),file=FOUT)
  write("",file=FOUT)

#------------------------------------
#Affichage des leave-one-out, validation croisee et bootstrap 
  if(loo==TRUE){ afficheloo(x,y,FOUT=FOUT,resultat=resultat1)}
  if(vc==TRUE){ affichevc(x,y,FOUT=FOUT,r=r,resultat=resultat2,K=N)}
  if(bootstrap==TRUE){ affichebootstrap(x,y,FOUT=FOUT,e=e,estimation=estimation)}
#  if(bootstrap==TRUE){ affichebootstrap(x,y,FOUT=FOUT,modele=modelePLS,forme=forme,degreformule=degreformule,e=e,estimation=estimation,intercept=intercept,tabA1=tabA1,tabA2=tabA2,tabA3=tabA3,tabA4=tabA4,tabB1=tabB1,tabC1=tabC1,tabC2=tabC2,tabD1=tabD1,tabD2=tabD2,tabD3=tabD3,tabD4=tabD4)}
  
#------------------------------------
#Affichage de la base de test
  if(is.null(xtest)==FALSE){
    if(is.null(ytest)==FALSE){ affichetest(FOUT=FOUT,resultatcriteres=resultattest)}
    else{
      FTEST <- file("resPLS-test.dat","w")
      write(predtest,ncolumns=1,file=FTEST)
      close(FTEST)}}

#-------------------------------------------------
#Assignement du modele en sortie de la fonction
  modelePLS<<-modelePLS

  print("Evaluation PLS terminee");}

#---------------------------------------------------------------------
# Evaluation des donnees par la methode SVM
#---------------------------------------------------------------------

if (methodes[3]==TRUE){

#Initialisation du modele
  modeleSVM<-NaN

  if (SVMexplore==TRUE){
    select(x,y);
    paramssvm=exploreSVM(xa,ya,xt,yt,logy,TRUE,critere,cmin,cmax,cpas,epsmin,epsmax,epspas,gmin,gmax,gpas)}
  else{ paramssvm=c(cmin,epsmin,gmin)}

# Attention, le modele optimal est construit sur les valeurs de x et y, qui peuvent avoir ete passees au log. A garder en tete.
  if (qualitatif==TRUE){ modeleSVM=svm(y~.,data=x,kernel="radial",cost=paramssvm[1],epsilon=paramssvm[2],gamma=paramssvm[3])}
  else{ modeleSVM=svm(x,y,kernel="radial",cost=paramssvm[1],epsilon=paramssvm[2],gamma=paramssvm[3])}

#Calcul des criteres de qualite
  pred<-predict(modeleSVM,x)
  if(logy==TRUE){ resultatcriteres<-criteres(exp(y),exp(pred),dim_x=dim_x)}
  else{ resultatcriteres<-criteres(y,pred,dim_x=dim_x)}
  resultatcriteres<-format(resultatcriteres,digits=5)

#Calcul sur la base de test
  if (is.null(xtest)==FALSE){
    predtest<-predict(modeleSVM,xtest)
    if (is.null(ytest)==FALSE){
      if(logy==TRUE){resultattest<-criteres(exp(ytest),exp(predtest),dim_x=dim_x)}
      else{resultattest<-criteres(ytest,predtest,dim_x=dim_x)}
      resultattest<-format(resultattest,digits=5)}}

#Parametrage de l'affichage des courbes
  if (affiche) {
    if (loo==TRUE | vc==TRUE | bootstrap==TRUE){ # une page par modele
      postscript(file="compar3.ps")
      oldpar=par
      par(mfcol=c(3,2))
    }
    if (logy==TRUE){
      plot(y,y,col=2,type="l",main=("Modele SVM"),xlab="log(observation)",ylab="log(prediction)") 
      points(y,pred)}
    else{
      plot(y,y,col=2,type="l",main=("Modele SVM"),xlab="observation",ylab="prediction") 
      points(y,pred)}
  }

#--------------------
#Leave-one-out
  if(loo==TRUE){
    resultat1<-leaveoneout(x,y,methode=3,c=paramssvm[1],eps=paramssvm[2],g=paramssvm[3],logy=logy,affiche=affiche)
    r<-length(resultat1)
    resultat1<-format(resultat1,digits=5)}

#--------------------
#Validation croisee
  if(vc==TRUE){
    resultat2<-cross(x,y,K=N,methode=3,c=paramssvm[1],eps=paramssvm[2],g=paramssvm[3],logy=logy,affiche=affiche)
    r<-length(resultat2)
    resultat2<-format(resultat2,digits=5)}

#-----------
#Bootstrap 
  if(bootstrap==TRUE){
    estimation<-bootstrap(x,y,B=B,methode=3,c=paramssvm[1],eps=paramssvm[2],g=paramssvm[3],elimination=TRUE,logy=logy,affiche=affiche)
    estimation<-format(estimation,digits=5)
    e<-length(estimation)}

#-------------------------
#Affichage du modele SVM
  write("",file=FOUT)
  write("**************************** Modele SVM ***************************",file=FOUT)
  write("",file=FOUT)
  write("kernel: radial",file=FOUT);
  write("   cost   epsilon  gamma  Nb_vecteurs_supports",file=FOUT);
  paramssvm[4]=length(c(modeleSVM$coefs));
  paramssvm=format(paramssvm,digits=5)
  write(paramssvm,file=FOUT,ncolumns=4);

#------------------------------------
#Affichage des leave-one-out, validation croisee et bootstrap 
  if(loo==TRUE){ afficheloo(x,y,FOUT=FOUT,resultat=resultat1)}
  if(vc==TRUE){ affichevc(x,y,FOUT=FOUT,r=r,resultat=resultat2,K=N)}
  if(bootstrap==TRUE){ affichebootstrap(x,y,FOUT=FOUT,e=e,estimation=estimation)}

#------------------------------------
#Affichage de la base de test
  if(is.null(xtest)==FALSE){
    if(is.null(ytest)==FALSE){ affichetest(FOUT=FOUT,resultatcriteres=resultattest)}
    else{
      FTEST <- file("resSVM-test.dat","w")
      write(predtest,ncolumns=1,file=FTEST)
      close(FTEST)}}

#-------------------------------------------------
#Assignement du modele en sortie de la fonction
  modeleSVM<<-modeleSVM

  print("Evaluation SVM terminee");}

#---------------------------------------------------------------------
# Evaluation des donnees par la methode de BOOSTING
#---------------------------------------------------------------------

if (methodes[4]==TRUE){

#Initialisation du modele
  modeleBOOSTING<-NaN

# Attention, le modele optimal est construit sur les valeurs de x et y, qui peuvent avoir ete passees au log. A garder en tete.
  modeleBOOSTING=gbm.fit(x,y,distribution="gaussian",n.trees=n.trees,interaction.depth=interaction.depth,verbose=TRUE)

#Calcul des criteres de qualite
  pred<-predict(modeleBOOSTING,x,n.trees=n.trees)
  if(logy==TRUE){ resultatcriteres<-criteres(exp(y),exp(pred),dim_x=dim_x)}
  else{ resultatcriteres<-criteres(y,pred,dim_x=dim_x)}
  resultatcriteres<-format(resultatcriteres,digits=5)

#Calcul sur la base de test
  if (is.null(xtest)==FALSE){
    predtest<-predict(modeleBOOSTING,xtest,n.trees=n.trees)
    if (is.null(ytest)==FALSE){
      if(logy==TRUE){resultattest<-criteres(exp(ytest),exp(predtest),dim_x=dim_x)}
      else{resultattest<-criteres(ytest,predtest,dim_x=dim_x)}
      resultattest<-format(resultattest,digits=5)}}

#Parametrage de l'affichage des courbes
  if (affiche) {
    if (loo==TRUE | vc==TRUE | bootstrap==TRUE){ # une page par modele
      postscript(file="compar4.ps")
      oldpar=par
      par(mfcol=c(3,2))
    }
    if (logy==TRUE){
      plot(y,y,col=2,type="l",main=("Modele BOOSTING"),xlab="log(observation)",ylab="log(prediction)") 
      points(y,pred)}
    else{
      plot(y,y,col=2,type="l",main=("Modele BOOSTING"),xlab="observation",ylab="prediction") 
      points(y,pred)}
  }

#--------------------
#Leave-one-out
  if(loo==TRUE){
    resultat1<-leaveoneout(x,y,methode=4,n.trees=n.trees,interaction.depth=interaction.depth,logy=logy,affiche=affiche)
    r<-length(resultat1)
    resultat1<-format(resultat1,digits=5)}

#--------------------
#Validation croisee
  if(vc==TRUE){
    resultat2<-cross(x,y,K=N,methode=4,n.trees=n.trees,interaction.depth=interaction.depth,logy=logy,affiche=affiche)
    r<-length(resultat2)
    resultat2<-format(resultat2,digits=5)}

#-----------
#Bootstrap 
  if(bootstrap==TRUE){
    estimation<-bootstrap(x,y,B=B,methode=4,n.trees=n.trees,interaction.depth=interaction.depth,elimination=TRUE,logy=logy,affiche=affiche)
    estimation<-format(estimation,digits=5)
    e<-length(estimation)}

#-------------------------
#Affichage du modele BOOSTING
  write("",file=FOUT)
  write("**************************** Modele BOOSTING ***************************",file=FOUT)
  write("",file=FOUT)
  write("distribution: gaussian",file=FOUT);
  write("nb_arbres  ordre_interactions",file=FOUT)
  output=format(c(n.trees,interaction.depth),digits=5)
  write(output,file=FOUT,ncolumns=2);
  
#------------------------------------
#Affichage des leave-one-out, validation croisee et bootstrap 
  if(loo==TRUE){ afficheloo(x,y,FOUT=FOUT,resultat=resultat1)}
  if(vc==TRUE){ affichevc(x,y,FOUT=FOUT,r=r,resultat=resultat2,K=N)}
  if(bootstrap==TRUE){ affichebootstrap(x,y,FOUT=FOUT,e=e,estimation=estimation)}

#------------------------------------
#Affichage de la base de test
  if(is.null(xtest)==FALSE){
    if(is.null(ytest)==FALSE){ affichetest(FOUT=FOUT,resultatcriteres=resultattest)}
    else{
      FTEST <- file("resBOOSTING-test.dat","w")
      write(predtest,ncolumns=1,file=FTEST)
      close(FTEST)}}

#-------------------------------------------------
#Assignement du modele en sortie de la fonction
  modeleBOOSTING<<-modeleBOOSTING

  print("Evaluation BOOSTING terminee");}

#---------------------------------------------------------------------
# Evaluation des donnees par la methode FORETS ALEATOIRES
#---------------------------------------------------------------------

if (methodes[5]==TRUE){

#Initialisation du modele
  modeleRF<-NaN

# Attention, le modele optimal est construit sur les valeurs de x et y, qui peuvent avoir ete passees au log. A garder en tete.
  modeleRF=randomForest(x,y,ntree=ntree,do.trace=TRUE)

#Calcul des criteres de qualite
  pred<-predict(modeleRF,x)
  if(logy==TRUE){ resultatcriteres<-criteres(exp(y),exp(pred),dim_x=dim_x)}
  else{ resultatcriteres<-criteres(y,pred,dim_x=dim_x)}
  resultatcriteres<-format(resultatcriteres,digits=5)

#Calcul sur la base de test
  if (is.null(xtest)==FALSE){
    predtest<-predict(modeleRF,xtest)
    if (is.null(ytest)==FALSE){
      if(logy==TRUE){resultattest<-criteres(exp(ytest),exp(predtest),dim_x=dim_x)}
      else{resultattest<-criteres(ytest,predtest,dim_x=dim_x)}
      resultattest<-format(resultattest,digits=5)}}

#Parametrage de l'affichage des courbes
  if (affiche) {
    if (loo==TRUE | vc==TRUE | bootstrap==TRUE){ # une page par modele
      postscript(file="compar5.ps")
      oldpar=par
      par(mfcol=c(3,2))
   }
    if (logy==TRUE){
      plot(y,y,col=2,type="l",main=("Modele FORETS ALEATOIRES"),xlab="log(observation)",ylab="log(prediction)") 
      points(y,pred)}
    else{
      plot(y,y,col=2,type="l",main=("Modele FORETS ALEATOIRES"),xlab="observation",ylab="prediction") 
      points(y,pred)}
  }

#--------------------
#Leave-one-out
  if(loo==TRUE){
    resultat1<-leaveoneout(x,y,methode=5,ntree=ntree,logy=logy,affiche=affiche)
    r<-length(resultat1)
    resultat1<-format(resultat1,digits=5)}

#--------------------
#Validation croisee
  if(vc==TRUE){
    resultat2<-cross(x,y,K=N,methode=5,ntree=ntree,logy=logy,affiche=affiche)
    r<-length(resultat2)
    resultat2<-format(resultat2,digits=5)}

#-----------
#Bootstrap 
  if(bootstrap==TRUE){
    estimation<-bootstrap(x,y,B=B,methode=5,ntree=ntree,elimination=TRUE,logy=logy,affiche=affiche)
    estimation<-format(estimation,digits=5)
    e<-length(estimation)}

#-------------------------
#Affichage du modele FORETS ALEATOIRES
  write("",file=FOUT)
  write("********************** Modele  FORETS ALEATOIRE **********************",file=FOUT)
  write("",file=FOUT)
  write("Nombre d'arbres:",file=FOUT);
  write(ntree,file=FOUT);

#------------------------------------
#Affichage des leave-one-out, validation croisee et bootstrap 
  if(loo==TRUE){ afficheloo(x,y,FOUT=FOUT,resultat=resultat1)}
  if(vc==TRUE){ affichevc(x,y,FOUT=FOUT,r=r,resultat=resultat2,K=N)}
  if(bootstrap==TRUE){ affichebootstrap(x,y,FOUT=FOUT,e=e,estimation=estimation)}

#------------------------------------
#Affichage de la base de test
  if(is.null(xtest)==FALSE){
    if(is.null(ytest)==FALSE){ affichetest(FOUT=FOUT,resultatcriteres=resultattest)}
    else{
      FTEST <- file("resRF-test.dat","w")
      write(predtest,ncolumns=1,file=FTEST)
      close(FTEST)}}

#-------------------------------------------------
#Assignement du modele en sortie de la fonction
  modeleRF<<-modeleRF

  print("Evaluation FORETS ALEATOIRES terminee");}

#---------------------------------------------------------------------
# Evaluation des donnees par la methode des RESEAUX de NEURONES
#---------------------------------------------------------------------

if (methodes[6]==TRUE){

#Initialisation du modele
  modeleRN<-NaN

# Attention, le modele optimal est construit sur les valeurs de x et y, qui peuvent avoir ete passees au log. A garder en tete.
  modeleRN=nnet(x,y,linout=TRUE,size=nunits,maxit=maxitRN,MaxNWts=nunits*nunits*dim_x)

#Calcul des criteres de qualite
  pred<-predict(modeleRN,x)
  if(logy==TRUE){ resultatcriteres<-criteres(exp(y),exp(pred),dim_x=dim_x)}
  else{ resultatcriteres<-criteres(y,pred,dim_x=dim_x)}
  resultatcriteres<-format(resultatcriteres,digits=5)

#Calcul sur la base de test
  if (is.null(xtest)==FALSE){
    predtest<-predict(modeleRN,xtest)
    if (is.null(ytest)==FALSE){
      if(logy==TRUE){resultattest<-criteres(exp(ytest),exp(predtest),dim_x=dim_x)}
      else{resultattest<-criteres(ytest,predtest,dim_x=dim_x)}
      resultattest<-format(resultattest,digits=5)}}

#Parametrage de l'affichage des courbes
  if (affiche) {
    if (loo==TRUE | vc==TRUE | bootstrap==TRUE){ # une page par modele
      postscript(file="compar6.ps")
      oldpar=par
      par(mfcol=c(3,2))
    }
    if (logy==TRUE){
      plot(y,y,col=2,type="l",main=("Modele RESEAU de NEURONES"),xlab="log(observation)",ylab="log(prediction)") 
      points(y,pred)}
    else{
      plot(y,y,col=2,type="l",main=("Modele RESEAU de NEURONES"),xlab="observation",ylab="prediction") 
      points(y,pred)}
  }

#--------------------
#Leave-one-out
  if(loo==TRUE){
    resultat1<-leaveoneout(x,y,methode=6,nunits=nunits,logy=logy,affiche=affiche)
    r<-length(resultat1)
    resultat1<-format(resultat1,digits=5)}

#--------------------
#Validation croisee
  if(vc==TRUE){
    resultat2<-cross(x,y,K=N,methode=6,nunits=nunits,logy=logy,affiche=affiche)
    r<-length(resultat2)
    resultat2<-format(resultat2,digits=5)}

#-----------
#Bootstrap 
  if(bootstrap==TRUE){
    estimation<-bootstrap(x,y,B=B,methode=6,nunits=nunits,elimination=TRUE,logy=logy,affiche=affiche)
    estimation<-format(estimation,digits=5)
    e<-length(estimation)}

#-------------------------
#Affichage du modele RESEAU de NEURONES
  write("",file=FOUT)
  write("******************** Modele RESEAU de NEURONES ********************",file=FOUT)
  write("",file=FOUT)
  write("Nombre de noeuds dans la couche cachee:",file=FOUT);
  write(nunits,file=FOUT);

#------------------------------------
#Affichage des leave-one-out, validation croisee et bootstrap 
  if(loo==TRUE){ afficheloo(x,y,FOUT=FOUT,resultat=resultat1)}
  if(vc==TRUE){ affichevc(x,y,FOUT=FOUT,r=r,resultat=resultat2,K=N)}
  if(bootstrap==TRUE){ affichebootstrap(x,y,FOUT=FOUT,e=e,estimation=estimation)}

#------------------------------------
#Affichage de la base de test
  if(is.null(xtest)==FALSE){
    if(is.null(ytest)==FALSE){ affichetest(FOUT=FOUT,resultatcriteres=resultattest)}
    else{
      FTEST <- file("resRN-test.dat","w")
      write(predtest,ncolumns=1,file=FTEST)
      close(FTEST)}}

#-------------------------------------------------
#Assignement du modele en sortie de la fonction
  modeleRN<<-modeleRN

  print("Evaluation RESEAU de NEURONES terminee");}

#---------------------------------------------------------------------
# Evaluation des donnees par la methode GAM
#---------------------------------------------------------------------

if (methodes[7]==TRUE){

#Initialisation du modele
  modeleGAM<-NaN

  xnames <- NULL # On transforme les noms des variables en V1, V2, V3, ...
  for (i in 1:dim_x) xnames <- c(xnames,paste("V",as.character(i),sep=""))
  x <- matrix(c(x),nr=dim(x)[1],nc=dim(x)[2],dimnames=list(1:dim(x)[1],xnames))

  if(forme=="maformule") formule <- maformule(dimx=dim_x,intercept=intercept,a1=tabA1,a2=tabA2,a3=tabA3,a4=tabA4,b1=tabB1,c1=tabC1,c2=tabC2,d1=tabD1,d2=tabD2,d3=tabD3,d4=tabD4,agam=tabAgam,bgam=tabBgam,cgam=tabCgam,dgam=tabDgam)
  else formule <- creationformule(dimx=dim_x,gam=TRUE,interaction.depth=degreformule)

# Attention, le modele optimal est construit sur les valeurs de x et y, qui peuvent avoir ete passees au log. A garder en tete.
  modeleGAM<-gam(formule,famille,data=data.frame(x,y))
  dim1_x <<- dim(as.data.frame(modeleGAM$coefficients))[1] # nb de parametres du modele

#Calcul des criteres de qualite
  pred<-predictSR(modeleGAM,data.frame(x),methode=7)
  if(logy==TRUE){ resultatcriteres<-criteres(exp(y),exp(pred),dim_x=dim1_x)}
  else{ resultatcriteres<-criteres(y,pred,dim_x=dim1_x)}
  resultatcriteres<-format(resultatcriteres,digits=5)

#Calcul sur la base de test
  if (is.null(xtest)==FALSE){
    predtest<-predictSR(modeleGAM,data.frame(xtest),methode=7)
    if (is.null(ytest)==FALSE){
      if(logy==TRUE){resultattest<-criteres(exp(ytest),exp(predtest),dim_x=dim1_x)}
      else{resultattest<-criteres(ytest,predtest,dim_x=dim1_x)}
      resultattest<-format(resultattest,digits=5)}}

#Parametrage de l'affichage des courbes
  if (affiche) {
    if (loo==TRUE | vc==TRUE | bootstrap==TRUE){ # une page par modele
      postscript(file="compar7.ps")
      oldpar=par
      par(mfcol=c(3,2))
    }
    if (logy==TRUE){
      plot(y,y,col=2,type="l",main=("Modele GAM"),xlab="log(observation)",ylab="log(prediction)") 
      points(y,pred)}
    else{
      plot(y,y,col=2,type="l",main=("Modele GAM"),xlab="observation",ylab="prediction") 
      points(y,pred)}
  }

#--------------------
#Leave-one-out
  if(loo==TRUE){
    resultat1<-leaveoneout(x,y,methode=7,formule=formule,famille=famille,nunits=nunits,logy=logy,affiche=affiche)
    r<-length(resultat1)
    resultat1<-format(resultat1,digits=5)}

#--------------------
#Validation croisee
  if(vc==TRUE){
    resultat2<-cross(x,y,K=N,methode=7,formule=formule,famille=famille,nunits=nunits,logy=logy,affiche=affiche)
    r<-length(resultat2)
    resultat2<-format(resultat2,digits=5)}

#-----------
#Bootstrap 
  if(bootstrap==TRUE){
    estimation<-bootstrap(x,y,B=B,methode=7,formule=formule,famille=famille,nunits=nunits,elimination=TRUE,logy=logy,affiche=affiche)
    estimation<-format(estimation,digits=5)
    e<-length(estimation)}
#-------------------------
#Affichage du modele GAM
  write("",file=FOUT)
  write("******************** Modele GAM ********************",file=FOUT)
  write("",file=FOUT)
  write("Coefficients :",file=FOUT)
  write.table(t(rownames(as.data.frame(modeleGAM$coefficients))),file=FOUT)
  write("",file=FOUT)

#------------------------------------
#Affichage des leave-one-out, validation croisee et bootstrap 
  if(loo==TRUE){ afficheloo(x,y,FOUT=FOUT,resultat=resultat1)}
  if(vc==TRUE){ affichevc(x,y,FOUT=FOUT,r=r,resultat=resultat2,K=N)}
  if(bootstrap==TRUE){ affichebootstrap(x,y,FOUT=FOUT,e=e,estimation=estimation)}

#------------------------------------
#Affichage de la base de test
  if(is.null(xtest)==FALSE){
    if(is.null(ytest)==FALSE){ affichetest(FOUT=FOUT,resultatcriteres=resultattest)}
    else{
      FTEST <- file("resGAM-test.dat","w")
      write(predtest,ncolumns=1,file=FTEST)
      close(FTEST)}}

#-------------------------------------------------
#Assignement du modele en sortie de la fonction
  modeleGAM<<-modeleGAM

  print("Evaluation GAM terminee");}

#---------------------------------------------------------------------
# Evaluation des donnees par la methode des PROCESSUS GAUSSIENS
#---------------------------------------------------------------------

if (methodes[8]==TRUE){

#Initialisation du modele
  modelePG<-NaN

# Attention, le modele optimal est construit sur les valeurs de x et y, qui peuvent avoir ete passees au log. A garder en tete.
  modelePG=mlegp(x,y,constantMean=constantPG,min.nugget=min.nugget,simplex.ntries=simplex.ntries)

#Calcul des criteres de qualite
  pred<-predict(modelePG,x)
  if(logy==TRUE){ resultatcriteres<-criteres(exp(y),exp(pred),dim_x=dim_x)}
  else{ resultatcriteres<-criteres(y,pred,dim_x=dim_x)}
  resultatcriteres<-format(resultatcriteres,digits=5)

#Calcul sur la base de test
  if (is.null(xtest)==FALSE){
    predtest<-predict(modelePG,xtest)
    if (is.null(ytest)==FALSE){
      if(logy==TRUE){resultattest<-criteres(exp(ytest),exp(predtest),dim_x=dim_x)}
      else{resultattest<-criteres(ytest,predtest,dim_x=dim_x)}
      resultattest<-format(resultattest,digits=5)}}

#Parametrage de l'affichage des courbes
  if (affiche) {
    if (loo==TRUE | vc==TRUE | bootstrap==TRUE){ # une page par modele
      postscript(file="compar8.ps")
      oldpar=par
      par(mfcol=c(3,2))
    }
    if (logy==TRUE){
      plot(y,y,col=2,type="l",main=("Modele PROCESSUS GAUSSIEN"),xlab="log(observation)",ylab="log(prediction)") 
      points(y,pred)}
    else{
      plot(y,y,col=2,type="l",main=("Modele PROCESSUS GAUSSIEN"),xlab="observation",ylab="prediction") 
      points(y,pred)}
  }

#--------------------
#Leave-one-out
  if(loo==TRUE){
    resultat1<-leaveoneout(x,y,methode=8,min.nugget=min.nugget,simplex.ntries=simplex.ntries,logy=logy,affiche=affiche)
    r<-length(resultat1)
    resultat1<-format(resultat1,digits=5)}

#--------------------
#Validation croisee
  if(vc==TRUE){
    resultat2<-cross(x,y,K=N,methode=8,min.nugget=min.nugget,simplex.ntries=simplex.ntries,logy=logy,affiche=affiche)
    r<-length(resultat2)
    resultat2<-format(resultat2,digits=5)}

#-----------
#Bootstrap 
  if(bootstrap==TRUE){
    estimation<-bootstrap(x,y,B=B,methode=8,min.nugget=min.nugget,simplex.ntries=simplex.ntries,elimination=TRUE,logy=logy,affiche=affiche)
    estimation<-format(estimation,digits=5)
    e<-length(estimation)}

#-------------------------
#Affichage du modele PROCESSUS GAUSSIEN
  write("",file=FOUT)
  write("******************** Modele PROCESSUS GAUSSIEN ********************",file=FOUT)
  write("",file=FOUT)
  write("Parametres de la composante lineaire",file=FOUT)
  write.table(t(data.frame(modelePG$Bhat)),quote=F,row.names=F,col.names=F,file=FOUT)
  write("",file=FOUT)
  write("Parametres de correlation",file=FOUT)
  write.table(t(data.frame(modelePG$beta)),quote=F,row.names=F,file=FOUT)
  write("",file=FOUT)
  write("Parametres puissance",file=FOUT)
  write.table(t(data.frame(modelePG$a)),quote=F,row.names=F,file=FOUT)
  write("",file=FOUT)
  write("Parametre variance",file=FOUT)
  write.table(t(data.frame(modelePG$sig2)),quote=F,row.names=F,col.names=F,file=FOUT)
  write("Parametre pepite",file=FOUT)
  write.table(t(data.frame(modelePG$nugget)),quote=F,row.names=F,col.names=F,file=FOUT)
  write("",file=FOUT)
  write(c("R2 (leave-one-out) = ",1-sum((modelePG$Z-modelePG$cv[,1])^2)/var(modelePG$Z)/modelePG$numObs),file=FOUT)
  write("",file=FOUT)

#------------------------------------
#Affichage des leave-one-out, validation croisee et bootstrap 
  if(loo==TRUE){ afficheloo(x,y,FOUT=FOUT,resultat=resultat1)}
  if(vc==TRUE){ affichevc(x,y,FOUT=FOUT,r=r,resultat=resultat2,K=N)}
  if(bootstrap==TRUE){ affichebootstrap(x,y,FOUT=FOUT,e=e,estimation=estimation)}

#------------------------------------
#Affichage de la base de test
  if(is.null(xtest)==FALSE){
    if(is.null(ytest)==FALSE){ affichetest(FOUT=FOUT,resultatcriteres=resultattest)}
    else{
      FTEST <- file("resPG-test.dat","w")
      write(predtest,ncolumns=1,file=FTEST)
      close(FTEST)}}

#-------------------------------------------------
#Assignement du modele en sortie de la fonction
  modelePG<<-modelePG

  print("Evaluation PROCESSUS GAUSSIEN terminee");}


#---------------------------------------------------------------------
# Evaluation des donnees par la methode PROCESSUS GAUSSIENS : PGdice
#---------------------------------------------------------------------

if (methodes[9]==TRUE){

#Initialisation du modele
# Attention : la formule peut avoir ete modifie par l'algo stepwise du GLM
  modelePGdice<-NaN

  if (methodes[1]==TRUE) modele<-modeleGLM
  else modele<-glm(formule,data=data.frame(x,y))
  ncomp=length(modele$coefficients)-1
  dim1_x <<- ncomp # nb de parametres du modele

# Attention, le modele optimal est construit sur les valeurs de x et y, qui peuvent avoir ete passees au log. A garder en tete.
  modelePGdice<-km(formule,as.data.frame(x),as.data.frame(y),covtype="powexp",nugget.estim=nugget.estim,optim.method=optim.method)
    
#Calcul des criteres de qualite
  pred<-predict.km(modelePGdice,data.frame(x),type="UK")$mean
  
  if(logy==TRUE) resultatcriteres<-criteres(exp(y),exp(pred),dim_x=dim1_x)
  else resultatcriteres<-criteres(y,pred,dim_x=dim1_x)
  resultatcriteres<-format(resultatcriteres,digits=5)
  
#Calcul sur la base de test
  if (is.null(xtest)==FALSE){
    predtest<-predict.km(modelePGdice,data.frame(xtest),type="UK")$mean
    if (is.null(ytest)==FALSE){
      if(logy==TRUE){resultattest<-criteres(exp(ytest),exp(predtest),dim_x=dim1_x)}
      else{resultattest<-criteres(ytest,predtest,dim_x=dim1_x)}
      resultattest<-format(resultattest,digits=5)}}

#Parametrage de l'affichage des courbes
  if (affiche) {
    if (loo==TRUE | vc==TRUE | bootstrap==TRUE){ # une page par modele
      postscript(file="compar9.ps")
      oldpar=par
      par(mfcol=c(3,2))
    }

    if (logy==TRUE){
      plot(y,y,col=2,type="l",main=("Modele PGdice"),xlab="log(observation)",ylab="log(prediction)")
      points(y,pred)}
    else{
      plot(y,y,col=2,type="l",main=("Modele PGdice"),xlab="observation",ylab="prediction")
      points(y,pred)}
  }

#--------------------
#Leave-one-out

  if(loo==TRUE){
    resultat1<-leaveoneout(x,y,methode=9,formule=formule,nugget.estim=nugget.estim,optim.method=optim.method,logy=logy,affiche=affiche)
    r<-length(resultat1)
    resultat1<-format(resultat1,digits=5)}
  
#--------------------
#Validation croisee

  if(vc==TRUE){
    resultat2<-cross(x,y,K=N,methode=9,formule=formule,nugget.estim=nugget.estim,optim.method=optim.method,logy=logy,affiche=affiche)
    r<-length(resultat2)
  write("",file=FOUT)
    resultat2<-format(resultat2,digits=5)}

#-----------
#Bootstrap 

  if(bootstrap==TRUE){
    estimation<-bootstrap(x,y,B=B,methode=9,formule=formule,nugget.estim=nugget.estim,optim.method=optim.method,logy=logy,affiche=affiche)
    estimation<-format(estimation,digits=5)
    e<-length(estimation)}

#-------------------------
#Affichage du modele PGdice
  write("",file=FOUT)
  write("**************************** Modele PGdice ***************************",file=FOUT)
  write("",file=FOUT)
  write("Parametres de la composante lineaire (constante en 1er)",file=FOUT)
  write(as.character(modelePGdice$trend$formula)[c(2,1,3)],file=FOUT)
  write.table((data.frame(modelePGdice$trend$coef)),quote=F,row.names=F,col.names=F,file=FOUT)
  write("",file=FOUT)
  write("Parametres de correlation (portee inverse puis puissance)",file=FOUT)
  write.table((data.frame(modelePGdice$covariance$coef)),quote=F,row.names=F,col.names=F,file=FOUT)
  write("",file=FOUT)
  write("Parametre variance",file=FOUT)
  write.table(t(data.frame(modelePGdice$covariance$var)),quote=F,row.names=F,col.names=F,file=FOUT)
  write("Parametre pepite",file=FOUT)
  write.table(t(data.frame(modelePGdice$covariance$nugget)),quote=F,row.names=F,col.names=F,file=FOUT)
  write("",file=FOUT)

#------------------------------------
#Affichage des leave-one-out, validation croisee et bootstrap 
  if(loo==TRUE){ afficheloo(x,y,FOUT=FOUT,resultat=resultat1)}
  if(vc==TRUE){ affichevc(x,y,FOUT=FOUT,r=r,resultat=resultat2,K=N)}
  if(bootstrap==TRUE){ affichebootstrap(x,y,FOUT=FOUT,e=e,estimation=estimation)}
#  if(bootstrap==TRUE){ affichebootstrap(x,y,FOUT=FOUT,modele=modelePGdice,forme=forme,degreformule=degreformule,e=e,estimation=estimation,intercept=intercept,tabA1=tabA1,tabA2=tabA2,tabA3=tabA3,tabA4=tabA4,tabB1=tabB1,tabC1=tabC1,tabC2=tabC2,tabD1=tabD1,tabD2=tabD2,tabD3=tabD3,tabD4=tabD4)}
  
#------------------------------------
#Affichage de la base de test
  if(is.null(xtest)==FALSE){
    if(is.null(ytest)==FALSE){ affichetest(FOUT=FOUT,resultatcriteres=resultattest)}
    else{
      FTEST <- file("resPGdice-test.dat","w")
      write(predtest,ncolumns=1,file=FTEST)
      close(FTEST)}}

#-------------------------------------------------
#Assignement du modele en sortie de la fonction
  modelePGdice<<-modelePGdice

  print("Evaluation PGdice terminee");}


#------------------------------------------------
#------------------------------------------------
#Recuperation de parametres
if (loo==T) Q2 <<- resultat1[1]

#-----------------------
#Affichage des courbes |
#-----------------------
if (affiche){
  par(oldpar)
  dev.off()

  if (Sys.getenv("OS")!="Windows_NT"){
    if (vc==TRUE | bootstrap==TRUE){
      if (methodes[1]==TRUE) system("gv compar1.ps &")
      if (methodes[2]==TRUE) system("gv compar2.ps &")
      if (methodes[3]==TRUE) system("gv compar3.ps &")
      if (methodes[4]==TRUE) system("gv compar4.ps &")
      if (methodes[5]==TRUE) system("gv compar5.ps &")
      if (methodes[6]==TRUE) system("gv compar6.ps &")
      if (methodes[7]==TRUE) system("gv compar7.ps &")
      if (methodes[8]==TRUE) system("gv compar8.ps &")
      if (methodes[9]==TRUE) system("gv compar9.ps &")
    }
    else{ # une page pour tous les modeles 
      system("gv compar.ps &")}}}

#-----------------------------------
#Fermeture du fichier de resultats |
#-----------------------------------

close(FOUT)

# Affichage du fichier de resultat
if (affiche_file==T){
  if (Sys.getenv("OS")!="Windows_NT") pager_OS="cat"
  else pager_OS=""

  file.show("res-SURFER.out",pager=pager_OS)
}

#-----------------------------------------------------------------
#-----------------------------------------------------------------
#Calcul des indices de SOBOL                               
#-----------------------------------------------------------------
#-----------------------------------------------------------------

if(sobol==TRUE){

#Suppression du fichier res-SOBOL.out
  FSOB <- file("res-SOBOL.out","w")
  close(FSOB)

  if (methodes[1]==TRUE & stepwise==FALSE){ 
    sobolS(modele=modeleGLM,dim_x=dim_x,methode=1,x=x,logx=logx,logy=logy,S=S,R=R,Sij=Sij,LHS=LHS,lois=lois,paramlois=paramlois)}
  if (methodes[2]==TRUE & (methodes[1]==FALSE | stepwise==FALSE)){ 
    sobolS(modele=modelePLS,dim_x=dim_x,methode=2,x=x,logx=logx,logy=logy,S=S,R=R,Sij=Sij,LHS=LHS,lois=lois,paramlois=paramlois)}
  if (methodes[3]==TRUE){ 
    sobolS(modele=modeleSVM,dim_x=dim_x,methode=3,x=x,logx=logx,logy=logy,S=S,R=R,Sij=Sij,LHS=LHS,lois=lois,paramlois=paramlois)}
  if (methodes[4]==TRUE){ 
    sobolS(modele=modeleBOOSTING,dim_x=dim_x,methode=4,x=x,logx=logx,logy=logy,S=S,R=R,Sij=Sij,LHS=LHS,n.trees=n.trees,lois=lois,paramlois=paramlois)}
  if (methodes[5]==TRUE){ 
    sobolS(modele=modeleRF,dim_x=dim_x,methode=5,x=x,logx=logx,logy=logy,S=S,R=R,Sij=Sij,LHS=LHS,lois=lois,paramlois=paramlois)}
  if (methodes[6]==TRUE){ 
    sobolS(modele=modeleRN,dim_x=dim_x,methode=6,x=x,logx=logx,logy=logy,S=S,R=R,Sij=Sij,LHS=LHS,lois=lois,paramlois=paramlois)}
  if (methodes[7]==TRUE){ 
    sobolS(modele=modeleGAM,dim_x=dim_x,methode=7,x=x,logx=logx,logy=logy,S=S,R=R,Sij=Sij,LHS=LHS,lois=lois,paramlois=paramlois)}
  if (methodes[8]==TRUE){ 
    sobolS(modele=modelePG,dim_x=dim_x,methode=8,x=x,logx=logx,logy=logy,S=S,R=R,Sij=Sij,LHS=LHS,lois=lois,paramlois=paramlois)}
  if (methodes[9]==TRUE){ 
    sobolS(modele=modelePGdice,dim_x=dim_x,methode=9,x=x,logx=logx,logy=logy,S=S,R=R,Sij=Sij,LHS=LHS,lois=lois,paramlois=paramlois)}

# affichage du fichier de rsultat
  if (affiche_file==T) file.show("res-SOBOL.out",pager=pager_OS)
  }

# pour ne pas finir l'execution de SSURFER
#browser()
}
# END OF FUNCTION
