statS<-function(x,nom=c(NA)){

#-----------------------------------------------------------------------
#Fonction calculant de nombreux parametres statistiques elementaires   |
#des colonnes de x, avec des tests d'adequation aux principales lois,  |
#et des graphiques (histogramme, densite, boxplot et Q-Q Plot)         |
# Mise en oeuvre des inegalites de Tchebytchev et Vysocanskii-Petunin  |
#----------------------------------------------------------------------|
#Arguments :                                                           |
#                                                                      |
#obligatoire : x : matrice des variables d'entree (format colonnes)    |
#                  x peut posseder des elements vides (contenant "NA") |
#                                                                      |
#facultatif : nom : vecteur des noms des variables (7 caracteres max)  |
#                   (par defaut : header du tableau x)                 |
#----------------------------------------------------------------------|
#Sortie : tous les resultats sont affiches dans le fichier res-STAT.out|
#----------------------------------------------------------------------|
#Utilisation :                                                         |
#  - Au prealable, il faut allouer x sous R                            |
#      Exemple a partir d'un fichier avec header :                     |
#            mat<-read.table(file="MOX_DEI.txt",header=TRUE,dec=",")   |
#            x<-mat[,2:length(mat)]                                    |
#            x<-x+runif(dim(x)[[1]]*dim(x)[[2]],min=-0.0005,max=0.0005)|
#                      # on rajoute un bruit pour avoir 1 var. continue|
#            statS(x)                                                  |
#      Exemple a partir d'un fichier sans header :                     |
#            x<-read.table(file="nom_fichier")                         |
#                                                                      |
#  - Il faut le meme nombre de donnees dans chaque colonne             |
#    ou alors on complete avec des "NA"                                |
#                                                                      |
# WARNING : PROBLEMES si les intervalles de variation des donnees sont |
# a valeurs trop faibles (<1e-6) ou trop fortes (>1e6)                 |
# En ce cas ("system is computationally singular", "Erreur dans optim")|
# il suffit de travailler sur des variables standardisees              |
#                                                                      |
#----------------------------------------------------------------------|
#      Auteur : B. Iooss                                               |
#----------------------------------------------------------------------|

# *********************************************************************
# FONCTION UTILISEE :
#      truncated.R (fonctions pour lois tronquees)
# *********************************************************************
# LIBRAIRIES REQUISES (A INSTALLER)
library(e1071)
library(evd)
library(triangle)

# dechargement du package fExtremes (risque d'interaction avec evd)
if ("package:fExtremes" %in% search()) detach(package:fExtremes)

# *********************************************************************

# INITIALISATIONS

ndigits<-4  # nombre de chiffres apres la virgule pour les stats
Nboot<-100 # nombre de repliques bootstrap

# noms des statistiques
nomstats<-c("nombre    ","moyenne   ","ecart-type","coeff var ","skewness  ","kurtosis  ","minimum   ","quartile_1","mediane   ","quartile_3","maximum   ","etendue   ","interquart","ecarmedian","Normale SW","Normale KS","Norm Tr KS","  MinData ","  MaxData ","Lognorm SW","Lognorm KS","LogN Tr KS","  Log(moy)","  Log(E-t)","Uniform KS","  Min     ","  Max     ","Triangl KS","  Mode    ","Exponen KS","  Taux    ","Gamma   KS","  Forme   ","  Echelle ","GEV     KS","  forme   ","  echelle ","  location","50% sig   ","50% sigmax","75% sig   ","75% sigmax","90% sig   ","90% sigmax","95% sig   ","95% sigmax","99% sig   ","99% sigmax","1% sig    ","1% sigmax ","5% sig    ","5% sigmax ","10% sig   ","10% sigmax","20% sig   ","20% sigmax","50% sig   ","50% sigmax","75% sig   ","75% sigmax","90% sig   ","90% sigmax","95% sig   ","95% sigmax","99% sig   ","99% sigmax","1% sig    ","1% sigmax ","5% sig    ","5% sigmax ","10% sig   ","10% sigmax","20% sig   ","20% sigmax")
nstats=length(nomstats)

# *********************************************************************

# Transformation de x en matrice
if (is.matrix(x)==FALSE & is.vector(x)==TRUE) x<-matrix(x,ncol=1)

# Cacul des dimensions de x
n<-dim(x)[1]
np<-dim(x)[2]

# Initialisations
if(is.na(nom[1])==TRUE) stats<-matrix(nrow=nstats,ncol=np,dimnames=list(nomstats,dimnames(x)[[2]]))
else stats<-matrix(nrow=nstats,ncol=np,dimnames=list(nomstats,c(nom)))

#Calcul de l'ecart-type max par bootstrap
sigbmax<-rep(0,np)
xb<-matrix(0,nr=n,nc=np)
for (k in 1:Nboot){
  indice<-rep(0,n)
  for(j in 1:n){
    alea<-n*(runif(1))
    indice[j]<-round(0.5+alea)} # Indices des echantillons bootstrap
  xb<-as.matrix(x[indice,]) # Echantillon bootstrap
  for(i in 1:np){ 
    if (length(xb[!is.na(xb[,i]),i]) > 1){
      if (sd(xb[,i],na.rm=T) > sigbmax[i]) sigbmax[i]<-sd(xb[,i],na.rm=T)}}}

#Boucle
for(i in 1:np){
  j<-1

  moy=mean(x[,i],na.rm=T)
  sig=sd(x[,i],na.rm=T)
  mini=min(x[,i],na.rm=T)
  maxi=max(x[,i],na.rm=T)

#Transformation de x pour lui enlever ses valeurs "NA"
#(l'option "is.na" ne marche pas pour les fcts shapiro.test et ks.test)
  xx<-x[!is.na(x[,i]),i]
  nxx=length(xx) # longueur de l'echantillon sans les valeurs NA

# test pour voir s'il y a des valeurs negatives
# dans ce cas, pas de lois lognormale, exp, Gamma
  LOG<-TRUE
  for (k in 1:nxx){
    if (xx[k] <=0 ){
      LOG<-FALSE
      break}}

#Nombre de donnees
  stats[j,i]=floor(nxx)
  j<-j+1

#Calcul de la moyenne
  stats[j,i]=moy
  j<-j+1

#Calcul de l'ecart-type
  stats[j,i]=sig
  j<-j+1

#Calcul du coefficient de variation
  stats[j,i]=sig/moy
  j<-j+1

#Calcul du skewness
  stats[j,i]=skewness(x[,i],na.rm=T)
  j<-j+1

#Calcul du kurtosis (centre)
  stats[j,i]=kurtosis(x[,i],na.rm=T)
  j<-j+1

#Calcul du minimum
  stats[j,i]=mini
  j<-j+1

#Calcul du 1er quartile
  stats[j,i]=quantile(x[,i],probs=0.25,na.rm=T)
  j<-j+1

#Calcul de la mediane
  stats[j,i]=median(x[,i],na.rm=T)
  j<-j+1

#Calcul du 3eme quartile
  stats[j,i]=quantile(x[,i],probs=0.75,na.rm=T)
  j<-j+1

#Calcul du maximum
  stats[j,i]=maxi
  j<-j+1

#Calcul de l'etendue (max-min)
  stats[j,i]=max(x[,i],na.rm=T)-min(x[,i],na.rm=T)
  j<-j+1

#Calcul de l'intervalle interquartile
  stats[j,i]=quantile(x[,i],probs=0.75,na.rm=T)-quantile(x[,i],probs=0.25,na.rm=T)
  j<-j+1

#Calcul de l'ecart-median
  stats[j,i]=mad(x[,i],na.rm=T)
  j<-j+1

# **************************************************************
#TESTS STATISTIQUES
#Les parametres des lois sont estimes par la methode des moments
#Le test de Shapiro est possible pour des echantillons < 5000

#Loi normale
  if (nxx<5000) stats[j,i]<-as.double(shapiro.test(xx)$p.value)
  else stats[j,i]=NA
  j<-j+1
  stats[j,i]<-as.double(ks.test(xx, "pnorm",mean=moy, sd=sig)$p.value,na.rm=T)
  j<-j+1

#Loi normale tronquee
  stats[j,i]<-as.double(ks.test(xx, "ptnorm",mean=moy,sd=sig,min=mini,max=maxi)$p.value,na.rm=T)
  j<-j+1
  stats[j,i]<-mini
  j<-j+1
  stats[j,i]<-maxi
  j<-j+1

#Loi lognormale
  if (LOG==TRUE & nxx<5000) stats[j,i]<-as.double(shapiro.test(log(xx))$p.value)
  else stats[j,i]=NA
  j<-j+1
  if (LOG==TRUE) stats[j,i]<-as.double(ks.test(xx, "plnorm", meanlog=mean(log(xx)), sdlog=sd(log(xx)))$p.value)
  else stats[j,i]=NA
  j<-j+1
 
#Loi lognormale tronquee
  if (LOG==TRUE){
    stats[j,i]<-as.double(ks.test(xx, "ptlnorm",meanlog=mean(log(xx)),sdlog=sd(log(xx)),min=mini,max=maxi)$p.value,na.rm=T)
    j <- j+1
    stats[j,i]<-mean(log(xx))
    j <- j+1
    stats[j,i]<-sd(log(xx))}
  else{
    stats[j,i]=NA
    j<-j+1
    stats[j,i]=NA
    j<-j+1
    stats[j,i]=NA}
  j<-j+1

#Loi uniforme
#avec min et max
#  stats[j,i]<-as.double(ks.test(xx, "punif", min=min(xx), max=max(xx))$p.value)
#  j<-j+1
#avec les moments
  stats[j,i]<-as.double(ks.test(xx, "punif", min=moy-sig*sqrt(3), max=moy+sig*sqrt(3))$p.value)
  j<-j+1
  stats[j,i]<-moy-sig*sqrt(3)
  j<-j+1
  stats[j,i]<-moy+sig*sqrt(3)
  j<-j+1

#Loi triangulaire
  mode <- 3*moy-(mini+maxi)
  if (mode>=mini & mode<=maxi){
    stats[j,i]<-as.double(ks.test(xx, "ptriangle", a=mini,b=maxi,c=mode)$p.value)
    j <- j+1
    stats[j,i]<-mode}
  else{
    stats[j,i] <- NA
    j<-j+1
    stats[j,i] <- NA}
  j<-j+1

#Loi exponentielle
  if (LOG==T){
    stats[j,i]<-as.double(ks.test(xx, "pexp", rate=1/moy)$p.value)
    j<-j+1
    stats[j,i]<-1/moy}
  else{
    stats[j,i] <- NA
    j<-j+1
    stats[j,i] <- NA}
  j<-j+1

#Loi Gamma
  if (LOG==T){
    stats[j,i]<-as.double(ks.test(xx, "pgamma", shape=moy^2/sig^2, scale=sig^2/moy)$p.value)
    j<-j+1
    stats[j,i]<-moy^2/sig^2
    j<-j+1
    stats[j,i]<-sig^2/moy}
  else{
    stats[j,i] <- NA
    j<-j+1
    stats[j,i] <- NA
    j<-j+1
    stats[j,i] <- NA}
  j<-j+1

#Loi Beta I
#  stats[j,i]<-as.double(ks.test(xx, "pbeta", shape1=moy^2/sig^2*(1-moy), shape2=moy/sig^2*(1-moy)^2)$p.value)
#  j<-j+1
#  stats[j,i]<-moy^2/sig^2*(1-moy)
#  j<-j+1
#  stats[j,i]<-moy/sig^2*(1-moy)^2
#  j<-j+1

#Loi des valeurs extremes generalisee (Generalized extreme value)
#  pargev=gevFit(xx,)$par.est # old version avec fExtremes
#  stats[j,i]<-as.double(ks.test(xx, "pgev", xi=pargev[1], mu=pargev[3], sigma=pargev[2])$p.value)

  if (mean(xx) < 1e-6){
     print(paste("WARNING Variable",i,": La loi gev ne peut etre ajustee pour des valeurs trop faibles => normaliser les donnees"))
     pargev=c(0,1,0)} # valeurs par defaut
  else pargev <- fgev(xx,std.err=F)$param # new version avec evd
  stats[j,i]<-as.double(ks.test(xx, "pgev", loc=pargev[1], scale=pargev[2], shape=pargev[3])$p.value)
  j<-j+1
  stats[j,i] <- pargev[3]
  j<-j+1
  stats[j,i] <- pargev[2]
  j<-j+1
  stats[j,i] <- pargev[1]
  j<-j+1

# **************************************************************
#INEGALITE de BIENAYME-TCHEBYTCHEV

#Precision relative de l'estimateur de la moyenne
# a 50% et avec l'ecart-type
  stats[j,i]<-sig/(abs(moy)*sqrt(nxx*0.5))*100
  j<-j+1
# a 50% et avec l'ecart-type max par bootstrap
  stats[j,i]<-sigbmax[i]/(abs(moy)*sqrt(nxx*0.5))*100
  j<-j+1
# a 75% avec l'ecart-type
  stats[j,i]<-sig/(abs(moy)*sqrt(nxx*0.25))*100
  j<-j+1
# a 75% avec l'ecart-type max par bootstrap
  stats[j,i]<-sigbmax[i]/(abs(moy)*sqrt(nxx*0.25))*100
  j<-j+1
# a 90% avec l'ecart-type
  stats[j,i]<-sig/(abs(moy)*sqrt(nxx*0.1))*100
  j<-j+1
# a 90% avec l'ecart-type max par bootstrap
  stats[j,i]<-sigbmax[i]/(abs(moy)*sqrt(nxx*0.1))*100
  j<-j+1
# a 95% avec l'ecart-type
  stats[j,i]<-sig/(abs(moy)*sqrt(nxx*0.05))*100
  j<-j+1
# a 95% avec l'ecart-type max par bootstrap
  stats[j,i]<-sigbmax[i]/(abs(moy)*sqrt(nxx*0.05))*100
  j<-j+1
# a 99% avec l'ecart-type
  stats[j,i]<-sig/(abs(moy)*sqrt(nxx*0.01))*100
  j<-j+1
# a 99% avec l'ecart-type max par bootstrap
  stats[j,i]<-sigbmax[i]/(abs(moy)*sqrt(nxx*0.01))*100
  j<-j+1

# Taille de l'echantillon en fct de la precision relative (niveau de confiance a 95%)
# 1% d'erreur et avec l'ecart-type
  stats[j,i]<-ceiling(sig^2/(0.0001*moy^2*0.05))
  j<-j+1
# 1% d'erreur et avec l'ecart-type max par bootstrap
  stats[j,i]<-ceiling(sigbmax[i]^2/(0.0001*moy^2*0.05))
  j<-j+1
# 5% d'erreur et avec l'ecart-type
  stats[j,i]<-ceiling(sig^2/(0.0025*moy^2*0.05))
  j<-j+1
# 5% d'erreur et avec l'ecart-type max par bootstrap
  stats[j,i]<-ceiling(sigbmax[i]^2/(0.0025*moy^2*0.05))
  j<-j+1
# 10% d'erreur et avec l'ecart-type
  stats[j,i]<-ceiling(sig^2/(0.01*moy^2*0.05))
  j<-j+1
# 10% d'erreur et avec l'ecart-type max par bootstrap
  stats[j,i]<-ceiling(sigbmax[i]^2/(0.01*moy^2*0.05))
  j<-j+1
# 20% d'erreur et avec l'ecart-type
  stats[j,i]<-ceiling(sig^2/(0.04*moy^2*0.05))
  j<-j+1
# 20% d'erreur et avec l'ecart-type max par bootstrap
  stats[j,i]<-ceiling(sigbmax[i]^2/(0.04*moy^2*0.05))
  j<-j+1

# **************************************************************
#INEGALITE de VYSOCANSKII-PETUNIN

#Precision relative de l'estimateur de la moyenne
# a 50% et avec l'ecart-type
  stats[j,i]<-2/3*sig/(abs(moy)*sqrt(nxx*0.5))*100
  j<-j+1
# a 50% et avec l'ecart-type max par bootstrap
  stats[j,i]<-2/3*sigbmax[i]/(abs(moy)*sqrt(nxx*0.5))*100
  j<-j+1
# a 75% avec l'ecart-type
  stats[j,i]<-2/3*sig/(abs(moy)*sqrt(nxx*0.25))*100
  j<-j+1
# a 75% avec l'ecart-type max par bootstrap
  stats[j,i]<-2/3*sigbmax[i]/(abs(moy)*sqrt(nxx*0.25))*100
  j<-j+1
# a 90% avec l'ecart-type
  stats[j,i]<-2/3*sig/(abs(moy)*sqrt(nxx*0.1))*100
  j<-j+1
# a 90% avec l'ecart-type max par bootstrap
  stats[j,i]<-2/3*sigbmax[i]/(abs(moy)*sqrt(nxx*0.1))*100
  j<-j+1
# a 95% avec l'ecart-type
  stats[j,i]<-2/3*sig/(abs(moy)*sqrt(nxx*0.05))*100
  j<-j+1
# a 95% avec l'ecart-type max par bootstrap
  stats[j,i]<-2/3*sigbmax[i]/(abs(moy)*sqrt(nxx*0.05))*100
  j<-j+1
# a 99% avec l'ecart-type
  stats[j,i]<-2/3*sig/(abs(moy)*sqrt(nxx*0.01))*100
  j<-j+1
# a 99% avec l'ecart-type max par bootstrap
  stats[j,i]<-2/3*sigbmax[i]/(abs(moy)*sqrt(nxx*0.01))*100
  j<-j+1

# Taille de l'echantillon en fct de la precision relative (niveau de confiance a 95%)
# 1% d'erreur et avec l'ecart-type
  stats[j,i]<-ceiling(4/9*sig^2/(0.0001*moy^2*0.05))
  j<-j+1
# 1% d'erreur et avec l'ecart-type max par bootstrap
  stats[j,i]<-ceiling(4/9*sigbmax[i]^2/(0.0001*moy^2*0.05))
  j<-j+1
# 5% d'erreur et avec l'ecart-type
  stats[j,i]<-ceiling(4/9*sig^2/(0.0025*moy^2*0.05))
  j<-j+1
# 5% d'erreur et avec l'ecart-type max par bootstrap
  stats[j,i]<-ceiling(4/9*sigbmax[i]^2/(0.0025*moy^2*0.05))
  j<-j+1
# 10% d'erreur et avec l'ecart-type
  stats[j,i]<-ceiling(4/9*sig^2/(0.01*moy^2*0.05))
  j<-j+1
# 10% d'erreur et avec l'ecart-type max par bootstrap
  stats[j,i]<-ceiling(4/9*sigbmax[i]^2/(0.01*moy^2*0.05))
  j<-j+1
# 20% d'erreur et avec l'ecart-type
  stats[j,i]<-ceiling(4/9*sig^2/(0.04*moy^2*0.05))
  j<-j+1
# 20% d'erreur et avec l'ecart-type max par bootstrap
  stats[j,i]<-ceiling(4/9*sigbmax[i]^2/(0.04*moy^2*0.05))
  j<-j+1

# **************************************************************
#GRAPHIQUES QQPLOT

  x11()
  par(mfcol=c(3,4))

#Boxplot
  par(mfg=c(1,1))
  boxplot(xx,main=c("Boxplot",dimnames(stats)[[2]][i]))

#Histogramme
  par(mfg=c(1,2))
  hist(xx,breaks="Sturges",freq=F,probability=T,xlab="",main=c("Histogram & density",dimnames(stats)[[2]][i]))
  lines(density(xx)) # Densite lissee

#Loi uniforme (methode des moments)
  par(mfg=c(1,3))
  plot(qunif(seq(1/nxx,1,1/nxx),min=moy-sig*sqrt(3),max=moy+sig*sqrt(3)),sort(xx),main=c("Uniform Q-Q Plot",dimnames(stats)[[2]][i]),xlab="Theoretical quantiles",ylab="Sample quantiles")
#  qqmath(~ xx,distribution=qunif,main=c("Uniform Q-Q Plot",dimnames(stats)[[2]][i]))
  abline(0,1,col=2)

# Loi triangulaire
  if (mode>=mini & mode<=maxi){
    par(mfg=c(1,4))
    plot(qtriangle(seq(1/nxx,1,1/nxx),a=mini,b=maxi,c=mode),sort(xx),main=c("Triangle Q-Q Plot",dimnames(stats)[[2]][i]),xlab="Theoretical quantiles",ylab="Sample quantiles")
    abline(0,1,col=2)}

# Loi normale
  par(mfg=c(2,1))
  qqnorm(xx,main=c("Normal Q-Q Plot",dimnames(stats)[[2]][i]))
  qqline(xx,col=2)
  abline(v=0)

# Loi normale tronquee
  par(mfg=c(2,2))
  plot(qtnorm(seq(1/nxx,1,1/nxx),mean=moy,sd=sig,min=mini,max=maxi),sort(xx),main=c("Normal Tr Q-Q Plot",dimnames(stats)[[2]][i]),xlab="Theoretical quantiles",ylab="Sample quantiles")
  abline(0,1,col=2)

#Loi log-normale
  if (LOG==TRUE){
    par(mfg=c(2,3))
    qqnorm(log(xx),main=c("Lognormal Q-Q Plot",dimnames(stats)[[2]][i]))
    qqline(log(xx),col=2)
    abline(v=0)}

# Loi lognormale tronquee
  if (LOG==TRUE){
    par(mfg=c(2,4))
    plot(qtlnorm(seq(1/nxx,1,1/nxx),meanlog=mean(log(xx)),sdlog=sd(log(xx)),min=mini,max=maxi),sort(xx),main=c("Lognormal Tr Q-Q Plot",dimnames(stats)[[2]][i]),xlab="Theoretical quantiles",ylab="Sample quantiles")
    abline(0,1,col=2)}
  
# Loi exponentielle
  if (LOG==T){
    par(mfg=c(3,1))
    plot(qexp(seq(1/nxx,1,1/nxx),rate=1/moy),sort(xx),main=c("Exponential Q-Q Plot",dimnames(stats)[[2]][i]),xlab="Theoretical quantiles",ylab="Sample quantiles")
    abline(0,1,col=2)}

# Loi Gamma
  if (LOG==T){
    par(mfg=c(3,2))
    plot(qgamma(seq(1/nxx,1,1/nxx),shape=moy^2/sig^2,scale=sig^2/moy),sort(xx),main=c("Gamma Q-Q Plot",dimnames(stats)[[2]][i]),xlab="Theoretical quantiles",ylab="Sample quantiles")
    abline(0,1,col=2)}

# Loi GEV
  par(mfg=c(3,3))
#  plot(qgev(seq(1/nxx,1,1/nxx),xi=pargev[1], mu=pargev[3], sigma=pargev[2]),sort(xx),main=c("GEV Q-Q Plot",dimnames(stats)[[2]][i]),xlab="Theoretical quantiles",ylab="Sample quantiles") # old version
  plot(qgev(seq(1/nxx-1e-6,1-1e-6,1/nxx),loc=pargev[1], scale=pargev[2], shape=pargev[3]),sort(xx),main=c("GEV Q-Q Plot",dimnames(stats)[[2]][i]),xlab="Theoretical quantiles",ylab="Sample quantiles")
  abline(0,1,col=2)

}

# *********************************************************************
# AFFICHAGE
# *********************************************************************

#Construction des data frame
result1<-data.frame(format(stats[1:14,],digits=ndigits))
result2<-data.frame(format(stats[15:38,],digits=ndigits-2))
result3<-data.frame(format(stats[39:48,],digits=ndigits-1))
result4<-data.frame(format(stats[49:56,],digits=ndigits-2))
result5<-data.frame(format(stats[57:66,],digits=ndigits-1))
result6<-data.frame(format(stats[67:nstats,],digits=ndigits-2))

#Creation du fichier de resultats
FILE<-"res-STAT.out"

FOUT <- file(FILE,"w")

write(c(" Nombre de variables =",np),file=FOUT,ncolumns=4)
write(" ",file=FOUT)
write("STATISTIQUES ELEMENTAIRES",file=FOUT)
write(" ",file=FOUT)

write.table(array(c("\t",dimnames(stats)[[2]]),dim=c(1,np+1)),file=FOUT,sep="\t",quote=FALSE,row.names=FALSE,col.names=FALSE)
#write(" ",file=FOUT)

write.table(result1,file=FOUT,quote=FALSE,sep="  ",col.names=FALSE)
write(" ",file=FOUT)

#******************************************************
write("TESTS STATISTIQUES d'adequation aux lois (valeurs des p values)",file=FOUT)
write(" Estimation des parametres par la methode des moments (sauf loi GEV)",file=FOUT)
write(" Pour lois normale et norm. tronquee : parametres des stats elementaires",file=FOUT)
write(" SW=test de Shapiro-Wilks, meilleur test pour taille faible (<50 donnees)",file=FOUT)
write(" KS=test de Kolmogorov-Smirnov, pas robuste pour taille faible",file=FOUT)
write(" 1-p value = niveau de confiance avec lequel on peut rejeter l'hypothese",file=FOUT)
write(" Ex: si p value>0.1, on ne peut pas rejeter l'adequation aux niveaux>=90%",file=FOUT)
write(" ",file=FOUT)

write.table(array(c("\t",dimnames(stats)[[2]]),dim=c(1,np+1)),file=FOUT,sep="\t",quote=FALSE,row.names=FALSE,col.names=FALSE)
write.table(result2,file=FOUT,quote=FALSE,sep="  ",col.names=FALSE)
write(" ",file=FOUT)

#******************************************************

write("INEGALITE de BIENAYME-TCHEBYTCHEV (BT)",file=FOUT)
write(" ",file=FOUT)

write(" Determination de la precision (en %) de l'estimateur de la moyenne",file=FOUT)
write(" en fonction du niveau de confiance desiree, et en utilisant",file=FOUT)
write(" soit l'estimateur de l'ecart-type (sig), soit sa borne max estimee par boostrap (sigmax)",file=FOUT)
write(" ",file=FOUT)
write.table(array(c("\t",dimnames(stats)[[2]]),dim=c(1,np+1)),file=FOUT,sep="\t",quote=FALSE,row.names=FALSE,col.names=FALSE)
write.table(result3,file=FOUT,quote=FALSE,sep="  ",col.names=FALSE)
write(" ",file=FOUT)

write(" Determination de la taille necessaire de l'echantillon",file=FOUT)
write(" en fonction de la precision desiree pour l'estimateur de la moyenne",file=FOUT)
write(" pour un niveau de confiance a 95 %, et en utilisant",file=FOUT)
write(" soit l'estimateur de l'ecart-type (sig), soit sa borne max estimee par boostrap (sigmax)",file=FOUT)
write(" ",file=FOUT)
write.table(array(c("\t",dimnames(stats)[[2]]),dim=c(1,np+1)),file=FOUT,sep="\t",quote=FALSE,row.names=FALSE,col.names=FALSE)
write.table(result4,file=FOUT,quote=FALSE,sep="  ",col.names=FALSE)
write(" ",file=FOUT)


write("INEGALITE de VYSOCANSKII-PETUNIN (VP)",file=FOUT)
write(" ",file=FOUT)

write(" Determination de la precision (en %) de l'estimateur de la moyenne",file=FOUT)
write(" en fonction du niveau de confiance desiree, et en utilisant",file=FOUT)
write(" soit l'estimateur de l'ecart-type (sig), soit sa borne max estimee par boostrap (sigmax)",file=FOUT)
write(" ",file=FOUT)
write.table(array(c("\t",dimnames(stats)[[2]]),dim=c(1,np+1)),file=FOUT,sep="\t",quote=FALSE,row.names=FALSE,col.names=FALSE)
write.table(result5,file=FOUT,quote=FALSE,sep="  ",col.names=FALSE)
write(" ",file=FOUT)

write(" Determination de la taille necessaire de l'echantillon",file=FOUT)
write(" en fonction de la precision desiree pour l'estimateur de la moyenne",file=FOUT)
write(" pour un niveau de confiance a 95 %, et en utilisant",file=FOUT)
write(" soit l'estimateur de l'ecart-type (sig), soit sa borne max estimee par boostrap (sigmax)",file=FOUT)
write(" ",file=FOUT)
write.table(array(c("\t",dimnames(stats)[[2]]),dim=c(1,np+1)),file=FOUT,sep="\t",quote=FALSE,row.names=FALSE,col.names=FALSE)
write.table(result6,file=FOUT,quote=FALSE,sep="  ",col.names=FALSE)
write(" ",file=FOUT)

#******************************************************
#Fermeture du fichier de resultats
close(FOUT)

# Affichage du fichier de resultat
if (Sys.getenv("OS")!="Windows_NT") pager_OS="cat"
else pager_OS=""

file.show(FILE,pager=pager_OS)

}
