#SMURFER - Version 1.0 - 10/01/2010

# ******************************************************************
#               INITIALISATION du CHEMIN d'INSTALLATION de SMURFER
# ******************************************************************

# Sous Unix/Linux
#.PATH_SMURFER <- "/home/iooss/src"
#.PATH_SMURFER <- "/home/bi165542/home-local/src"
#.PATH_SMURFER <- "/usbkey"
#.PATH_SMURFER <- "/media/DISK_IMG"

# Sous Windows
#.PATH_SMURFER <- "~/Mes Documents/Programmes"
#.PATH_SMURFER <- "//Lurs/_Comptes/BI165542/personnel/documents/Mes documents Windows/Programmes"
#.PATH_SMURFER <- "E:/SMURFER"
.PATH_SMURFER <- "K:/SMURFER"

# ******************************************************************
#                           NETTOYAGE
# ******************************************************************

# Si on veut on effacer tous les modeles et les objets en memoire
# (sauf ceux commencant par un ".")
#rm(list=ls(all.names=FALSE)) 

# ***************************
# Redefinition indispensable
#T <- TRUE
#F <- FALSE

# ******************************************************************
#                               LIBRAIRIES
# ******************************************************************

library(boot)
library(e1071)
library(evd)
library(gbm)
library(pls)
library(randomForest)
library(nnet)
library(mgcv)
library(triangle)
library(sensitivity)
library(lhs)
library(mlegp)
library(car)
library(JointModeling)
library(DiceKriging)

# ******************************************************************
#                               SOURCES SMURFER
# ******************************************************************

# On charge toutes les fonctions R. 
# Cela remplace pour toutes les fonctions: 
#	source(paste(.PATH_SMURFER,"/SMURFER/current/MSE.R",sep=""))

pathsmurfer <- paste(.PATH_SMURFER,"/SMURFER/current",sep="")
for (nm in list.files(path=pathsmurfer, pattern = "\\.[RrSsQq]$")) {
    cat(nm,":")           
    source(file.path(pathsmurfer, nm))
    cat("\n")}

# exemple pour charger un tableau
#source(paste(.PATH_SMURFER,"/SMURFER/current/ex-tableaux/tableau53.R",sep=""))
# exemple pour charger une fonction
#source(paste(.PATH_SMURFER,"/SMURFER/current/ex-fonctions/fonctions.R",sep=""))

# ******************************************************************
#                     LECTURE FICHIER de DONNEES
# ******************************************************************

# EDE
donnees4<-read.table(paste(.PATH_SMURFER,"/SMURFER/VALIDATION/VALIDATION_v0.5/exemple_EDE_total",sep=""),dec=",")
donnees4<-as.matrix(donnees4)
x1<-donnees4[,1:8]
x2<-factor(donnees4[,9],levels=c(0,3))
xd4<-matrix(c(x1,x2),nr=193,nc=9,dimnames=list(1:193,c("V1","V2","V3","V4","V5","V6","V7","V8","V9")))
yd4<-donnees4[,10]

# Dosimetres
donnees5<-read.table(paste(.PATH_SMURFER,"/SMURFER/VALIDATION/VALIDATION_v0.5/donnees_cuivre_calcul3",sep=""),dec=",")
donnees5<-as.matrix(donnees5)
xd5<-donnees5[,1:10]
yd5<-donnees5[,11:14]

# GASCON
gascon<-read.table(paste(.PATH_SMURFER,"/SMURFER/VALIDATION/VALIDATION_v0.5/gascon.dat",sep=""))
gascon<-as.matrix(gascon)
xg<-gascon[,1:48]
yg<-gascon[,49]
xgm<-xg
for(i in 1:48){
xgm[,i]<-xg[,i]-mean(xg[,i])
}
#xgm<-matrix(c(xgm),nr=874,nc=48,dimnames=list(1:874,c("V1","V2","V3","V4","V5","V6","V7","V8","V9","V10","V11","V12","V13","V14","V15","V16","V17","V18","V19","V20","V21","V22","V23","V24","V25","V26","V27","V28","V29","V30","V31","V32","V33","V34","V35","V36","V37","V38","V39","V40","V41","V42","V43","V44","V45","V46","V47","V48")))
ygm<-c(yg-mean(yg))

# CHAMP GAUSSIEN
champ<-read.table(paste(.PATH_SMURFER,"/SMURFER/VALIDATION/VALIDATION_v0.5/champ.dat",sep=""))
champ<-as.matrix(champ)
xc<-champ[,1:2]
yc<-champ[,3]
champtest<-read.table(paste(.PATH_SMURFER,"/SMURFER/VALIDATION/VALIDATION_v0.5/champ_test.dat",sep=""))
champtest<-as.matrix(champtest)
xctest<-champtest[,1:2]
yctest<-champtest[,3]
# Pour centrer et rduire les entres/sorties
xcm1<-c(xc[,1]-mean(xc[,1]))
xcm2<-c(xc[,2]-mean(xc[,2]))
xcm<-matrix(c(xcm1,xcm2),nr=1600,nc=2)
ycm<-c(yc-mean(yc))

# BCI
# Si la 1ere ligne du fichier contient les noms des variables
matBCI<-read.table(paste(.PATH_SMURFER,"/SMURFER/VALIDATION/VALIDATION_v1.1/bci.dat",sep=""),header=TRUE)
xBCI<-matBCI[,2:length(matBCI)]

# Marthe
param<-read.table(paste(.PATH_SMURFER,"/SMURFER/VALIDATION/VALIDATION_v1.1/elena_marthe.inout",sep=""),header=TRUE)
param<-as.matrix(param)
xmar<-param[,1:20]
ymar<-param[,21:40]

# Meteor
meteor<-read.table(paste(.PATH_SMURFER,"/SMURFER/VALIDATION/VALIDATION_v1.3/meteor_lhs.inout",sep=""),header=TRUE)
meteor<-as.matrix(meteor)
xmet<-meteor[,1:5]
ymet<-meteor[,6:9]

# ****************************************************************************
#                       EXECUTIONS FONCTIONS STATS
# ****************************************************************************

formuleWilks(0.9,0.9,F,1)

#statsconverg(xBCI[,2],nrepet=3,nboot=500,fboot=10,moments=T)
#bornemaxIC(xBCI[,2],kmax=33,nech=100,nboot=500)

#statsconverg(ymar[,1],nrepet=1,nboot=100,fboot=10,kboot=10,moments=T)
#quantileconverg(ymar[,2],nrepet=1,nboot=100,fboot=10,moments=T)
#moyenne(ymar)
#correlationS(xmar)
#statS(ymar[,1:3])
#sensibilite(xmar,ymar,affiche=F,nseg=5,nsegy=5)

#sensibilite(xmet,ymet)

#moyenne(yc)
#correlationS(xc)
#statS(xc)
#sensibilite(xc,yc)

#statS(xd5)
#sensibilite(xd5,yd5,nseg=5)

#x <- samplingLHS(dim_x=3,N=1000,lois=c(0,2,1),paramlois=array(c(0,1,0,0,0,1,0,0,0,1,0,0),dim=c(4,3)),correl=2)

# Fcts d'echantillonnage 

#x=samplingSimple(dim_x=2,N=100,lois=c(1,2),paramlois=array(c(0,1,0,0,0,1,0,0),dim=c(4,2)),tronq=c(T,T),paramtronq=array(c(-2,3,1,20),dim=c(2,2)))

# exemple : 10 points en dimension 10
#pairs(samplingSimple(dim_x=10,N=10,lois=c(0,2,1,6,7,0,2,1,6,7),paramlois=array(c(0,1,0,0,0,1,0,0,0,1,0,0,-1,3,4,0,-2,5,5.5,10,0,1,0,0,0,1,0,0,0,1,0,0,-1,3,4,0,-2,5,5.5,10),dim=c(4,10))))
#pairs(samplingLHS(dim_x=10,N=10,lois=c(0,2,1,6,7,0,2,1,6,7),paramlois=array(c(0,1,0,0,0,1,0,0,0,1,0,0,-1,3,4,0,-2,5,5.5,10,0,1,0,0,0,1,0,0,0,1,0,0,-1,3,4,0,-2,5,5.5,10),dim=c(4,10))))
#pairs(samplingOptLHS(dim_x=10,N=10,lois=c(0,2,1,6,7,0,2,1,6,7),paramlois=array(c(0,1,0,0,0,1,0,0,0,1,0,0,-1,3,4,0,-2,5,5.5,10,0,1,0,0,0,1,0,0,0,1,0,0,-1,3,4,0,-2,5,5.5,10),dim=c(4,10)),dup=1000))
#pairs(samplingLHScontraint(2,N=10,paramlois=array(c(0,1,0,0,0,2,0,0),dim=c(4,2)),contraint=c(0,1)))

# ******************************************************************
#                       EXECUTIONS PROGRAMME
# ******************************************************************

# on efface les modeles (si on veut)
rm(modeleGLM,modelePLS,modeleSVM,modeleBOOSTING,modeleRF,modeleRN,modeleGAM,modeleGP,modeleGPdice)

# CHAMP GAUSSIEN
#source(paste(.PATH_SMURFER,"/SMURFER/current/ex-tableaux/tableau2.R",sep=""))
#smurfer(xc,yc,methodes=c(F,F,F,F,F,F,T,F,F),xtest=xctest,ytest=yctest,sobol=T,R=10,S=1000,Sij=TRUE,lois=c(0,2),paramlois=array(c(0,1,0,0),dim=c(4,2)),vc=T,N=10,bootstrap=T,B=5,affiche=T,forme="maformule",tabA1=tabA1,tabAgam=tabAgam,tabBgam=tabBgam,degreformule=2,stepwise=TRUE,SVMexplore=FALSE,cmin=51,epsmin=0.01,gmin=5.5,ntree=2000)

# METEOR
#source(paste(.PATH_SMURFER,"/SMURFER/current/ex-tableaux/tableau5.R",sep=""))
#smurfer(xmet,ymet[,1],methodes=c(T,F,F,F,F,F,T,F,F),affiche=T,loo=T,vc=T,bootstrap=T,forme="maformule",stepwise=T,degreformule=2,tabA1=tabA1,tabA2=tabA2,tabA3=tabA3,tabB1=tabB1,tabAgam=tabAgam,tabBgam=tabBgam)
#sobolS(modele=modeleGLM,dim_x=5,methode=1,R=10,S=10000,Sij=T,lois=rep(1,5),paramlois=array(c(2.6,0.05,0,0,4.0965,0.005,0,0,4.745,0.005,0,0,11.3,0.5,0,0,0.005,0.00125,0,0),dim=c(4,5)),LHS=T)
#smurfer(xmet[1:100,],ymet[1:100,1],methodes=c(T,F,F,F,F,F,T,F,F),loo=T,affiche=T,vc=T,N=10,bootstrap=F,forme="creationformule",degreformule=1)
#sensibilite(xmet,ymet,affiche=T,nseg=5,nsegy=5)

# GASCON
#smurfer(xg,yg,logy=T,logx=rep(T,dim(xg)[2]),methodes=c(F,T,F,F,T,F,F,F,F),sobol=F,R=2,S=1000,Sij=F,LHS=T,vc=T,N=3,bootstrap=T,B=5,affiche=F,forme="creationformule",tabA1=tabA1,degreformule=1,stepwise=F,SVMexplore=F,cmin=51,epsmin=0.01,gmin=5.5,n.trees=100,ntree=500,nunits=2,maxitRN=100)
#smurfer(xg,yg,logy=T,logx=rep(T,dim(xg)[2]),methodes=c(T,F,F,F,F,F,F,F,F),forme="creationformule",degreformule=1,stepwise=T,penaltyAIC=10)

# PG
#source(paste(.PATH_SMURFER,"/SMURFER/current/ex-tableaux/tableau5.R",sep=""))
smurfer(xmet[1:100,],ymet[1:100,1],methodes=c(F,F,F,F,F,F,F,F,T),loo=F,affiche=T,vc=F,N=10,bootstrap=F,forme="creationformule",degreformule=1,stepwise=F)
#sobolS(modele=modelePGdice,dim_x=5,methode=9,R=10,S=1000,Sij=F,lois=rep(1,5),paramlois=array(c(2.6,0.05,0,0,4.0965,0.005,0,0,4.745,0.005,0,0,11.3,0.5,0,0,0.005,0.00125,0,0),dim=c(4,5)),LHS=T)
#smurfer(xmar[1:300,],ymar[1:300,1],methodes=c(T,F,F,F,F,F,F,F,T),loo=F,affiche=F,vc=T,N=10,bootstrap=F,forme="creationformule",degreformule=1,min.nugget=1e-6,stepwise=T)

# ******************************************************************
#                          Quelques POSSIBILITES d'OPTIONS
# ******************************************************************

#OPTIONS log  : logy=TRUE,logx=rep(TRUE,dim(xc)[2]),logy=TRUE,logx=c(rep(TRUE,dim(xc)[2]-1),FALSE),

#sobolS(modele=modeleGLM,dim_x=48,S=1000,Sij=F,lois=rep(0,48),paramlois=array(c(0,1,0,0),dim=c(4,48)),logy=T,logx=rep(T,48))

#OPTIONS lois : lois=rep(0,3),paramlois=array(c(0,1,0,0,0,2,0,0,0,3,0,0),dim=c(4,3))
#			      paramlois=array(c(0,1,0,0),dim=c(4,3))
#			      paramlois[,2]=c(0,2,0,0)
#               lois=c(0,2),paramlois=array(c(0,1,0,0,0,1,0,0),dim=c(4,2))

# ******************************************************************
# EXEMPLES d'UTILISATION du PACKAGE SENSITIVITY
# ******************************************************************

#smurfer(xmet,ymet[,1],methodes=c(T,F,F,F,F,F,F,F,F),affiche=F,vc=F,bootstrap=F,forme="creationformule",stepwise=F,degreformule=1)
#sa <- srcpcc(model=modeleGLM,x=as.data.frame(xmet),rank=T,nboot=10)
#print(sa)
#par(mfrow = c(2,1))
#plot(sa, ask = FALSE)

#sa <- morris(model=morris.fun,factors=20,levels=10,R=10)
#print(sa)
#par(mfrow = c(1,1))
#plot(sa)

#sa <- fast(model=NULL, factors = 3, n = 1000,q = "qunif", q.arg = list(min = -pi, max = pi))
#y <- ishigami.fun(sa$x)
#tell(sa,y)
#print(sa)
#plot(sa)

#sa <- fast(model = ishigami.fun, factors = 3, n = 1000, q=c("qunif","qtnorm","qunif"), q.arg = list(list(min = -pi, max = pi), list(mean=0,sd=1,min=-3,max=3),list(min=-pi,max=pi)))
#sa <- fast(model = ishigami.fun, factors = 3, n = 999, q=c("qunif","qnorm","qunif"), q.arg = list(list(min = -pi, max = pi), list(mean=0,sd=1),list(min=-pi,max=pi)))
